:- module(helper, [     convert_list_to_float_array/3,
                        convert_list_to_float_array/2,
                        convert_float_array_to_list/3,
                        convert_float_array_to_2d_list/4,
                        len/2,
                        convert_record_to_arr/2,
                        take_csv_row/3,
                        take_csv_row/4,
                        take_rows_from_iris_CSV/2]).


:- use_module(library(structs)).
:- use_module(library(csv)).


%% TODO: update Comment docs
%% TODO: test the helper predicate

%% Functions for editing float arrays

convert_list_to_float_array(Arr, DataPointDim, array(Size, DataPointDim,Mem)) :-
        len(Arr, Size),
        Size >= DataPointDim,
        DataPointDim > 0,
        mod(Size, DataPointDim) =:= 0,
        new(float_array, Size, Mem),
        fill_float_array(Arr, 0, Mem).

convert_list_to_float_array(Arr, array(Size,Mem)) :-
        len(Arr, Size),
        new(float_array, Size, Mem),
        fill_float_array(Arr, 0, Mem).

fill_float_array([], _, _):- !.
fill_float_array([H|Tail], Index, Mem) :-
        put_contents(Mem, Index, H),
        New_index is Index + 1,
        fill_float_array(Tail, New_index, Mem).

%% takes library(struct) array and fills a [] with the content of the array from 0 to to given Index
convert_float_array_to_list(Mem, Count, Out) :-
        convert_float_array_to_list(Mem, 0, Count, Out).

convert_float_array_to_list(_, Count, Count, []) :- !.

convert_float_array_to_list(Mem, Index, Count, [Val|Rest]) :-
        NewIndex is Index + 1,
        convert_float_array_to_list(Mem, NewIndex, Count, Rest),
        get_contents(Mem, Index , Val).

%% takes library(struct) array and fills a 2D [[a], [b]] with the content of the array from 0 to to given Index
%% The data gets transposed since in most mlpack matrices is every col a data point 
convert_float_array_to_2d_list(Mem, ColNum, RowNum, Out) :-
        convert_float_array_to_2d_list(Mem, RowNum, ColNum, 0, Out).

convert_float_array_to_2d_list(_, _, RowNum, RowNum, []) :- !.
        
convert_float_array_to_2d_list(Mem, ColNum, RowNum, RowCount, [Val | Rest]) :-
        NewRowCount is RowCount + 1,
        StartIndex is RowCount * ColNum,
        EndIndex is (NewRowCount * ColNum),
        convert_float_array_to_list(Mem, StartIndex, EndIndex, Val),
        convert_float_array_to_2d_list(Mem, ColNum, RowNum, NewRowCount, Rest).
        
        
        




%% returns the length of a list
len([], 0):- !.
len([_|Tail], List_L) :-
        len(Tail, Tail_L),
        List_L is Tail_L + 1.

%% Funktions for reading the csv Files
%% take the elements in a csv record and put them in a list
convert_record_to_arr([], []):- !.
convert_record_to_arr([float(Num,_)|Tail], [Num|Rest]) :-
        convert_record_to_arr(Tail, Rest),!.
convert_record_to_arr([string(_)|Tail], Rest) :-
        convert_record_to_arr(Tail, Rest).

take_csv_row(File, skipFirstRow, Count, H) :-
        read_record(File, _),
        take_csv_row(File, Count, H).
take_csv_row(_, 0, []) :- !.
take_csv_row(File, Count, H) :-
        Count > 0,
        read_record(File, Rec),
        convert_record_to_arr(Rec, Out),
        NewCount is Count - 1,
        append(Out, Rest, H),
        take_csv_row(File, NewCount, Rest),
        !.

take_rows_from_iris_CSV(RowAmount, RowList) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,RowAmount, RowList).
