
:- module(approx_kfn_tests, [run_approx_kfn_tests/0]).

:- use_module(library(plunit)).

:- use_module(approx_kfn).
:- use_module('../../helper_files/helper.pl').

reset_DrusillaModel_NoTrain :-
        initDrusillaModelNoTrain(2, 2).

reset_DrusillaModel_WithTrain :-
        initDrusillaModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, 2).

reset_QDAFNModel_NoTrain :-
        initQDAFNModelNoTrain(2, 2).

reset_QDAFNModel_WithTrain :-
        initQDAFNModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, 2).

%%
%% TESTING predicate initDrusillaModelNoTrain/2
%%
:- begin_tests(initDrusillaModelNoTrain).      

%% Failure Tests
                                            
test(approx_KFN_InitDrusillaNoTrain_Negative_L, fail) :-
        initDrusillaModelNoTrain(-5,5).

test(approx_KFN_InitDrusillaNoTrain_Negative_M, fail) :-
        initDrusillaModelNoTrain(5,-5).
        

%% Successful Tests

test(approx_KFN_InitDrusillaNoTrain_Normal) :-
        initDrusillaModelNoTrain(5,5).

test(approx_KFN_InitDrusillaNoTrain_Alternative) :-
        initDrusillaModelNoTrain(10,3).

:- end_tests(initDrusillaModelNoTrain).



%%
%% TESTING predicate initDrusillaModelWithTrain/4
%%
:- begin_tests(initDrusillaModelWithTrain).      

%% Failure Tests
                                            
test(approx_KFN_InitDrusillaWithTrain_Negative_L, fail) :-
        initDrusillaModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, -5, 5).

test(approx_KFN_InitDrusillaWithTrain_Negative_M, fail) :-
        initDrusillaModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 5, -5).

test(approx_KFN_InitDrusillaWithTrain_Normal,
     [error(_,system_error('DrusillaSelect::Train(): l and m are too large!  Choose smaller values.  l*m must be smaller than the number of points in the dataset.'))]) :-
        initDrusillaModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 5, 5).


%% Successful Tests

test(approx_KFN_InitDrusillaWithTrain_Normal) :-
        initDrusillaModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, 2).

test(approx_KFN_InitDrusillaWithTrain_Alternative) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        initDrusillaModelWithTrain(Data, 4, 5, 2).

:- end_tests(initDrusillaModelWithTrain).



%%
%% TESTING predicate searchDrusilla/7
%%
:- begin_tests(searchDrusilla).      

%% Failure Tests

test(approx_KFN_SearchDrusilla_Negative_K, fail) :-
        reset_DrusillaModel_WithTrain,
        searchDrusilla([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, -2, _, _, _, _).
                                            
test(approx_KFN_SearchDrusilla_Diffrent_Query_Dims, [error(_,system_error('subtraction: incompatible matrix dimensions: 4x1 and 3x1'))]) :-
        reset_DrusillaModel_WithTrain,
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        searchDrusilla(Data, 4, 2, _, _, _, _).

test(approx_KFN_SearchDrusilla_Befor_Train, [error(_,system_error('The Model is not Trained!'))]) :-
        reset_DrusillaModel_NoTrain,
        searchDrusilla([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, _, _, _, _).
        

%% Successful Tests

test(approx_KFN_SearchDrusilla_Normal) :-
        reset_DrusillaModel_WithTrain,
        searchDrusilla([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, NeighborsList, _, DistancesList, _),
        print('\nNeighbors: '),
        print(NeighborsList),
        print('\nDistances: '),
        print(DistancesList).
                                            
test(approx_KFN_SearchDrusilla_Alternative) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        initDrusillaModelWithTrain(Data, 4, 2, 5),
        searchDrusilla(Data, 4, 3, NeighborsList, _, DistancesList, _),
        print('\nNeighbors: '),
        print(NeighborsList),
        print('\nDistances: '),
        print(DistancesList).

:- end_tests(searchDrusilla).



%% TESTING predicate trainDrusilla/4
%%
:- begin_tests(trainDrusilla).      

%% Failure Tests
                                            
test(approx_KFN_TrainDrusilla_Negative_L, fail) :-
        reset_DrusillaModel_NoTrain,
        trainDrusilla([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, -5, 5).

test(approx_KFN_TrainDrusilla_Negative_M, fail) :-
        reset_DrusillaModel_NoTrain,
        trainDrusilla([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 5, -5).

test(approx_KFN_TrainDrusilla_Too_Large_M_And_L,
     [error(_,system_error('DrusillaSelect::Train(): l and m are too large!  Choose smaller values.  l*m must be smaller than the number of points in the dataset.'))]) :-
        reset_DrusillaModel_NoTrain,
        trainDrusilla([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 5, 5).
        

%% Successful Tests

test(approx_KFN_TrainDrusilla_Normal) :-
        reset_DrusillaModel_NoTrain,
        trainDrusilla([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, 1).

test(approx_KFN_TrainDrusilla_Alternative) :-
        reset_DrusillaModel_NoTrain,
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        trainDrusilla(Data, 4, 2, 3).

:- end_tests(trainDrusilla).



%%
%% TESTING predicate initQDAFNModelNoTrain/2
%%
:- begin_tests(initQDAFNModelNoTrain).      

%% Failure Tests
                                            
test(approx_KFN_InitQDAFNNoTrain_Negative_L, fail) :-
        initQDAFNModelNoTrain(-5,5).

test(approx_KFN_InitQDAFNNoTrain_Negative_M, fail) :-
        initQDAFNModelNoTrain(5,-5).
        

%% Successful Tests

test(approx_KFN_InitQDAFNNoTrain_Normal) :-
        initQDAFNModelNoTrain(5,5).

test(approx_KFN_InitQDAFNNoTrain_Alternative) :-
        initQDAFNModelNoTrain(10,3).

:- end_tests(initQDAFNModelNoTrain).



%%
%% TESTING predicate initQDAFNModelWithTrain/4
%%
:- begin_tests(initQDAFNModelWithTrain).      

%% Failure Tests
                                            
test(approx_KFN_InitQDAFNWithTrain_Negative_L, fail) :-
        initQDAFNModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, -5, 5).

test(approx_KFN_InitQDAFNWithTrain_Negative_M, fail) :-
        initQDAFNModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 5, -5).

test(approx_KFN_InitQDAFNWithTrain_Too_Large_L_And_M, [error(_,system_error('l*m must be smaller than the number of points in the dataset.'))]) :-
        initQDAFNModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 5, 5).


%% Successful Tests

test(approx_KFN_InitQDAFNWithTrain_Normal) :-
        initQDAFNModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 1, 2).

test(approx_KFN_InitQDAFNWithTrain_Alternative) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        initQDAFNModelWithTrain(Data, 4, 5, 2).

:- end_tests(initQDAFNModelWithTrain).



%%
%% TESTING predicate searchQDAFN/7
%%
:- begin_tests(searchQDAFN).      

%% Failure Tests

test(approx_KFN_SearchQDAFN_Negative_K, fail) :-
        reset_QDAFNModel_WithTrain,
        searchQDAFN([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, -2, _, _, _, _).
                                            
test(approx_KFN_SearchQDAFN_Diffrent_Query_Dims, [error(_,system_error('dot(): objects must have the same number of elements'))]) :-
        reset_QDAFNModel_WithTrain,
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        searchQDAFN(Data, 4, 2, _, _, _, _).

test(approx_KFN_SearchQDAFN_Befor_Train, [error(_,system_error('The Model is not Trained!'))]) :-
        reset_QDAFNModel_NoTrain,
        searchQDAFN([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, _, _, _, _).

test(approx_KFN_SearchQDAFN_Too_Large_K, [error(_,system_error('QDAFN::Search(): requested k is greater than value of m!'))]) :-
        reset_QDAFNModel_WithTrain,
        searchQDAFN([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 3, _, _, _, _).
        

%% Successful Tests

test(approx_KFN_SearchQDAFN_Normal) :-
        reset_QDAFNModel_WithTrain,
        searchQDAFN([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, NeighborsList, _, DistancesList, _),
        print('\nNeighbors: '),
        print(NeighborsList),
        print('\nDistances: '),
        print(DistancesList).
                                            
test(approx_KFN_SearchQDAFN_Alternative) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        initQDAFNModelWithTrain(Data, 4, 2, 3),
        searchQDAFN(Data, 4, 2, NeighborsList, _, DistancesList, _),
        print('\nNeighbors: '),
        print(NeighborsList),
        print('\nDistances: '),
        print(DistancesList).

:- end_tests(searchQDAFN).



%% TESTING predicate trainQDAFN/4
%%
:- begin_tests(trainQDAFN).      

%% Failure Tests
                                            
test(approx_KFN_TrainQDAFN_Negative_L, fail) :-
        reset_QDAFNModel_NoTrain,
        trainQDAFN([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, -5, 5).

test(approx_KFN_TrainQDAFN_Negative_M, fail) :-
        reset_QDAFNModel_NoTrain,
        trainQDAFN([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 5, -5).

test(approx_KFN_TrainQDAFN_Too_Large_M_And_L,
     [error(_,system_error('l*m must be smaller than the number of points in the dataset.'))]) :-
        reset_QDAFNModel_NoTrain,
        trainQDAFN([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 5, 5).
        

%% Successful Tests

test(approx_KFN_TrainQDAFN_Normal) :-
        reset_QDAFNModel_NoTrain,
        trainQDAFN([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, 1).

test(approx_KFN_TrainQDAFN_Alternative) :-
        reset_QDAFNModel_NoTrain,
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        trainQDAFN(Data, 4, 2, 3).

:- end_tests(trainQDAFN).

run_approx_kfn_tests :-
        run_tests.

