
:- module(bayesian_linear_regression_tests, [run_bayesian_linear_regression_tests/0]).

:- use_module(library(plunit)).

%%:- compile(bayesian_linear_regression).
:- use_module(bayesian_linear_regression).
:- use_module('../../helper_files/helper.pl').


reset_Model :-
        blr_initModel(1,0,50,0.0001).


%%
%% TESTING predicate blr_initModel/4
%%
:- begin_tests(blr_initModel).      

%% Failure Tests
                                            
test(bay_lin_reg_InitModel_Negative_NIterMax, fail) :-
        blr_initModel(0,0,-50,0.0001).

test(bay_lin_reg_InitModel_Negative_Tolerance, fail) :-
        blr_initModel(0,0,50,-0.0001).
        

%% Successful Tests

test(bay_lin_reg_InitModel_Default_Inputs) :-
        blr_initModel(1,0,50,0.0001).

test(bay_lin_reg_InitModel_Alternative_Inputs) :-
        blr_initModel(1,1,0,0.0071).

:- end_tests(blr_initModel).



%%
%% TESTING predicate blr_alpha/1
%%
:- begin_tests(blr_alpha).      

%% Failure Tests

test(bay_lin_reg_Alpha_Wrong_Input, fail) :-
        reset_Model,
        blr_alpha(1).
        

%% Successful Tests

test(bay_lin_reg_Alpha_Std_init, [true(Alpha =:= 0.0)]) :-
        reset_Model,
        blr_alpha(Alpha).

test(bay_lin_reg_Alpha_After_Train, [true(Alpha =:= 0.12986500952614138)]) :-
        reset_Model,
        blr_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1]),
        blr_alpha(Alpha).
        
:- end_tests(blr_alpha).



%%
%% TESTING predicate blr_beta/1
%%
:- begin_tests(blr_beta).      

%% Failure Tests

test(bay_lin_reg_Beta_Wrong_Input, fail) :-
        reset_Model,
        blr_beta(1).
        

%% Successful Tests

test(bay_lin_reg_Beta_Std_init, [true(Beta =:= 0.0)]) :-
        reset_Model,
        blr_beta(Beta).

test(bay_lin_reg_Beta_After_Train, [true(Beta =:= 2.317989668988762E+31)]) :-
        reset_Model,
        blr_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1]),
        blr_beta(Beta).
        
:- end_tests(blr_beta).



%%
%% TESTING predicate blr_dataOffset/1
%%
:- begin_tests(blr_dataOffset).      

%% Failure Tests
                                            
test(bay_lin_reg_DataOffset_Before_Train, [error(_,system_error('The Model is not Trained!'))]) :-
        reset_Model,
        blr_dataOffset(_).
        

%% Successful Tests

test(bay_lin_reg_DataOffset_DirektInput) :-
        reset_Model,
        blr_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1]),
        blr_dataOffset(ResponsesOffsetList),
        print('\nResponsesOffset: '),
        print(ResponsesOffsetList).

test(bay_lin_reg_DataOffset_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        blr_train(Data, 4, [0,1,0,1,1,0,1,1,1,0]),
        blr_dataOffset(ResponsesOffsetList),
        print('\nResponsesOffset: '),
        print(ResponsesOffsetList).
        
:- end_tests(blr_dataOffset).



%%
%% TESTING predicate blr_dataScale/1
%%
:- begin_tests(blr_dataScale).      

%% Failure Tests
                                            
test(bay_lin_reg_DataScale_Before_Train, [error(_,system_error('The Model is not Trained!'))]) :-
        reset_Model,
        blr_dataScale(_).
        

%% Successful Tests

test(bay_lin_reg_DataScale_DirektInput) :-
        reset_Model,
        blr_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1]),
        blr_dataScale(DataOffsetList),
        print('\nDataOffset: '),
        print(DataOffsetList).

test(bay_lin_reg_DataScale_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        blr_train(Data, 4, [0,1,0,1,1,0,1,1,1,0]),
        blr_dataScale(DataOffsetList),
        print('\nDataOffset: '),
        print(DataOffsetList).
        
:- end_tests(blr_dataScale).



%%
%% TESTING predicate blr_omega/1
%%
:- begin_tests(blr_omega).      

%% Failure Tests
                                            
test(bay_lin_reg_Omega_Before_Train, [error(_,system_error('The Model is not Trained!'))]) :-
        reset_Model,
        blr_omega(_).
        

%% Successful Tests

test(bay_lin_reg_Omega_DirektInput) :-
        reset_Model,
        blr_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1]),
        blr_omega(OmegaList),
        print('\nOmega: '),
        print(OmegaList).

test(bay_lin_reg_Omega_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        blr_train(Data, 4, [0,1,0,1,1,0,1,1,1,0]),
        blr_omega(OmegaList),
        print('\nOmega: '),
        print(OmegaList).
        
:- end_tests(blr_omega).



%%
%% TESTING predicate blr_predict/3
%%
:- begin_tests(blr_predict).      

%% Failure Tests
                                            
test(bay_lin_reg_Predict_Before_Train, [error(_,system_error('The Model is not Trained!'))]) :-
        reset_Model,
        blr_predict([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _).

test(bay_lin_reg_Predict_Different_Dims_Than_Trained, [error(_,system_error('each_col(): incompatible size; expected 4x1, got 3x1'))]) :-
        reset_Model,
        blr_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1]),
        blr_predict([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, _).
        

%% Successful Tests

test(bay_lin_reg_Predict_Direct_Input) :-
        reset_Model,
        blr_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1]),
        blr_predict([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, PredictionsList),
        print('\nPredictions: '),
        print(PredictionsList).

test(bay_lin_reg_Predict_CSV_Input) :-
        reset_Model,
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        blr_train(Data, 4, [0,1,0,1,1,0,1,1,1,0]),
        blr_predict([3, 2, 0, 5, 1, 4, 1, 0, 4, 3, 3, 5, 0, 5, 5, 2, 5, 5, 0, 2], 4, PredictionsList),
        print('\nPredictions: '),
        print(PredictionsList).
        
:- end_tests(blr_predict).



%%
%% TESTING predicate blr_predictWithStd/3
%%
:- begin_tests(blr_predictWithStd).      

%% Failure Tests
                                            
test(bay_lin_reg_PredictWithStd_Before_Train, [error(_,system_error('The Model is not Trained!'))]) :-
        reset_Model,
        blr_predictWithStd([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _).

test(bay_lin_reg_PredictWithStd_Different_Dims_Than_Trained, [error(_,system_error('each_col(): incompatible size; expected 4x1, got 3x1'))]) :-
        reset_Model,
        blr_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1]),
        blr_predictWithStd([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, _, _).
        

%% Successful Tests

test(bay_lin_reg_PredictWithStd_Direct_Input) :-
        reset_Model,
        blr_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1]),
        blr_predictWithStd([3, 2, 0, 5, 1, 4, 0, 0, 4, 3, 3, 5, 0, 5, 5], 3, PredictionsList, STDList),
        print('\nPredictions: '),
        print(PredictionsList),
        print('\nSTD: '),
        print(STDList).

test(bay_lin_reg_PredictWithStd_CSV_Input) :-
        reset_Model,
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        blr_train(Data, 4, [0,1,0,1,1,0,1,1,1,0]),
        blr_predictWithStd([3, 2, 0, 5, 1, 4, 1, 0, 4, 3, 3, 5, 0, 5, 5, 2, 5, 5, 0, 2], 4, PredictionsList, STDList),
        print('\nPredictions: '),
        print(PredictionsList),
        print('\nSTD: '),
        print(STDList).
        
:- end_tests(blr_predictWithStd).



%%
%% TESTING predicate blr_rmse/4
%%
:- begin_tests(blr_rmse).      

%% Failure Tests
                                            
test(bay_lin_reg_RMSE_Before_Train, [error(_,system_error('The Model is not Trained!'))]) :-
        reset_Model,
        blr_rmse([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,0,0,0], _).

test(bay_lin_reg_RMSE_Too_Small_Label_Dims, [error(_,system_error('subtraction: incompatible matrix dimensions: 1x2 and 1x4'))]) :-
        reset_Model,
        blr_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1]),
        blr_rmse([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], _).

test(bay_lin_reg_RMSE_Too_Large_Label_Dims, [error(_,system_error('subtraction: incompatible matrix dimensions: 1x6 and 1x4'))]) :-
        reset_Model,
        blr_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1]),
        blr_rmse([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,0,1,0,0,0], _).

%% doesnt cause an exception
test(bay_lin_reg_RMSE_Wrong_Label_Value) :-
        reset_Model,
        blr_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1]),
        blr_rmse([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,5,0,-1], _).
        
test(bay_lin_reg_RMSE_Wrong_Data_Dims, [error(_,system_error('each_col(): incompatible size; expected 4x1, got 3x1'))]) :-
        reset_Model,
        blr_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1]),
        blr_rmse([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, [0,1,0,1], _).

test(bay_lin_reg_RMSE_Wrong_Amount_Off_DataPoints, [error(_,system_error('subtraction: incompatible matrix dimensions: 1x10 and 1x5'))]) :-
        reset_Model,
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        blr_train(Data, 4, [0,1,0,1,1,0,1,1,1,0]),
        blr_rmse([3, 2, 0, 5, 1, 4, 1, 0, 4, 3, 3, 5, 0, 5, 5, 2, 5, 5, 0, 2], 4, [0,1,0,1,1,0,1,1,1,0], _).


%% Successful Tests

test(bay_lin_reg_RMSE_Direct_Input) :-
        reset_Model,
        blr_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1]),
        blr_rmse([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], RMSE),
        print('\nRMSE: '),
        print(RMSE).

test(bay_lin_reg_RMSE_CSV_Input) :-
        reset_Model,
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,5, Data),
        blr_train(Data, 4, [0,1,0,1,1]),
        blr_rmse([3, 2, 0, 5, 1, 4, 1, 0, 4, 3, 3, 5, 0, 5, 5, 2, 5, 5, 0, 2], 4, [0,1,0,1,1], RMSE),
        print('\nRMSE: '),
        print(RMSE).
        
:- end_tests(blr_rmse).



%%
%% TESTING predicate blr_train/3
%%
:- begin_tests(blr_train).      

%% Failure Tests

test(bay_lin_reg_Train_Too_Small_Label_Dims, [error(_,system_error('Target dim doesnt fit to the Data dim'))]) :-
        reset_Model,
        blr_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1]).

test(bay_lin_reg_Train_Too_Large_Label_Dims, [error(_,system_error('Target dim doesnt fit to the Data dim'))]) :-
        reset_Model,
        blr_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,0,1,0,0,0]).


%% Successful Tests

test(bay_lin_reg_Train_Direct_Input) :-
        reset_Model,
        blr_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1]).

test(bay_lin_reg_Train_CSV_Input) :-
        reset_Model,
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        blr_train(Data, 4, [0,1,0,1,1,0,1,1,1,0]).
        
:- end_tests(blr_train).



%%
%% TESTING predicate blr_variance/1
%%
:- begin_tests(blr_variance).      

%% Failure Tests

test(bay_lin_reg_Variance_Before_Train, [error(_,system_error('The Model is not Trained!'))]) :-
        reset_Model,
        blr_variance(_).
        

%% Successful Tests

test(bay_lin_reg_Variance_After_Train, [true(Variance =:= 4.3140830754274083E-32)]) :-
        reset_Model,
        blr_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1]),
        blr_variance(Variance).
        
:- end_tests(blr_variance).


run_bayesian_linear_regression_tests :-
        run_tests.

