#include <sicstus/sicstus.h>
/* ex_glue.h is generated by splfr from the foreign/[2,3] facts.
   Always include the glue header in your foreign resource code.
*/
#include "dbscan_glue.h"
#include <mlpack/methods/dbscan/dbscan.hpp>
#include <mlpack/methods/range_search/rs_model.hpp>
#include <mlpack/core.hpp>

// including helper functions for converting between arma structures and arrays
#include "../../helper_files/helper.hpp"

// some of the most used namespaces
using namespace arma;
using namespace mlpack;
using namespace std;
using namespace mlpack::dbscan;
using namespace mlpack::range;
using namespace mlpack::metric;


template<typename RangeSearchType = range::RangeSearch<>, typename PointSelectionPolicy = OrderedPointSelection>
void initAndCluster(double epsilon, SP_integer minPoints, 
					SP_integer batchMode, 
					float *dataMatArr, SP_integer dataMatSize, SP_integer dataMatRowNum,
					float **assignmentsArr, SP_integer *assignmentsArrSize, 
					float **centroidsMatArr, SP_integer *centroidsMatColNum, SP_integer *centroidsMatRowNum)
{
	// convert the Prolog arrays to arma::mat
	mat data = convertArrayToMat(dataMatArr, dataMatSize, dataMatRowNum);
	
	// create the ReturnVector
	Row< size_t > assignmentsReturnVector;
	// create the ReturnMat
	mat centroidsReturnMat;
	

	DBSCAN<RangeSearchType, PointSelectionPolicy>(epsilon, minPoints, (batchMode == 1)).Cluster(data, assignmentsReturnVector, centroidsReturnMat);


	// return the Vector
	returnVectorInformation(assignmentsReturnVector, assignmentsArr, assignmentsArrSize);

	// return the Matrix
	returnMatrixInformation(centroidsReturnMat, centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
}


// input: 	const double 			epsilon,
//			const size_t 			minPoints,
//			const bool 				batchMode = true,
//			RangeSearchType 		rangeSearch = RangeSearchType(),
//			PointSelectionPolicy 	pointSelector = PointSelectionPolicy()
//
//			const MatType & 		data,
//			arma::Row< size_t > & 	assignments,
//			arma::mat & 			centroids
// output: 
// description: 
//			This is a one predicate model where you configure the model with the input parameters and get returned the results in the same predicate.
//
void dbscan(double epsilon, SP_integer minPoints, 
				SP_integer batchMode, 
				char const *selectionType, char const *treeType, 
				float *dataMatArr, SP_integer dataMatSize, SP_integer dataMatRowNum,
				float **assignmentsArr, SP_integer *assignmentsArrSize, 
				float **centroidsMatArr, SP_integer *centroidsMatColNum, SP_integer *centroidsMatRowNum)
{
	if(strcmp(selectionType, "ordered") == 0) 
	{
		if (strcmp(treeType, "kd") == 0)
		{
			initAndCluster<RangeSearch<EuclideanDistance, mat, tree::KDTree>, OrderedPointSelection>(epsilon, minPoints, 
																											batchMode, 
																											dataMatArr, dataMatSize, dataMatRowNum, 
																											assignmentsArr, assignmentsArrSize, 
																											centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
		}
		else if (strcmp(treeType, "r") == 0)
		{
			initAndCluster<RangeSearch<EuclideanDistance, mat, tree::RTree>, OrderedPointSelection>(epsilon, minPoints, 
																											batchMode, 
																											dataMatArr, dataMatSize, dataMatRowNum, 
																											assignmentsArr, assignmentsArrSize, 
																											centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
		}
		else if (strcmp(treeType, "r_star") == 0)
		{
			initAndCluster<RangeSearch<EuclideanDistance, mat, tree::RStarTree>, OrderedPointSelection>(epsilon, minPoints, 
																											batchMode, 
																											dataMatArr, dataMatSize, dataMatRowNum, 
																											assignmentsArr, assignmentsArrSize, 
																											centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
		}
		else if (strcmp(treeType, "x") == 0)
		{
			initAndCluster<RangeSearch<EuclideanDistance, mat, tree::XTree>, OrderedPointSelection>(epsilon, minPoints, 
																											batchMode, 
																											dataMatArr, dataMatSize, dataMatRowNum, 
																											assignmentsArr, assignmentsArrSize, 
																											centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
		}
		else if (strcmp(treeType, "hilbert_r") == 0)
		{
			initAndCluster<RangeSearch<EuclideanDistance, mat, tree::HilbertRTree>, OrderedPointSelection>(epsilon, minPoints, 
																											batchMode, 
																											dataMatArr, dataMatSize, dataMatRowNum, 
																											assignmentsArr, assignmentsArrSize, 
																											centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
		}
		else if (strcmp(treeType, "r_plus") == 0)
		{
			initAndCluster<RangeSearch<EuclideanDistance, mat, tree::RPlusTree>, OrderedPointSelection>(epsilon, minPoints, 
																											batchMode, 
																											dataMatArr, dataMatSize, dataMatRowNum, 
																											assignmentsArr, assignmentsArrSize, 
																											centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
		}
		else if (strcmp(treeType, "r_plus_plus") == 0)
		{
			initAndCluster<RangeSearch<EuclideanDistance, mat, tree::RPlusPlusTree>, OrderedPointSelection>(epsilon, minPoints, 
																											batchMode, 
																											dataMatArr, dataMatSize, dataMatRowNum, 
																											assignmentsArr, assignmentsArrSize, 
																											centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
		}
		else if (strcmp(treeType, "cover") == 0)
		{
			initAndCluster<RangeSearch<EuclideanDistance, mat, tree::CoverTree>, OrderedPointSelection>(epsilon, minPoints, 
																											batchMode, 
																											dataMatArr, dataMatSize, dataMatRowNum, 
																											assignmentsArr, assignmentsArrSize, 
																											centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
		}
		else if (strcmp(treeType, "ball") == 0)
		{
			initAndCluster<RangeSearch<EuclideanDistance, mat, tree::BallTree>, OrderedPointSelection>(epsilon, minPoints, 
																											batchMode, 
																											dataMatArr, dataMatSize, dataMatRowNum, 
																											assignmentsArr, assignmentsArrSize, 
																											centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
		}
		else
		{
			raisePrologDomainExeption(treeType, 5, "The given TreeType is unkown!", "dbscan");
			return;
		}
	}
	else if (strcmp(selectionType, "random") == 0)
	{
		if (strcmp(treeType, "kd") == 0)
		{
			initAndCluster<RangeSearch<EuclideanDistance, mat, tree::KDTree>, RandomPointSelection>(epsilon, minPoints, 
																											batchMode, 
																											dataMatArr, dataMatSize, dataMatRowNum, 
																											assignmentsArr, assignmentsArrSize, 
																											centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
		}
		else if (strcmp(treeType, "r") == 0)
		{
			initAndCluster<RangeSearch<EuclideanDistance, mat, tree::RTree>, RandomPointSelection>(epsilon, minPoints, 
																											batchMode, 
																											dataMatArr, dataMatSize, dataMatRowNum, 
																											assignmentsArr, assignmentsArrSize, 
																											centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
		}
		else if (strcmp(treeType, "r_star") == 0)
		{
			initAndCluster<RangeSearch<EuclideanDistance, mat, tree::RStarTree>, RandomPointSelection>(epsilon, minPoints, 
																											batchMode, 
																											dataMatArr, dataMatSize, dataMatRowNum, 
																											assignmentsArr, assignmentsArrSize, 
																											centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
		}
		else if (strcmp(treeType, "x") == 0)
		{
			initAndCluster<RangeSearch<EuclideanDistance, mat, tree::XTree>, RandomPointSelection>(epsilon, minPoints, 
																											batchMode, 
																											dataMatArr, dataMatSize, dataMatRowNum, 
																											assignmentsArr, assignmentsArrSize, 
																											centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
		}
		else if (strcmp(treeType, "hilbert_r") == 0)
		{
			initAndCluster<RangeSearch<EuclideanDistance, mat, tree::HilbertRTree>, RandomPointSelection>(epsilon, minPoints, 
																											batchMode, 
																											dataMatArr, dataMatSize, dataMatRowNum, 
																											assignmentsArr, assignmentsArrSize, 
																											centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
		}
		else if (strcmp(treeType, "r_plus") == 0)
		{
			initAndCluster<RangeSearch<EuclideanDistance, mat, tree::RPlusTree>, RandomPointSelection>(epsilon, minPoints, 
																											batchMode, 
																											dataMatArr, dataMatSize, dataMatRowNum, 
																											assignmentsArr, assignmentsArrSize, 
																											centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
		}
		else if (strcmp(treeType, "r_plus_plus") == 0)
		{
			initAndCluster<RangeSearch<EuclideanDistance, mat, tree::RPlusPlusTree>, RandomPointSelection>(epsilon, minPoints, 
																											batchMode, 
																											dataMatArr, dataMatSize, dataMatRowNum, 
																											assignmentsArr, assignmentsArrSize, 
																											centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
		}
		else if (strcmp(treeType, "cover") == 0)
		{
			initAndCluster<RangeSearch<EuclideanDistance, mat, tree::CoverTree>, RandomPointSelection>(epsilon, minPoints, 
																											batchMode, 
																											dataMatArr, dataMatSize, dataMatRowNum, 
																											assignmentsArr, assignmentsArrSize, 
																											centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
		}
		else if (strcmp(treeType, "ball") == 0)
		{
			initAndCluster<RangeSearch<EuclideanDistance, mat, tree::BallTree>, RandomPointSelection>(epsilon, minPoints, 
																											batchMode, 
																											dataMatArr, dataMatSize, dataMatRowNum, 
																											assignmentsArr, assignmentsArrSize, 
																											centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
		}
		else
		{
			raisePrologDomainExeption(treeType, 5, "The given TreeType is unkown!", "dbscan");
			return;
		}
	}
	else
	{
		raisePrologDomainExeption(selectionType, 4, "The given SelectionType is unkown!", "dbscan");
		return;
	}
	
}
