:- module(dbscan, [dbscan/10]).

%% requirements of library(struct)
:- load_files(library(str_decl),
                [when(compile_time), if(changed)]).

%% needed for using the array type
:- use_module(library(structs)).
:- use_module('../../helper_files/helper.pl').

%% type definitions for the float array
:- foreign_type
        float32          = float_32,
        float_array      = array(float32).

%% definitions for the connected function


%% --Input--
%%              double  epsilon
%%              int     minPoints
%%              bool    batchMode       => (1)true / (0)false
%%              str     selectionType   => "ordered", "random"
%%              str     treeType        => "kd", "r", "r_star", "x", "hilbert_r", "r_plus", "r_plus_plus", "cover", "ball"
%%              mat     data            => data(float_array), dataSize, dataRowNum
%%
%% --Output--
%%              vec     assignments     => assign(float_array), assignSize
%%              mat     centroids       => cent(float_array), centColNum, centRowNum
%%
%% --Description--
%%            This is a one predicate model where you configure the model with the input parameters and get returned the results in the same predicate.
%%
dbscan(Epsilon, MinPoints, BatchMode, SelectionType, TreeType, DataList, DataRows, AssignList, CentroidsList, ZCols) :-
        Epsilon > 0,
        MinPoints > 0,
        convert_list_to_float_array(DataList, DataRows, array(Xsize, Xrownum, X)),
        dbscanI(Epsilon, MinPoints, BatchMode, SelectionType, TreeType, X, Xsize, Xrownum, Y, Ysize, Z, ZCols, ZRows),
        convert_float_array_to_list(Y, Ysize, AssignList),
        convert_float_array_to_2d_list(Z, ZCols, ZRows, CentroidsList).

foreign(dbscan, c, dbscanI(  +float32, +integer, +integer, 
                            +string, +string, 
                            +pointer(float_array), +integer, +integer,
                            -pointer(float_array), -integer, 
                            -pointer(float_array), -integer, -integer)).


%% Defines the functions that get connected from main.cpp
foreign_resource(dbscan, [dbscan]).

:- load_foreign_resource(dbscan).
