
:- module(dbscan_tests, [run_dbscan_tests/0]).

:- use_module(library(plunit)).

:- use_module(dbscan).
:- use_module('../../helper_files/helper.pl').


%%
%% TESTING predicate dbscan/10
%%
:- begin_tests(dbscan).     

%% Failure Tests
                                            
test(dbscan_Negative_Epsilon, fail) :-
        dbscan(-1, 1, 0, ordered, kd, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _).
        
test(dbscan_Negative_MinPoints, fail) :-
        dbscan(1, -1, 0, ordered, kd, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _).

test(dbscan_wrong_Selection_Input, [error(domain_error('The given SelectionType is unkown!', wrongSelection), _)]) :-
        dbscan(1.0, 1, 0, wrongSelection, kd, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _).

test(dbscan_wrong_TreeType_Input_With_Orderd_Selection, [error(domain_error('The given TreeType is unkown!', wrongTree), _)]) :-
        dbscan(1.0, 1, 0, ordered, wrongTree, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _).

test(dbscan_wrong_TreeType_Input_With_Random_Selection, [error(domain_error('The given TreeType is unkown!', wrongTree), _)]) :-
        dbscan(1.0, 1, 0, random, wrongTree, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _).
        

%% Successful Tests

test(dbscan_KD_CSV) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,30, Data),
        dbscan(1.0, 1, 0, ordered, kd, Data, 4, AssignList, CentroidsList, _),
        dbscan(1.0, 1, 0, random, kd, Data, 4, AssignListRand, CentroidsListRand, _),
        print('\nOrdered'),
        print(AssignList),
        print('\nOrdered'),
        print(CentroidsList),
        print('\nRandom'),
        print(AssignListRand),
        print('\nRandom'),
        print(CentroidsListRand).

test(dbscan_R_CSV) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,30, Data),
        dbscan(1.0, 1, 0, ordered, r, Data, 4, AssignList, CentroidsList, _),
        dbscan(1.0, 1, 0, random, r, Data, 4, AssignListRand, CentroidsListRand, _),
        print('\nOrdered'),
        print(AssignList),
        print('\nOrdered'),
        print(CentroidsList),
        print('\nRandom'),
        print(AssignListRand),
        print('\nRandom'),
        print(CentroidsListRand).

test(dbscan_RSTAR_CSV) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,30, Data),
        dbscan(1.0, 1, 0, ordered, r_star, Data, 4, AssignList, CentroidsList, _),
        dbscan(1.0, 1, 0, random, r_star, Data, 4, AssignListRand, CentroidsListRand, _),
        print('\nOrdered'),
        print(AssignList),
        print('\nOrdered'),
        print(CentroidsList),
        print('\nRandom'),
        print(AssignListRand),
        print('\nRandom'),
        print(CentroidsListRand).

test(dbscan_X_CSV) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,30, Data),
        dbscan(1.0, 1, 0, ordered, x, Data, 4, AssignList, CentroidsList, _),
        dbscan(1.0, 1, 0, random, x, Data, 4, AssignListRand, CentroidsListRand, _),
        print('\nOrdered'),
        print(AssignList),
        print('\nOrdered'),
        print(CentroidsList),
        print('\nRandom'),
        print(AssignListRand),
        print('\nRandom'),
        print(CentroidsListRand).

test(dbscan_Hilbert_R_CSV) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,30, Data),
        dbscan(1.0, 1, 0, ordered, hilbert_r, Data, 4, AssignList, CentroidsList, _),
        dbscan(1.0, 1, 0, random, hilbert_r, Data, 4, AssignListRand, CentroidsListRand, _),
        print('\nOrdered'),
        print(AssignList),
        print('\nOrdered'),
        print(CentroidsList),
        print('\nRandom'),
        print(AssignListRand),
        print('\nRandom'),
        print(CentroidsListRand).

test(dbscan_RPlus_CSV) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,30, Data),
        dbscan(1.0, 1, 0, ordered, r_plus, Data, 4, AssignList, CentroidsList, _),
        dbscan(1.0, 1, 0, random, r_plus, Data, 4, AssignListRand, CentroidsListRand, _),
        print('\nOrdered'),
        print(AssignList),
        print('\nOrdered'),
        print(CentroidsList),
        print('\nRandom'),
        print(AssignListRand),
        print('\nRandom'),
        print(CentroidsListRand).

test(dbscan_RPlusPlus_CSV) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,30, Data),
        dbscan(1.0, 1, 0, ordered, r_plus_plus, Data, 4, AssignList, CentroidsList, _),
        dbscan(1.0, 1, 0, random, r_plus_plus, Data, 4, AssignListRand, CentroidsListRand, _),
        print('\nOrdered'),
        print(AssignList),
        print('\nOrdered'),
        print(CentroidsList),
        print('\nRandom'),
        print(AssignListRand),
        print('\nRandom'),
        print(CentroidsListRand).

test(dbscan_Cover_CSV) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,30, Data),
        dbscan(1.0, 1, 0, ordered, cover, Data, 4, AssignList, CentroidsList, _),
        dbscan(1.0, 1, 0, random, cover, Data, 4, AssignListRand, CentroidsListRand, _),
        print('\nOrdered'),
        print(AssignList),
        print('\nOrdered'),
        print(CentroidsList),
        print('\nRandom'),
        print(AssignListRand),
        print('\nRandom'),
        print(CentroidsListRand).


test(dbscan_Ball_CSV) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,30, Data),
        dbscan(1.0, 1, 0, ordered, ball, Data, 4, AssignList, CentroidsList, _),
        dbscan(1.0, 1, 0, random, ball, Data, 4, AssignListRand, CentroidsListRand, _),
        print('\nOrdered'),
        print(AssignList),
        print('\nOrdered'),
        print(CentroidsList),
        print('\nRandom'),
        print(AssignListRand),
        print('\nRandom'),
        print(CentroidsListRand).


%% Some use cases
test(dbscan_Orderd_KD) :-
        dbscan(1.0, 1, 0, ordered, kd, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, AssignList, CentroidsList, _),
        print(AssignList),
        print('\n'),
        print(CentroidsList).

test(dbscan_Orderd_KD_CSV) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        dbscan(1.0, 1, 0, ordered, kd, Data, 4, AssignList, CentroidsList, _),
        print(AssignList),
        print('\n'),
        print(CentroidsList).

test(dbscan_Random_KD) :-
        dbscan(3.2, 4, 1, random, kd, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0.0,0.0,0.0,0.0], [[4.824999809265137,3.200000047683716,1.399999976158142]], _).

test(dbscan_Random_KD_CSV) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        dbscan(3.2, 4, 1, random, kd, Data, 4, [0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0], [[4.860000133514404,3.309999942779541,1.4500000476837158,0.2199999988079071]], _).

test(dbscan_Random_RSTAR_CSV) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,100, Data),
        dbscan(1.0, 10, 1, random, r_star, Data, 4, AssignList, CentroidsList, _),
        print(AssignList),
        print('\n'),
        print(CentroidsList).

:- end_tests(dbscan).

run_dbscan_tests :-
        run_tests.
