
:- module(decision_tree_tests, [run_decision_tree_tests/0]).

:- use_module(library(plunit)).

:- use_module(decision_tree).
:- use_module('../../helper_files/helper.pl').

reset_Model_With_Train :-
        decision_tree_initModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,0], 2, 10, 0.5, 0).

%%
%% TESTING predicate decision_tree_initModel/7
%%
:- begin_tests(decision_tree_initModel).      

%% Failure Tests
                                            
test(decision_tree_Negative_NumClass, fail) :-
        decision_tree_initModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,0,0,0], -1, 1, 0.5, 0).

test(decision_tree_Negative_LeafSize, fail) :-
        decision_tree_initModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,0,0,0], 1, -1, 0.5, 0).

test(decision_tree_Negative_GainSplit, fail) :-
        decision_tree_initModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,0,0,0], 1, 1, -0.5, 0).

test(decision_tree_Too_High_GainSplit, fail) :-
        decision_tree_initModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,0,0,0], 1, 1, 1.5, 0).

test(decision_tree_Negative_MaxDepth, fail) :-
        decision_tree_initModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,0,0,0], 1, 1, 0.5, -1).

test(decision_tree_Init_With_Wrong_Label_Dims1, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        decision_tree_initModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,0], 1, 1, 0.5, 1).

test(decision_tree_Init_With_Wrong_Label_Dims2, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        decision_tree_initModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,0,0,0,0,0], 1, 1, 0.5, 1).

        

%% Successful Tests

test(initModel_Direkt_Input_Use) :-
        decision_tree_initModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,0,0,0], 2, 10, 0.5, 0).

test(initModel_Direkt_CSV_Use) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        decision_tree_initModel(Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 2, 0.7, 3).

:- end_tests(decision_tree_initModel).


%%
%% TESTING predicate decision_tree_classifyPoint/3
%%
:- begin_tests(decision_tree_classifyPoint).      

%% Failure Tests

%% the point dimensionaly seems to not matter for mlpack
%% so im not certain if this should be forced to fail
test(classify_Point_With_Wrong_Dims) :-
        reset_Model_With_Train,
        decision_tree_classifyPoint([5.1,3.5,1.4,1.2,3.3], Prediction, AssignList),
        print(Prediction),
        print('\n'),
        print(AssignList).
        

%% Successful Tests

test(classify_Point1) :-
        reset_Model_With_Train,
        decision_tree_classifyPoint([5.1,3.5,1.4], Prediction, AssignList),
        print(Prediction),
        print('\n'),
        print(AssignList).

test(classify_Point2) :-
        reset_Model_With_Train,
        decision_tree_classifyPoint([6.2,1.9,2.3], Prediction, AssignList),
        print(Prediction),
        print('\n'),
        print(AssignList).

:- end_tests(decision_tree_classifyPoint).


%%
%% TESTING predicate decision_tree_classifyMatrix/4
%%
:- begin_tests(decision_tree_classifyMatrix).      

%% Failure Tests

%% the point dimensionaly seems to not matter for mlpack
%% so im not certain if this should be forced to fail
test(classify_Matrix_With_Wrong_Dims1) :-
        reset_Model_With_Train,
        decision_tree_classifyMatrix([3, 2, 0, 5, 1, 4, 1, 0, 4, 3, 3, 5, 0, 5, 5, 2, 5, 5, 0, 2], 5, PredictionList, ProbsList, _),
        print(PredictionList),
        print('\n'),
        print(ProbsList).

%% the point dimensionaly seems to not matter for mlpack
%% so im not certain if this should be forced to fail
test(classify_Matrix_With_Wrong_Dims2) :-
        reset_Model_With_Train,
        decision_tree_classifyMatrix([3, 2, 0, 5, 1, 4, 0, 0, 4, 3, 3, 5, 0, 5, 5, 2, 5, 5, 0, 2], 2, PredictionList, ProbsList, _),
        print(PredictionList),
        print('\n'),
        print(ProbsList).


%% Successful Tests

test(classify_Matrix_Direkt_Input1) :-
        reset_Model_With_Train,
        decision_tree_classifyMatrix([5.1,3.5,1.4, 5.1,3.5,1.4, 5.1,3.5,1.4, 5.1,3.5,1.4, 5.1,3.5,1.4], 3, PredictionList, ProbsList, _),
        print(PredictionList),
        print('\n'),
        print(ProbsList).

test(classify_Matrix_Direkt_Input2) :-
        reset_Model_With_Train,
        decision_tree_classifyMatrix([2, 2, 3, 5, 1, 4, 1, 1, 4, 0, 3, 5, 0, 5, 5], 3, PredictionList, ProbsList, _),
        print(PredictionList),
        print('\n'),
        print(ProbsList).

test(classify_Matrix_CSV_Trained) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,30, Data),
        decision_tree_train(Data, 4, [0,1,0,1,1,0,1,1,1,0, 0,1,0,1,1,0,1,1,1,0, 0,1,0,1,1,0,1,1,1,0], 2, 5, 0.0007, 0, _),
        decision_tree_classifyMatrix([2, 2, 3, 5, 1, 4, 1, 1, 4, 0, 3, 5, 0, 5, 5, 2, 2, 6, 0, 1], 4, PredictionList, ProbsList, _),
        print(PredictionList),
        print('\n'),
        print(ProbsList).

:- end_tests(decision_tree_classifyMatrix).


%%
%% TESTING predicate decision_tree_train/8
%%
:- begin_tests(decision_tree_train).      

%% Failure Tests
                                            
test(decision_tree_Train_Negative_NumClass, fail) :-
        decision_tree_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,0,0,0], -1, 1, 0.5, 0, _).

test(decision_tree_Train_Negative_LeafSize, fail) :-
        decision_tree_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,0,0,0], 1, -1, 0.5, 0, _).

test(decision_tree_Train_Negative_GainSplit, fail) :-
        decision_tree_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,0,0,0], 1, 1, -0.5, 0, _).

test(decision_tree_Train_Too_High_GainSplit, fail) :-
        decision_tree_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,0,0,0], 1, 1, 1.5, 0, _).

test(decision_tree_Train_Negative_MaxDepth, fail) :-
        decision_tree_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,0,0,0], 1, 1, 0.5, -1, _).

test(decision_tree_Train_Wrong_Label_Dims1, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        decision_tree_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,0], 1, 1, 0.5, 1, _).

test(decision_tree_Train_Wrong_Label_Dims2, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        decision_tree_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,0,0,0,0,0], 1, 1, 0.5, 1, _).
        

%% Successful Tests

test(initModel_Direkt_Input_Use, [true(Entropy =:= 0.0)]) :-
        decision_tree_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,0,0,0], 2, 10, 0.5, 0, Entropy).

test(initModel_Direkt_CSV_Use, [true(Entropy =:= 0.48)]) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        decision_tree_train(Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 2, 0.7, 3, Entropy).

test(initModel_Direkt_Input_Use, [true(Entropy =:= 0.0)]) :-
        reset_Model_With_Train,
        decision_tree_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,0,0,0], 3, 10, 0.7, 0, Entropy).

test(initModel_Direkt_CSV_Use, [true(Entropy =:= 0.3767195767195767)]) :-
        reset_Model_With_Train,
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,30, Data),
        decision_tree_train(Data, 4, [0,1,0,1,1,0,1,1,1,0, 0,1,0,1,1,0,1,1,1,0, 0,1,0,1,1,0,1,1,1,0], 2, 5, 0.0005, 0, Entropy).

:- end_tests(decision_tree_train).

run_decision_tree_tests :-
        run_tests.
