
:- module(fastmks, [    fastmks_initModel/10,
                        fastmks_searchWithQuery/8,
                        fastmks_searchNoQuery/5]).

%% requirements of library(struct)
:- load_files(library(str_decl),
        [when(compile_time), if(changed)]).

%% needed for using the array type
:- use_module(library(structs)).
:- use_module('../../helper_files/helper.pl').

%% type definitions for the float array
:- foreign_type
        float32          = float_32,
        float_array      = array(float32).


%% definitions for the connected function

%% --Input--
%%              mat     referenceData,
%%              string  kernel          "linear", "polynomial", "cosine", "gaussian", "epanechnikov", "triangular", "hyptan"
%%              float32 degree,
%%              float32 offset,
%%              float32 bandwidth,
%%              float32 scale,
%%              bool    singleMode      => (1)true / (0)false,
%%              bool    naive           => (1)true / (0)false,
%%              double  base
%%
%% --Output--
%%
%% --Description--
%%              Initializes the model on the given reference set.
%%
fastmks_initModel(DataList, DataRows, Kernel, Degree, Offset, Bandwidth, Scale, SingleMode, Naive, Base) :-
        Base > 1.0,
        Bandwidth > 0.0,
        convert_list_to_float_array(DataList, DataRows, array(Xsize, Xrows, X)),
        initModelI(X, Xsize, Xrows, Kernel, Degree, Offset, Bandwidth, Scale, SingleMode, Naive, Base).

foreign(initModel, c, initModelI(       +pointer(float_array), +integer, +integer, 
                                        +string, 
                                        +float32, +float32, +float32, +float32, 
                                        +integer, +integer, 
                                        +float32)).



%% --Input--
%%              mat     querySet,
%%              int     k,
%%              double  base
%%
%% --Output--
%%              mat     indices,
%%              mat     kernels
%%
%% --Description--
%%              Search with a different query set.
%%
fastmks_searchWithQuery(DataList, DataRows, K, IndicesList, YCols, KernelsList, ZCols, Base) :-
        K > 0,
        Base > 1.0,
        convert_list_to_float_array(DataList, DataRows, array(Xsize, Xrows, X)),
        searchWithQueryI(X, Xsize, Xrows, K, Y, YCols, YRows, Z, ZCols, ZRows, Base),
        convert_float_array_to_2d_list(Y, YCols, YRows, IndicesList),
        convert_float_array_to_2d_list(Z, ZCols, ZRows, KernelsList).

foreign(searchWithQuery, c, searchWithQueryI(   +pointer(float_array), +integer, +integer, 
                                                +integer, 
                                                -pointer(float_array), -integer, -integer, 
                                                -pointer(float_array), -integer, -integer, 
                                                +float32)).



%% --Input--
%%              int     k
%%
%% --Output--
%%              mat     indices,
%%              mat     kernels
%%
%% --Description--
%%              Search with the reference set as the query set.
%%
fastmks_searchNoQuery(K, IndicesList, YCols, KernelsList, ZCols) :-
        K > 0,
        searchNoQueryI(K, Y, YCols, YRows, Z, ZCols, ZRows),
        convert_float_array_to_2d_list(Y, YCols, YRows, IndicesList),
        convert_float_array_to_2d_list(Z, ZCols, ZRows, KernelsList).

foreign(searchNoQuery, c, searchNoQueryI(       +integer, 
                                                -pointer(float_array), -integer, -integer, 
                                                -pointer(float_array), -integer, -integer)).


%% Defines the functions that get connected from main.cpp
foreign_resource(fastmks, [     initModel,
                                searchWithQuery,
                                searchNoQuery]).


:- load_foreign_resource(fastmks).
