
:- module(fastmks_tests, [run_fastmks_tests/0]).

:- use_module(library(plunit)).

:- use_module(fastmks).
:- use_module('../../helper_files/helper.pl').

reset_Model :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        fastmks_initModel(Data, 4, linear, 0.0, 0.0, 1.0, 0.0, 0, 0, 1.2).


%%
%% TESTING predicate fastmks_initModel/10
%%
:- begin_tests(fastmks_initModel).      

%% Failure Tests

test(searchWithQuery_Fastmks_Search_Before_Init, [error(_,system_error('The Model hasnt been trained yet!'))]) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,5, Data),
        fastmks_searchWithQuery(Data, 4, 2, _, _, _, _, 1.1).

test(searchNoQuery_Fastmks_Search_Before_Init, [error(_,system_error('The Model hasnt been trained yet!'))]) :-
        fastmks_searchNoQuery(2, _, _, _, _).
                                            
test(initModel_Fatsmks_WrongKernel_Input, [error(domain_error('The given kernel is unkown!' , wrongKernel), _)]) :-
        fastmks_initModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, wrongKernel, 0.0, 0.0, 1.0, 0.0, 0, 0, 1.2).

test(initModel_Fatsmks_Bad_Base_Input, fail) :-
        fastmks_initModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, linear, 0.0, 0.0, 1.0, 0.0, 0, 0, -0.1).
        

%% Successful Tests

test(iniModel_Fastmks_Linear) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        fastmks_initModel(Data, 4, linear, 0.0, 0.0, 1.0, 0.0, 0, 0, 1.5).

test(iniModel_Fastmks_Polynomial) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        fastmks_initModel(Data, 4, polynomial, 0.0, 0.0, 1.0, 0.0, 0, 0, 1.5).

test(iniModel_Fastmks_Cosine) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        fastmks_initModel(Data, 4, cosine, 0.0, 0.0, 1.0, 0.0, 0, 0, 1.5).

test(iniModel_Fastmks_Gaussian) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        fastmks_initModel(Data, 4, gaussian, 0.5, 0.5, 0.5, 0.5, 0, 0, 1.5).

test(iniModel_Fastmks_Epanechnikov) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        fastmks_initModel(Data, 4, epanechnikov, 0.0, 0.0, 1.0, 0.0, 0, 0, 1.5).

test(iniModel_Fastmks_Triangular) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        fastmks_initModel(Data, 4, triangular, 0.0, 0.0, 1.0, 0.0, 0, 0, 1.5).

test(iniModel_Fastmks_Hyptan) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        fastmks_initModel(Data, 4, hyptan, 0.0, 0.0, 1.0, 1.0, 0, 0, 1.5).

:- end_tests(fastmks_initModel).


%%
%% TESTING predicate fastmks_searchWithQuery/8
%%
:- begin_tests(fastmks_searchWithQuery).      

%% Failure Tests
                                            
test(searchWithQuery_Fastmks_Negative_K, fail) :-
        reset_Model,
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,5, Data),
        fastmks_searchWithQuery(Data, 4, -2, _, _, _, _, 1.1).

test(searchWithQuery_Fastmks_Negative_Base, fail) :-
        reset_Model,
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,5, Data),
        fastmks_searchWithQuery(Data, 4, 2, _, _, _, _, -1.1).
        

%% Successful Tests

test(searchWithQuery_Fastmks_New_Query) :-
        reset_Model,
        fastmks_searchWithQuery([3, 2, 0, 5, 1, 4, 0, 0, 4, 3, 3, 5, 0, 5, 5, 2, 5, 5, 0, 2], 4, 2, IndicesList, _, KernelsList, _, 1.1),
        print('Indices:\n'),
        print(IndicesList),
        print('Kernels:\n'),
        print(KernelsList).

test(searchWithQuery_Fastmks_Training_Data_Query) :-
        reset_Model,
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,5, Data),
        fastmks_searchWithQuery(Data, 4, 2, IndicesList, _, KernelsList, _, 1.1),
        print('Indices:\n'),
        print(IndicesList),
        print('Kernels:\n'),
        print(KernelsList).

:- end_tests(fastmks_searchWithQuery).


%%
%% TESTING predicate fastmks_searchNoQuery/5
%%
:- begin_tests(fastmks_searchNoQuery).      

%% Failure Tests
                                            
test(searchNoQuery_Fastmks_Negative_K, fail) :-
        reset_Model,
        fastmks_searchNoQuery(-2, _, _, _, _).
        

%% Successful Tests

test(testDescription) :-
        reset_Model,
        fastmks_searchNoQuery(2, IndicesList, _, KernelsList, _),
        print('Indices:\n'),
        print(IndicesList),
        print('Kernels:\n'),
        print(KernelsList).

test(testDescription) :-
        reset_Model,
        fastmks_searchNoQuery(5, IndicesList, _, KernelsList, _),
        print('Indices:\n'),
        print(IndicesList),
        print('Kernels:\n'),
        print(KernelsList).

:- end_tests(fastmks_searchNoQuery).

run_fastmks_tests :-
        run_tests.

