
:- module(hoeffding_tree_tests, [run_hoeffding_tree_tests/0]).

:- use_module(library(plunit)).

:- use_module(hoeffding_tree).
:- use_module('../../helper_files/helper.pl').

reset_Model :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        hoeffding_tree_initAndBuildModel(gini_binary, Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 0, 0.95, 5000, 100, 100, 10, 100).

%%
%% TESTING predicate hoeffding_tree_initAndBuildModel/12
%%
:- begin_tests(hoeffding_tree_initAndBuildModel).      

%% Failure Tests

test(hoeffding_Init_Classify_Befor_Init, [error(_,system_error('The model is not initialized!'))]) :-
        hoeffding_tree_classify([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _).
                                            
test(hoeffding_Init_Wrong_TreeType, [error(domain_error('The given TreeType is unkown!' , wrongType), _)]) :-
        hoeffding_tree_initAndBuildModel(wrongType, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0, 0.95, 5000, 100, 100, 10, 100).

test(hoeffding_Init_Negative_NumClass, fail) :-
        hoeffding_tree_initAndBuildModel(gini_binary, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], -2, 0, 0.95, 5000, 100, 100, 10, 100).

test(hoeffding_Init_Bad_SuccessProbability, fail) :-
        hoeffding_tree_initAndBuildModel(gini_binary, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0, -1.0, 5000, 100, 100, 10, 100),
        hoeffding_tree_initAndBuildModel(gini_binary, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0, 2.0, 5000, 100, 100, 10, 100).

test(hoeffding_Init_Negative_MaxSamples, fail) :-
        hoeffding_tree_initAndBuildModel(gini_binary, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0, 0.95, -5000, 100, 100, 10, 100).

test(hoeffding_Init_Negative_CheckInterval, fail) :-
        hoeffding_tree_initAndBuildModel(gini_binary, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0, 0.95, 5000, -100, 100, 10, 100).

test(hoeffding_Init_Negative_MinSamples, fail) :-
        hoeffding_tree_initAndBuildModel(gini_binary, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0, 0.95, 5000, 100, -100, 10, 100).

test(hoeffding_Init_Negative_Bins, fail) :-
        hoeffding_tree_initAndBuildModel(gini_binary, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0, 0.95, 5000, 100, 100, -10, 100).

test(hoeffding_Init_Negative_ObservationsBeforeBinning, fail) :-
        hoeffding_tree_initAndBuildModel(gini_binary, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0, 0.95, 5000, 100, 100, 10, -100).


test(hoeffding_Init_With_Wrong_Label_Dims1, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        hoeffding_tree_initAndBuildModel(gini_binary, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], 2, 0, 0.95, 5000, 100, 100, 10, 100).

test(hoeffding_Init_With_Wrong_Label_Dims2, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        hoeffding_tree_initAndBuildModel(gini_binary, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,0,1], 2, 0, 0.95, 5000, 100, 100, 10, 100).
        

%% Successful Tests

test(hoeffding_Init_GiniHoeffding_Direkt_Input) :-
        hoeffding_tree_initAndBuildModel(gini_hoeffding, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0, 0.95, 5000, 100, 100, 10, 100).

test(hoeffding_Init_GiniHoeffding_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        hoeffding_tree_initAndBuildModel(gini_hoeffding, Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 0, 0.95, 5000, 100, 100, 10, 100).

test(hoeffding_Init_GiniBinary_Direkt_Input) :-
        hoeffding_tree_initAndBuildModel(gini_binary, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0, 0.95, 5000, 100, 100, 10, 100).

test(hoeffding_Init_GiniBinary_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        hoeffding_tree_initAndBuildModel(gini_binary, Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 0, 0.95, 5000, 100, 100, 10, 100).

test(hoeffding_Init_InfoHoeffding_Direkt_Input) :-
        hoeffding_tree_initAndBuildModel(info_hoeffding, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0, 0.95, 5000, 100, 100, 10, 100).

test(hoeffding_Init_InfoHoeffding_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        hoeffding_tree_initAndBuildModel(info_hoeffding, Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 0, 0.95, 5000, 100, 100, 10, 100).

test(hoeffding_Init_InfoBinary_Direkt_Input) :-
        hoeffding_tree_initAndBuildModel(info_binary, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0, 0.95, 5000, 100, 100, 10, 100).

test(hoeffding_Init_InfoBinary_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        hoeffding_tree_initAndBuildModel(info_binary, Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 0, 0.95, 5000, 100, 100, 10, 100).

:- end_tests(hoeffding_tree_initAndBuildModel).


%%
%% TESTING predicate hoeffding_tree_classify/4
%%
:- begin_tests(hoeffding_tree_classify).      

%% Failure Tests

%% seems to be able to classify diffrent Dimensions than trained
test(hoeffding_Classify_Different_Dims_To_Train) :-
        reset_Model,
        hoeffding_tree_classify([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 2, PredicList, ProbsList),
        print('\nPredictions: '),
        print(PredicList),
        print('\nProbabilities: '),
        print(ProbsList).
        

%% Successful Tests

test(hoeffding_Classify_GiniHoeffding) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        hoeffding_tree_initAndBuildModel(gini_hoeffding, Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 0, 0.95, 5000, 100, 100, 10, 100),
        hoeffding_tree_classify([3, 2, 0, 5, 1, 4, 1, 0, 4, 3, 3, 5, 0, 5, 5, 2, 5, 5, 0, 2], 4, PredicList, ProbsList),
        print('\nPredictions: '),
        print(PredicList),
        print('\nProbabilities: '),
        print(ProbsList).

test(hoeffding_Classify_GiniBinary) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        hoeffding_tree_initAndBuildModel(gini_binary, Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 0, 0.95, 5000, 100, 100, 10, 100),
        hoeffding_tree_classify([3, 2, 0, 5, 1, 4, 1, 0, 4, 3, 3, 5, 0, 5, 5, 2, 5, 5, 0, 2], 4, PredicList, ProbsList),
        print('\nPredictions: '),
        print(PredicList),
        print('\nProbabilities: '),
        print(ProbsList).

test(hoeffding_Classify_InfoHoeffding) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        hoeffding_tree_initAndBuildModel(info_hoeffding, Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 0, 0.95, 5000, 100, 100, 10, 100),
        hoeffding_tree_classify([3, 2, 0, 5, 1, 4, 1, 0, 4, 3, 3, 5, 0, 5, 5, 2, 5, 5, 0, 2], 4, PredicList, ProbsList),
        print('\nPredictions: '),
        print(PredicList),
        print('\nProbabilities: '),
        print(ProbsList).

test(hoeffding_Classify_InfoBinary) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        hoeffding_tree_initAndBuildModel(info_binary, Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 0, 0.95, 5000, 100, 100, 10, 100),
        hoeffding_tree_classify([3, 2, 0, 5, 1, 4, 1, 0, 4, 3, 3, 5, 0, 5, 5, 2, 5, 5, 0, 2], 4, PredicList, ProbsList),
        print('\nPredictions: '),
        print(PredicList),
        print('\nProbabilities: '),
        print(ProbsList).

:- end_tests(hoeffding_tree_classify).


%%
%% TESTING predicate hoeffding_tree_train/4
%%
:- begin_tests(hoeffding_tree_train).      

%% Failure Tests
                                            
test(hoeffding_Train_With_Wrong_Label_Dims1, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model,
        hoeffding_tree_train([3, 2, 0, 5, 1, 4, 1, 0, 4, 3, 3, 5, 0, 5, 5, 2, 5, 5, 0, 2], 4, [0,1,0,0], 0).

test(hoeffding_Train_With_Wrong_Label_Dims2, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model,
        hoeffding_tree_train([3, 2, 0, 5, 1, 4, 1, 0, 4, 3, 3, 5, 0, 5, 5, 2, 5, 5, 0, 2], 4, [0,1,0,1,0,0,1], 0).

        
test(hoeffding_Train_Bad_Data_Dims) :-
        reset_Model,
        hoeffding_tree_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 0).


%% Successful Tests

test(hoeffding_Train_Normal_Use) :-
        reset_Model,
        hoeffding_tree_train([3, 2, 0, 5, 1, 4, 1, 0, 4, 3, 3, 5, 0, 5, 5, 2, 5, 5, 0, 2], 4, [0,0,1,1,0], 1),
        hoeffding_tree_train([3, 2, 0, 5, 1, 4, 1, 0, 4, 3, 3, 5, 0, 5, 5, 2, 5, 5, 0, 2], 4, [0,0,1,1,0], 0).

:- end_tests(hoeffding_tree_train).

run_hoeffding_tree_tests :-
        run_tests.

