
:- module(kde, [        kde_initAndBuildModel/13,
                        kde_evaluateWithQuery/3,
                        kde_evaluateNoQuery/1]).

%% requirements of library(struct)
:- load_files(library(str_decl),
        [when(compile_time), if(changed)]).

%% needed for using the array type
:- use_module(library(structs)).
:- use_module('../../helper_files/helper.pl').

%% type definitions for the float array
:- foreign_type
        float32          = float_32,
        float_array      = array(float32).

%% definitions for the connected function



%% --Input--
%%              float32 bandwidth               => 1.0,
%%              float32 relError                => 0.05,
%%              float32 absError                => 0.0,
%%              string  kernelType              "gaussian", "epanechnikov", "laplacian", "spherical", "triangular",
%%              string  treeType                "kd-tree", "ball-tree", "cover-tree", "octree", "r-tree",
%%              string  algorithm               "dual-tree", "single-tree",
%%              bool    monteCarlo              (1)true / (0)false => false,
%%              float32 mcProb                  => 0.95,
%%              int     initialSampleSize       => 100,
%%              float32 mcEntryCoef             => 3.0,
%%              float32 mcBreakCoef             => 0.4,
%%              mat     referenceSet
%%
%% --Output--
%%
%% --Description--
%%              Build the KDE model with the given parameters and then trains it with the given reference data.
%%
kde_initAndBuildModel(Bandwidth, RelError, AbsError, KernelType, TreeType, Algorithm, MonteCarlo, McProb, InitialSampleSize, MCEntryCoef, MCBreakCoef, DataList, DataRows) :-
        Bandwidth > 0.0,
        RelError >= 0.0, RelError =< 1.0,
        AbsError >= 0.0,
        McProb >= 0.0, McProb < 1.0,
        InitialSampleSize > 0,
        MCEntryCoef >= 1,
        MCBreakCoef > 0.0, MCBreakCoef =< 1.0,
        convert_list_to_float_array(DataList, DataRows, array(Xsize, Xrownum, X)),
        initAndBuildModelI(Bandwidth, RelError, AbsError, KernelType, TreeType, Algorithm, MonteCarlo, McProb, InitialSampleSize, MCEntryCoef, MCBreakCoef, X, Xsize, Xrownum).

foreign(initAndBuildModel, c, initAndBuildModelI(+float32, +float32, +float32, 
                                                +string, +string, +string, 
                                                +integer, 
                                                +float32, +integer, +float32, +float32,
                                                +pointer(float_array), +integer, +integer)).



%% --Input--
%%              mat     querySet
%%
%% --Output--
%%              vec     estimations
%%
%% --Description--
%%              initAndBuildModel/14 has to be called before.
%%
kde_evaluateWithQuery(QueryList, QueryRows, EstimationList) :-
        convert_list_to_float_array(QueryList, QueryRows, array(Xsize, Xrownum, X)),
        evaluateWithQueryI(X, Xsize, Xrownum, Y, Ysize),
        convert_float_array_to_list(Y, Ysize, EstimationList).

foreign(evaluateWithQuery, c, evaluateWithQueryI(+pointer(float_array), +integer, +integer, 
                                                -pointer(float_array), -integer)).



%% --Input--
%%
%% --Output--
%%              vec     estimations
%%
%% --Description--
%%              Perform kernel density estimation on the reference set.
%%              If possible, it returns normalized estimations.
%%              initAndBuildModel/14 has to be called before.
%%
kde_evaluateNoQuery(EstimationList) :-
        evaluateNoQueryI(Y, Ysize),
        convert_float_array_to_list(Y, Ysize, EstimationList).

foreign(evaluateNoQuery, c, evaluateNoQueryI(-pointer(float_array), -integer)).


%% Defines the functions that get connected from main.cpp
foreign_resource(kde, [ initAndBuildModel,
                        evaluateWithQuery,
                        evaluateNoQuery]).

:- load_foreign_resource(kde).
