#include <sicstus/sicstus.h>
/* ex_glue.h is generated by splfr from the foreign/[2,3] facts.
   Always include the glue header in your foreign resource code.
*/
#include "kernel_pca_glue.h"
#include <mlpack/methods/kernel_pca/kernel_pca.hpp>
#include <mlpack/core.hpp>

#include <mlpack/methods/kernel_pca/kernel_rules/nystroem_method.hpp>

#include <mlpack/methods/nystroem_method/ordered_selection.hpp>
#include <mlpack/methods/nystroem_method/random_selection.hpp>

// including helper functions for converting between arma structures and 
#include "../../helper_files/helper.hpp"

// some of the most used namespaces
using namespace arma;
using namespace mlpack;
using namespace std;
using namespace mlpack::kpca;
using namespace mlpack::kernel;


template<typename KernelType>
bool useNystroemKernelPCA(char const *nystroemMethod, KernelType kernel, bool centerTranformedData, mat data, mat transformedDataReturnMat, vec eigvalReturnVector, mat eigvecReturnMat, size_t returnDim)
{
    if (strcmp(nystroemMethod, "kmeans") == 0)        
    {
        KernelPCA<KernelType, NystroemKernelRule<KernelType, KMeansSelection<>>>(kernel, (centerTranformedData == 1))
            .Apply(data, transformedDataReturnMat, eigvalReturnVector, eigvecReturnMat, returnDim);
    }
    else if (strcmp(nystroemMethod, "ordered") == 0)
    {
        KernelPCA<KernelType, NystroemKernelRule<KernelType, OrderedSelection>>(kernel, (centerTranformedData == 1))
            .Apply(data, transformedDataReturnMat, eigvalReturnVector, eigvecReturnMat, returnDim);
    }
    else if (strcmp(nystroemMethod, "random") == 0)
    {
        KernelPCA<KernelType, NystroemKernelRule<KernelType, RandomSelection>>(kernel, (centerTranformedData == 1))
            .Apply(data, transformedDataReturnMat, eigvalReturnVector, eigvecReturnMat, returnDim);
    }
    else
    {
        raisePrologDomainExeption(nystroemMethod, 4, "The given NystroemMethod is unknown!", "kernel_pca");
        return false;
    }
    return true;
}

// TODO: 
// input:   const KernelType 	kernel = KernelType(),
//          const bool 	        centerTransformedData = false
//          const arma::mat & 	data,
//          arma::mat & 	    transformedData,
//          arma::vec & 	    eigval,
//          arma::mat & 	    eigvec,
//          const size_t 	    newDimension
// output: 
// description: 
void kernel_pca(char const *kernel, SP_integer centerTranformedData, SP_integer useNystroem, char const *nystroemMethod, 
                    double degree, double offset, double bandwidth, double scale,
                    float *dataMatArr, SP_integer dataMatSize, SP_integer dataMatRowNum,
                    float **transformedDataMatArr, SP_integer *transformedDataMatColNum, SP_integer *transformedDataMatRowNum,
                    float **eigvalArr, SP_integer *eigvalArrSize,
                    float **eigvecMatArr, SP_integer *eigvecMatColNum, SP_integer *eigvecMatRowNum, 
                    SP_integer newDimension)
{
    // convert the Prolog array to arma::mat
    mat data = convertArrayToMat(dataMatArr, dataMatSize, dataMatRowNum);

    
    // create the ReturnMat
    mat transformedDataReturnMat;
    // create the ReturnVector
    vec eigvalReturnVector;
    // create the ReturnMat
    mat eigvecReturnMat;

    
    if (strcmp(kernel, "linear") == 0)        
    {
        LinearKernel lk;
        if(useNystroem == 1)
        {
            if(!useNystroemKernelPCA<LinearKernel>(nystroemMethod, lk, (centerTranformedData == 1), data, transformedDataReturnMat, eigvalReturnVector, eigvecReturnMat, newDimension))
                return;
        }
        else
        {
            KernelPCA<LinearKernel>(lk, (centerTranformedData == 1))
            .Apply(data, transformedDataReturnMat, eigvalReturnVector, eigvecReturnMat, newDimension);
        }
    }
    else if (strcmp(kernel, "polynomial") == 0)
    {
        PolynomialKernel pk(degree, offset);
        if(useNystroem == 1)
        {
            if(!useNystroemKernelPCA<PolynomialKernel>(nystroemMethod, pk, (centerTranformedData == 1), data, transformedDataReturnMat, eigvalReturnVector, eigvecReturnMat, newDimension))
                return;
        }
        else
        {
            KernelPCA<PolynomialKernel>(pk, (centerTranformedData == 1))
            .Apply(data, transformedDataReturnMat, eigvalReturnVector, eigvecReturnMat, newDimension);
        }
    }
    else if (strcmp(kernel, "cosine") == 0)
    {
        CosineDistance cd;
        if(useNystroem == 1)
        {
            if(!useNystroemKernelPCA<CosineDistance>(nystroemMethod, cd, (centerTranformedData == 1), data, transformedDataReturnMat, eigvalReturnVector, eigvecReturnMat, newDimension))
                return;
        }
        else
        {
            KernelPCA<CosineDistance>(cd, (centerTranformedData == 1))
            .Apply(data, transformedDataReturnMat, eigvalReturnVector, eigvecReturnMat, newDimension);
        }
    }
    else if (strcmp(kernel, "gaussian") == 0)
    {
        GaussianKernel gk(bandwidth);
        if(useNystroem == 1)
        {
            if(!useNystroemKernelPCA<GaussianKernel>(nystroemMethod, gk, (centerTranformedData == 1), data, transformedDataReturnMat, eigvalReturnVector, eigvecReturnMat, newDimension))
                return;
        }
        else
        {
            KernelPCA<GaussianKernel>(gk, (centerTranformedData == 1))
            .Apply(data, transformedDataReturnMat, eigvalReturnVector, eigvecReturnMat, newDimension);
        }
    }
    else if (strcmp(kernel, "epanechnikov") == 0)
    {
        EpanechnikovKernel ek(bandwidth);
        if(useNystroem == 1)
        {
            if(!useNystroemKernelPCA<EpanechnikovKernel>(nystroemMethod, ek, (centerTranformedData == 1), data, transformedDataReturnMat, eigvalReturnVector, eigvecReturnMat, newDimension))
                return;
        }
        else
        {
            KernelPCA<EpanechnikovKernel>(ek, (centerTranformedData == 1))
            .Apply(data, transformedDataReturnMat, eigvalReturnVector, eigvecReturnMat, newDimension);
        }
    }
    else if (strcmp(kernel, "laplacian") == 0)
    {
        LaplacianKernel tk(bandwidth);
        if(useNystroem == 1)
        {
            if(!useNystroemKernelPCA<LaplacianKernel>(nystroemMethod, tk, (centerTranformedData == 1), data, transformedDataReturnMat, eigvalReturnVector, eigvecReturnMat, newDimension))
                return;
        }
        else
        {
            KernelPCA<LaplacianKernel>(tk, (centerTranformedData == 1))
            .Apply(data, transformedDataReturnMat, eigvalReturnVector, eigvecReturnMat, newDimension);
        }
    }
    else if (strcmp(kernel, "hyptan") == 0)
    {
        HyperbolicTangentKernel htk(scale, offset);
        if(useNystroem == 1)
        {
            if(!useNystroemKernelPCA<HyperbolicTangentKernel>(nystroemMethod, htk, (centerTranformedData == 1), data, transformedDataReturnMat, eigvalReturnVector, eigvecReturnMat, newDimension))
                return;
        }
        else
        {
            KernelPCA<HyperbolicTangentKernel>(htk, (centerTranformedData == 1))
            .Apply(data, transformedDataReturnMat, eigvalReturnVector, eigvecReturnMat, newDimension);
        }
    }
    else
    {
        raisePrologDomainExeption(kernel, 1, "The given Kernel is unkown!", "kernel_pca");
        return;
    }

    if (newDimension < eigvecReturnMat.n_rows && newDimension > 0 && !useNystroem)
    transformedDataReturnMat.shed_rows(newDimension, transformedDataReturnMat.n_rows - 1);
    else
    {
        transformedDataReturnMat.shed_rows(data.n_rows, transformedDataReturnMat.n_rows - 1);
    }

    if(isnan(transformedDataReturnMat[0]))
    {
        raisePrologSystemExeption("Matrix contains nan!");
        return;
    }


    // return the transformedData Matrix
    returnMatrixInformation(transformedDataReturnMat, transformedDataMatArr, transformedDataMatColNum, transformedDataMatRowNum);
    
    // return the eigval Vector
    returnVectorInformation(eigvalReturnVector, eigvalArr, eigvalArrSize);

    // return the eigvec Matrix
    returnMatrixInformation(eigvecReturnMat, eigvecMatArr, eigvecMatColNum, eigvecMatRowNum);
}
