
:- module(kernel_pca, [kernel_pca/16]).

%% requirements of library(struct)
:- load_files(library(str_decl),
        [when(compile_time), if(changed)]).

%% needed for using the array type
:- use_module(library(structs)).
:- use_module('../../helper_files/helper.pl').

%% type definitions for the float array
:- foreign_type
        float32          = float_32,
        float_array      = array(float32).

%% definitions for the connected function


%% --Input--
%%              string  kernel                  "linear","gaussian","polynomial","hyptan","laplacian","epanechnikov","cosine",
%%              bool    centerTranformedData    => (1)true / (0)false,
%%              bool    useNystroem              => (1)true / (0)false,
%%              string  nystroemMethod          "kmeans", "ordered", "random",
%%              float32 degree                  needed by polynomial,
%%              float32 offset                  needed by polynomial, hyptan,
%%              float32 bandwidth               needed by gaussian, epanechnikov, laplacian,
%%              float32 scale                   needed by hyptan,
%%              mat     data
%%              int     newDimension
%%
%% --Output--
%%              mat     transformedData,
%%              vec     eigenValues,
%%              mat     eigenVectores
%%
%% --Description--
%%              This program performs Kernel Principal Components Analysis (KPCA) on the specified dataset with the specified kernel. This will transform the data onto the kernel principal components, and optionally reduce the dimensionality by ignoring the kernel principal components with the smallest eigenvalues.
%%
kernel_pca(Kernel, CenterTransformedData, UseNystroem, NystroemMethod, Degree, Offset, Bandwidth, Scale, DataList, DataRows,
                  TransformedList, TDataCols,
                  EigValList,
                  EigVecList, EigVecCols,
                  NewDimension) :-
        Bandwidth > 0.0,
        NewDimension >= 0,
        Bandwidth =< DataRows,
        convert_list_to_float_array(DataList, DataRows, array(Xsize, Xrownum, X)),
        kernel_pcaI(Kernel, CenterTransformedData, UseNystroem, NystroemMethod, Degree, Offset, Bandwidth, Scale,
                    X, Xsize, Xrownum,
                    TData, TDataCols, TDataRows,
                    EigVal, EigValsize,
                    EigVec, EigVecCols, EigVecRows,
                    NewDimension),
        convert_float_array_to_2d_list(TData, TDataCols, TDataRows, TransformedList),
        convert_float_array_to_list(EigVal, EigValsize, EigValList),
        convert_float_array_to_2d_list(EigVec, EigVecCols, EigVecRows, EigVecList).

foreign(kernel_pca, c, kernel_pcaI(     +string, +integer, +integer, +string,
                                        +float32, +float32, +float32, +float32,
                                        +pointer(float_array), +integer, +integer,
                                        -pointer(float_array), -integer, -integer,
                                        -pointer(float_array), -integer,
                                        -pointer(float_array), -integer, -integer,
                                        +integer)).


%% Defines the functions that get connected from main.cpp
foreign_resource(kernel_pca, [kernel_pca]).

:- load_foreign_resource(kernel_pca).
