
:- module(kernel_pca_tests, [run_kernel_pca_tests/0]).

:- use_module(library(plunit)).

:- use_module(kernel_pca).
:- use_module('../../helper_files/helper.pl').


%%
%% TESTING predicate kernel_pca/16
%%
:- begin_tests(kernel_pca).      

%% Failure Tests
              

test(kernel_pca_Init_Wrong_Kernel, [error(domain_error('The given Kernel is unkown!' , wrongType), _)]) :-
        kernel_pca(wrongType, 0, 0, kmeans, 1.0, 0.0, 1.0, 1.0, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _, _, _, 0).

test(kernel_pca_Init_Wrong_NystroemMethod, [error(domain_error('The given NystroemMethod is unknown!' , wrongType), _)]) :-
        kernel_pca(linear, 0, 1, wrongType, 1.0, 0.0, 1.0, 1.0, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _, _, _, 0).

test(kernel_pca_Zero_Bandwidth, fail) :-
        kernel_pca(linear, 0, 0, kmeans, 1.0, 0.0, 0.0, 1.0, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _, _, _, 0).
        

%% Successful Tests

test(kernel_pca_Linear_Random) :-
        kernel_pca(linear, 0, 0, random, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, TransformedList, _, EigValList, EigVecList, _, 0),
        print('\nTransformedData: '),
        print(TransformedList),
        print('\nEigenValues: '),
        print(EigValList),
        print('\nEigenVectors: '),
        print(EigVecList).

test(kernel_pca_Linear_Ordered) :-
        kernel_pca(linear, 0, 0, ordered, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, TransformedList, _, EigValList, EigVecList, _, 0),
        print('\nTransformedData: '),
        print(TransformedList),
        print('\nEigenValues: '),
        print(EigValList),
        print('\nEigenVectors: '),
        print(EigVecList).

test(kernel_pca_Linear_Kmeans) :-
        kernel_pca(linear, 0, 0, kmeans, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, TransformedList, _, EigValList, EigVecList, _, 0),
        print('\nTransformedData: '),
        print(TransformedList),
        print('\nEigenValues: '),
        print(EigValList),
        print('\nEigenVectors: '),
        print(EigVecList).

test(kernel_pca_Gaussian) :-
        kernel_pca(gaussian, 0, 0, random, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _, _, _, 0),
        kernel_pca(gaussian, 0, 0, ordered, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _, _, _, 0),
        kernel_pca(gaussian, 0, 0, kmeans, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, TransformedList, _, EigValList, EigVecList, _, 0),
        print('\nTransformedData: '),
        print(TransformedList),
        print('\nEigenValues: '),
        print(EigValList),
        print('\nEigenVectors: '),
        print(EigVecList).

test(kernel_pca_Polynomial) :-
        kernel_pca(polynomial, 0, 0, random, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _, _, _, 0),
        kernel_pca(polynomial, 0, 0, ordered, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _, _, _, 0),
        kernel_pca(polynomial, 0, 0, kmeans, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, TransformedList, _, EigValList, EigVecList, _, 0),
        print('\nTransformedData: '),
        print(TransformedList),
        print('\nEigenValues: '),
        print(EigValList),
        print('\nEigenVectors: '),
        print(EigVecList).

test(kernel_pca_Hyptan, [error(_, system_error('Matrix contains nan!'))]) :-
        kernel_pca(hyptan, 0, 0, random, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _, _, _, 0),
        kernel_pca(hyptan, 0, 0, ordered, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _, _, _, 0),
        kernel_pca(hyptan, 0, 0, kmeans, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, TransformedList, _, EigValList, EigVecList, _, 0),
        print('\nTransformedData: '),
        print(TransformedList),
        print('\nEigenValues: '),
        print(EigValList),
        print('\nEigenVectors: '),
        print(EigVecList).

test(kernel_pca_Laplacian) :-
        kernel_pca(laplacian, 0, 0, random, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _, _, _, 0),
        kernel_pca(laplacian, 0, 0, ordered, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _, _, _, 0),
        kernel_pca(laplacian, 0, 0, kmeans, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, TransformedList, _, EigValList, EigVecList, _, 0),
        print('\nTransformedData: '),
        print(TransformedList),
        print('\nEigenValues: '),
        print(EigValList),
        print('\nEigenVectors: '),
        print(EigVecList).

test(kernel_pca_Epanechnikov) :-
        kernel_pca(epanechnikov, 0, 0, random, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _, _, _, 0),
        kernel_pca(epanechnikov, 0, 0, ordered, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _, _, _, 0),
        kernel_pca(epanechnikov, 0, 0, kmeans, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, TransformedList, _, EigValList, EigVecList, _, 0),
        print('\nTransformedData: '),
        print(TransformedList),
        print('\nEigenValues: '),
        print(EigValList),
        print('\nEigenVectors: '),
        print(EigVecList).

test(kernel_pca_Cosine) :-
        kernel_pca(cosine, 0, 0, random, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _, _, _, 0),
        kernel_pca(cosine, 0, 0, ordered, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _, _, _, 0),
        kernel_pca(cosine, 0, 0, kmeans, 1.0, 0.0, 1.0, 1.0,
                   [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, TransformedList, _, EigValList, EigVecList, _, 0),
        print('\nTransformedData: '),
        print(TransformedList),
        print('\nEigenValues: '),
        print(EigValList),
        print('\nEigenVectors: '),
        print(EigVecList).


test(kernel_pca_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        kernel_pca(linear, 0, 0, kmeans, 1.0, 0.0, 1.0, 1.0, Data, 4, TransformedList, _, EigValList, EigVecList, _, 0),
        print('\nTransformedData: '),
        print(TransformedList),
        print('\nEigenValues: '),
        print(EigValList),
        print('\nEigenVectors: '),
        print(EigVecList).

test(kernel_pca_CSV_Input_Same_NewDims) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        kernel_pca(linear, 0, 0, kmeans, 1.0, 0.0, 1.0, 1.0, Data, 4, TransformedList, _, EigValList, EigVecList, _, 4),
        print('\nTransformedData: '),
        print(TransformedList),
        print('\nEigenValues: '),
        print(EigValList),
        print('\nEigenVectors: '),
        print(EigVecList).

test(kernel_pca_CSV_Input_Smaller_NewDims) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        kernel_pca(linear, 0, 0, kmeans, 1.0, 0.0, 1.0, 1.0, Data, 4, TransformedList, _, EigValList, EigVecList, _, 2),
        print('\nTransformedData: '),
        print(TransformedList),
        print('\nEigenValues: '),
        print(EigValList),
        print('\nEigenVectors: '),
        print(EigVecList).

:- end_tests(kernel_pca).

run_kernel_pca_tests :-
        run_tests.

