
:- module(kfn, [        kfn_initAndBuildModel/7,
                        kfn_searchWithQuery/7,
                        kfn_searchNoQuery/5]).

%% requirements of library(struct)
:- load_files(library(str_decl),
        [when(compile_time), if(changed)]).

%% needed for using the array type
:- use_module(library(structs)).
:- use_module('../../helper_files/helper.pl').

%% type definitions for the float array
:- foreign_type
        float32          = float_32,
        float_array      = array(float32).

%% definitions for the connected function



%% --Input--
%%              string  treeType        => "kd", "vp", "rp", "max_rp", "ub", "cover", "r", "r_star", "x", "ball", "hilbert_r", "r_plus", "r_plus_plus", "oct",
%%              string  searchMode      => "naive", "single_tree", "dual_tree", "greedy",
%%              bool    randomBasis     => (1)true / (0)false,
%%              int     leafSize        => 20,
%%              float32 epsilon         => 0.0,
%%              mat     referenceSet
%%
%% --Output--
%%
%% --Description--
%%              Initialize the Model and build it.
%%
kfn_initAndBuildModel(TreeType, SearchMode, RandomBasis, LeafSize, Epsilon, ReferenceList, ReferenceRows) :-
        LeafSize >= 1,
        Epsilon >= 0,
        convert_list_to_float_array(ReferenceList, ReferenceRows, array(Xsize, Xrownum, X)),
        initAndBuildModelI(TreeType, SearchMode, RandomBasis, LeafSize, Epsilon, X, Xsize, Xrownum).

foreign(initAndBuildModel, c, initAndBuildModelI(+string, +string,
                                                +integer, +integer, +float32,
                                                +pointer(float_array), +integer, +integer)).



%% --Input--
%%              mat     querySet,
%%              int     k
%%
%% --Output--
%%              mat     neighbors,
%%              mat     distances
%%
%% --Description--
%%              Perform neighbor search on the queryset.
%%
kfn_searchWithQuery(QueryList, QueryRows, K, NeighborsList, YCols, DistancesList, ZCols) :-
        K > 0,
        convert_list_to_float_array(QueryList, QueryRows, array(Xsize, Xrownum, X)),
        searchWithQueryI(X, Xsize, Xrownum, K, Y, YCols, YRows, Z, ZCols, ZRows),
        convert_float_array_to_2d_list(Y, YCols, YRows, NeighborsList),
        convert_float_array_to_2d_list(Z, ZCols, ZRows, DistancesList).

foreign(searchWithQuery, c, searchWithQueryI(   +pointer(float_array), +integer, +integer,
                                                +integer,
                                                -pointer(float_array), -integer, -integer,
                                                -pointer(float_array), -integer, -integer)).



%% --Input--
%%              int     k
%%
%% --Output--
%%              mat     neighbors,
%%              mat     distances
%%
%% --Description--
%%              Perform monochromatic neighbor search.
%%
kfn_searchNoQuery(K, NeighborsList, YCols, DistancesList, ZCols) :-
        K > 0,
        searchNoQueryI(K, Y, YCols, YRows, Z, ZCols, ZRows),
        convert_float_array_to_2d_list(Y, YCols, YRows, NeighborsList),
        convert_float_array_to_2d_list(Z, ZCols, ZRows, DistancesList).

foreign(searchNoQuery, c, searchNoQueryI(       +integer,
                                                -pointer(float_array), -integer, -integer,
                                                -pointer(float_array), -integer, -integer)).


%% Defines the functions that get connected from kfn.cpp
foreign_resource(kfn, [ initAndBuildModel,
                        searchWithQuery,
                        searchNoQuery]).

:- load_foreign_resource(kfn).
