#include <sicstus/sicstus.h>
/* ex_glue.h is generated by splfr from the foreign/[2,3] facts.
   Always include the glue header in your foreign resource code.
*/
#include "kmeans_glue.h"
#include <mlpack/methods/kmeans/kmeans.hpp>

// Include initialization strategies.
#include <mlpack/methods/kmeans/sample_initialization.hpp>
#include <mlpack/methods/kmeans/random_partition.hpp>

// Include empty cluster policies.
#include <mlpack/methods/kmeans/max_variance_new_cluster.hpp>
#include <mlpack/methods/kmeans/kill_empty_clusters.hpp>
#include <mlpack/methods/kmeans/allow_empty_clusters.hpp>
 
// Include Lloyd step types.
#include <mlpack/methods/kmeans/dual_tree_kmeans.hpp>
#include <mlpack/methods/kmeans/elkan_kmeans.hpp>
#include <mlpack/methods/kmeans/hamerly_kmeans.hpp>
#include <mlpack/methods/kmeans/pelleg_moore_kmeans.hpp>
#include <mlpack/core.hpp>

// including helper functions for converting between arma structures and arrays
#include "../../helper_files/helper.hpp"

// some of the most used namespaces
using namespace arma;
using namespace mlpack;
using namespace std;
using namespace mlpack::kmeans;
using namespace mlpack::metric;

// internal Funktion 
// initiats KMeans with the given Class Template and inputs the params and returns the results
template< template<class,class> class LloydStepType=NaiveKMeans>
void InitAndClusterKMeans(SP_integer maxIterations, 
							const char *initialPartition, 
							const char *emptyCluster, 
							float *dataMatArr, SP_integer dataMatSize, SP_integer dataMatRowNum, 
							SP_integer clusters,
							float **assignmentsArr, SP_integer *assignmentsArrSize, 
							float **centroidsMatArr, SP_integer *centroidsMatColNum, SP_integer *centroidsMatRowNum)
{
	// convert the Prolog array to arma::mat
	mat data = convertArrayToMat(dataMatArr, dataMatSize, dataMatRowNum);

	// create the ReturnVector
	arma::Row<size_t> assignmentsReturnVector;	
	// create the ReturnMat
	mat centroidsReturnMat;


	// Initialize the model with the given Policies and cluster the data with Kmeans

	if (strcmp(initialPartition, "sampleInitialization") == 0)
	{
		// SampleInitialization
		if (strcmp(emptyCluster, "maxVarianceNewCluster") == 0)
		{
			// MaxVarianceNewCluster
			KMeans<EuclideanDistance, SampleInitialization, MaxVarianceNewCluster, LloydStepType>(maxIterations).Cluster(data, clusters, assignmentsReturnVector, centroidsReturnMat, false, false);
		}
		else if (strcmp(emptyCluster, "killEmptyCluster") == 0)
		{
			// KillEmptyClusters
			KMeans<EuclideanDistance, SampleInitialization, KillEmptyClusters, LloydStepType>(maxIterations).Cluster(data, clusters, assignmentsReturnVector, centroidsReturnMat, false, false);
		}
		else if (strcmp(emptyCluster, "allowEmptyCluster") == 0)
		{
			// AllowEmptyClusters
			KMeans<EuclideanDistance, SampleInitialization, AllowEmptyClusters, LloydStepType>(maxIterations).Cluster(data, clusters, assignmentsReturnVector, centroidsReturnMat, false, false);
		}
		else
		{
			raisePrologDomainExeption(emptyCluster, 3, "The given EmptyClusterPolicy is unknown!", "KMeans");
			return;
		}
	}
	else if (strcmp(initialPartition, "randomPartition") == 0)
	{
		// RandomPartition
		if (strcmp(emptyCluster, "maxVarianceNewCluster") == 0)
		{
			// MaxVarianceNewCluster
			KMeans<EuclideanDistance, RandomPartition, MaxVarianceNewCluster, LloydStepType>(maxIterations).Cluster(data, clusters, assignmentsReturnVector, centroidsReturnMat, false, false);
		}
		else if (strcmp(emptyCluster, "killEmptyCluster") == 0)
		{
			// KillEmptyClusters
			KMeans<EuclideanDistance, RandomPartition, KillEmptyClusters, LloydStepType>(maxIterations).Cluster(data, clusters, assignmentsReturnVector, centroidsReturnMat, false, false);
		}
		else if (strcmp(emptyCluster, "allowEmptyCluster") == 0)
		{
			// AllowEmptyClusters
			KMeans<EuclideanDistance, RandomPartition, AllowEmptyClusters, LloydStepType>(maxIterations).Cluster(data, clusters, assignmentsReturnVector, centroidsReturnMat, false, false);
		}
		else
		{
			raisePrologDomainExeption(emptyCluster, 3, "The given EmptyClusterPolicy is unknown!", "KMeans");
			return;
		}
	}
	else
	{
		raisePrologDomainExeption(initialPartition, 2, "The given InitialPartitionPolicy is unknown!", "KMeans");
		return;
	}
	

	// return the Vector
	returnVectorInformation(assignmentsReturnVector, assignmentsArr, assignmentsArrSize);
	// return the Matrix
	returnMatrixInformation(centroidsReturnMat, centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
}


// input: 	const size_t 					maxIterations = 1000,
//			const InitialPartitionPolicy 	partitioner = InitialPartitionPolicy(),
//			const EmptyClusterPolicy 		emptyClusterAction = EmptyClusterPolicy()
//
//			const MatType & 		data,
//			const size_t 			clusters,
//			arma::Row< size_t > & 	assignments,
//			arma::mat & 			centroids,
//			const bool 				initialAssignmentGuess = false,
//			const bool 				initialCentroidGuess = false 
// output: 
// description: 
//			Runs kmeans with naive as the algorithm for the Lloyd iteration.
//
void naiveKMeans(SP_integer maxIterations, 
					const char *initialPartition, 
					const char *emptyCluster, 
					float *dataMatArr, SP_integer dataMatSize, SP_integer dataMatRowNum, 
					SP_integer clusters,
					float **assignmentsArr, SP_integer *assignmentsArrSize, 
					float **centroidsMatArr, SP_integer *centroidsMatColNum, SP_integer *centroidsMatRowNum)
{
	InitAndClusterKMeans<NaiveKMeans>(maxIterations, initialPartition, emptyCluster, dataMatArr, dataMatSize, dataMatRowNum, clusters,
					 assignmentsArr, assignmentsArrSize, centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
}


// input: 	const size_t 					maxIterations = 1000,
//			const InitialPartitionPolicy 	partitioner = InitialPartitionPolicy(),
//			const EmptyClusterPolicy 		emptyClusterAction = EmptyClusterPolicy()
//
//			const MatType & 		data,
//			const size_t 			clusters,
//			arma::Row< size_t > & 	assignments,
//			arma::mat & 			centroids,
//			const bool 				initialAssignmentGuess = false,
//			const bool 				initialCentroidGuess = false 
// output: 
// description: 
//			Runs kmeans with dualTree as the algorithm for the Lloyd iteration.
//
void dualTreeKMeans(SP_integer maxIterations, 
					const char *initialPartition, 
					const char *emptyCluster, 
					float *dataMatArr, SP_integer dataMatSize, SP_integer dataMatRowNum, 
					SP_integer clusters,
					float **assignmentsArr, SP_integer *assignmentsArrSize, 
					float **centroidsMatArr, SP_integer *centroidsMatColNum, SP_integer *centroidsMatRowNum)
{
	InitAndClusterKMeans<DualTreeKMeans>(maxIterations, initialPartition, emptyCluster, dataMatArr, dataMatSize, dataMatRowNum, clusters,
					 assignmentsArr, assignmentsArrSize, centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
}


// input: 	const size_t 					maxIterations = 1000,
//			const InitialPartitionPolicy 	partitioner = InitialPartitionPolicy(),
//			const EmptyClusterPolicy 		emptyClusterAction = EmptyClusterPolicy()
//
//			const MatType & 		data,
//			const size_t 			clusters,
//			arma::Row< size_t > & 	assignments,
//			arma::mat & 			centroids,
//			const bool 				initialAssignmentGuess = false,
//			const bool 				initialCentroidGuess = false 
// output: 
// description: 
//			Runs kmeans with elkan as the algorithm for the Lloyd iteration.
//
void elkanKMeans(SP_integer maxIterations, 
					const char *initialPartition, 
					const char *emptyCluster, 
					float *dataMatArr, SP_integer dataMatSize, SP_integer dataMatRowNum, 
					SP_integer clusters,
					float **assignmentsArr, SP_integer *assignmentsArrSize, 
					float **centroidsMatArr, SP_integer *centroidsMatColNum, SP_integer *centroidsMatRowNum)
{
	InitAndClusterKMeans<ElkanKMeans>(maxIterations, initialPartition, emptyCluster, dataMatArr, dataMatSize, dataMatRowNum, clusters,
					 assignmentsArr, assignmentsArrSize, centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
}


// input: 	const size_t 					maxIterations = 1000,
//			const InitialPartitionPolicy 	partitioner = InitialPartitionPolicy(),
//			const EmptyClusterPolicy 		emptyClusterAction = EmptyClusterPolicy()
//
//			const MatType & 		data,
//			const size_t 			clusters,
//			arma::Row< size_t > & 	assignments,
//			arma::mat & 			centroids,
//			const bool 				initialAssignmentGuess = false,
//			const bool 				initialCentroidGuess = false 
// output: 
// description: 
//			Runs kmeans with hamerly as the algorithm for the Lloyd iteration.
//
void hamerlyKMeans(SP_integer maxIterations, 
					const char *initialPartition, 
					const char *emptyCluster, 
					float *dataMatArr, SP_integer dataMatSize, SP_integer dataMatRowNum, 
					SP_integer clusters,
					float **assignmentsArr, SP_integer *assignmentsArrSize, 
					float **centroidsMatArr, SP_integer *centroidsMatColNum, SP_integer *centroidsMatRowNum)
{
	InitAndClusterKMeans<HamerlyKMeans>(maxIterations, initialPartition, emptyCluster, dataMatArr, dataMatSize, dataMatRowNum, clusters,
					 assignmentsArr, assignmentsArrSize, centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
}


// input: 	const size_t 					maxIterations = 1000,
//			const InitialPartitionPolicy 	partitioner = InitialPartitionPolicy(),
//			const EmptyClusterPolicy 		emptyClusterAction = EmptyClusterPolicy()
//
//			const MatType & 		data,
//			const size_t 			clusters,
//			arma::Row< size_t > & 	assignments,
//			arma::mat & 			centroids,
//			const bool 				initialAssignmentGuess = false,
//			const bool 				initialCentroidGuess = false 
// output: 
// description: 
//			Runs kmeans with pelleg Moore as the algorithm for the Lloyd iteration.
//
void pellegMooreKMeans(SP_integer maxIterations, 
						const char *initialPartition, 
						const char *emptyCluster, 
						float *dataMatArr, SP_integer dataMatSize, SP_integer dataMatRowNum, 
						SP_integer clusters,
						float **assignmentsArr, SP_integer *assignmentsArrSize, 
						float **centroidsMatArr, SP_integer *centroidsMatColNum, SP_integer *centroidsMatRowNum)
{
	InitAndClusterKMeans<PellegMooreKMeans>(maxIterations, initialPartition, emptyCluster, dataMatArr, dataMatSize, dataMatRowNum, clusters,
					 assignmentsArr, assignmentsArrSize, centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
}
