
:- module(kmeans, [     naiveKMeans/9,
                        dualTreeKMeans/9, 
                        elkanKMeans/9, 
                        hamerlyKMeans/9, 
                        pellegMooreKMeans/9]).

%% requirements of library(struct)
:- load_files(library(str_decl),
                [when(compile_time), if(changed)]).

%% needed for using the array type
:- use_module(library(structs)).
:- use_module('../../helper_files/helper.pl').

%% type definitions for the float array
:- foreign_type
        float32          = float_32,
        float_array      = array(float32).

%% definitions for the connected function


%% --Input--
%%              int     maxIterations           => 1000,
%%              string  initialPartition        => "sampleInitialization", "randomPartition",
%%              string  emptyCluster            => "maxVarianceNewCluster", "killEmptyCluster", "allowEmptyCluster",
%%              mat     data,
%%              int     clusters                => 0
%%
%% --Output--
%%              vec     assignments,
%%              mat     centroids
%%
%% --Description--
%%              Runs kmeans with naive as the algorithm for the Lloyd iteration.
%%
naiveKMeans(MaxIterations, InitialPartition, EmptyCluster, DataList, DataRows, Clusters, AssignmentsList, CentroidsList, ZCols) :-
        MaxIterations >= 0,
        Clusters > 0,
        convert_list_to_float_array(DataList, DataRows, array(Xsize, Xrows, X)),
        naiveKMeansI(MaxIterations, InitialPartition, EmptyCluster, X, Xsize, Xrows, Clusters, Y, Ysize, Z, ZCols, ZRows),
        convert_float_array_to_list(Y, Ysize, AssignmentsList),
        convert_float_array_to_2d_list(Z, ZCols, ZRows, CentroidsList).

foreign(naiveKMeans, c, naiveKMeansI(   +integer, +string, +string, 
                                        +pointer(float_array), +integer, +integer, 
                                        +integer,
                                        -pointer(float_array), -integer, 
                                        -pointer(float_array), -integer, -integer)).


%% --Input--
%%              int     maxIterations           => 1000,
%%              string  initialPartition        => "sampleInitialization", "randomPartition",
%%              string  emptyCluster            => "maxVarianceNewCluster", "killEmptyCluster", "allowEmptyCluster",
%%              mat     data,
%%              int     clusters                => 0
%%
%% --Output--
%%              vec     assignments,
%%              mat     centroids
%%
%% --Description--
%%              Runs kmeans with dualTree as the algorithm for the Lloyd iteration.
%%
dualTreeKMeans(MaxIterations, InitialPartition, EmptyCluster, DataList, DataRows, Clusters, AssignmentsList, CentroidsList, ZCols) :-
        MaxIterations >= 0,
        Clusters > 0,
        convert_list_to_float_array(DataList, DataRows, array(Xsize, Xrows, X)),
        dualTreeKMeansI(MaxIterations, InitialPartition, EmptyCluster, X, Xsize, Xrows, Clusters, Y, Ysize, Z, ZCols, ZRows),
        convert_float_array_to_list(Y, Ysize, AssignmentsList),
        convert_float_array_to_2d_list(Z, ZCols, ZRows, CentroidsList).

foreign(dualTreeKMeans, c, dualTreeKMeansI(     +integer, +string, +string, 
                                                +pointer(float_array), +integer, +integer, 
                                                +integer,
                                                -pointer(float_array), -integer, 
                                                -pointer(float_array), -integer, -integer)).


%% --Input--
%%              int     maxIterations           => 1000,
%%              string  initialPartition        => "sampleInitialization", "randomPartition",
%%              string  emptyCluster            => "maxVarianceNewCluster", "killEmptyCluster", "allowEmptyCluster",
%%              mat     data,
%%              int     clusters                => 0
%%
%% --Output--
%%              vec     assignments,
%%              mat     centroids
%%
%% --Description--
%%              Runs kmeans with elkan as the algorithm for the Lloyd iteration.
%%
elkanKMeans(MaxIterations, InitialPartition, EmptyCluster, DataList, DataRows, Clusters, AssignmentsList, CentroidsList, ZCols) :-
        MaxIterations >= 0,
        Clusters > 0,
        convert_list_to_float_array(DataList, DataRows, array(Xsize, Xrows, X)),
        elkanKMeansI(MaxIterations, InitialPartition, EmptyCluster, X, Xsize, Xrows, Clusters, Y, Ysize, Z, ZCols, ZRows),
        convert_float_array_to_list(Y, Ysize, AssignmentsList),
        convert_float_array_to_2d_list(Z, ZCols, ZRows, CentroidsList).

foreign(elkanKMeans, c, elkanKMeansI(   +integer, +string, +string, 
                                        +pointer(float_array), +integer, +integer, 
                                        +integer,
                                        -pointer(float_array), -integer, 
                                        -pointer(float_array), -integer, -integer)).


%% --Input--
%%              int     maxIterations           => 1000,
%%              string  initialPartition        => "sampleInitialization", "randomPartition",
%%              string  emptyCluster            => "maxVarianceNewCluster", "killEmptyCluster", "allowEmptyCluster",
%%              mat     data,
%%              int     clusters                => 0
%%
%% --Output--
%%              vec     assignments,
%%              mat     centroids
%%
%% --Description--
%%              Runs kmeans with hamerly as the algorithm for the Lloyd iteration.
%%
hamerlyKMeans(MaxIterations, InitialPartition, EmptyCluster, DataList, DataRows, Clusters, AssignmentsList, CentroidsList, ZCols) :-
        MaxIterations >= 0,
        Clusters > 0,
        convert_list_to_float_array(DataList, DataRows, array(Xsize, Xrows, X)),
        hamerlyKMeansI(MaxIterations, InitialPartition, EmptyCluster, X, Xsize, Xrows, Clusters, Y, Ysize, Z, ZCols, ZRows),
        convert_float_array_to_list(Y, Ysize, AssignmentsList),
        convert_float_array_to_2d_list(Z, ZCols, ZRows, CentroidsList).

foreign(hamerlyKMeans, c, hamerlyKMeansI(       +integer, +string, +string, 
                                                +pointer(float_array), +integer, +integer, 
                                                +integer,
                                                -pointer(float_array), -integer, 
                                                -pointer(float_array), -integer, -integer)).


%% --Input--
%%              int     maxIterations           => 1000,
%%              string  initialPartition        => "sampleInitialization", "randomPartition",
%%              string  emptyCluster            => "maxVarianceNewCluster", "killEmptyCluster", "allowEmptyCluster",
%%              mat     data,
%%              int     clusters                => 0
%%
%% --Output--
%%              vec     assignments,
%%              mat     centroids
%%
%% --Description--
%%              Runs kmeans with pelleg Moore as the algorithm for the Lloyd iteration.
%%
pellegMooreKMeans(MaxIterations, InitialPartition, EmptyCluster, DataList, DataRows, Clusters, AssignmentsList, CentroidsList, ZCols) :-
        MaxIterations >= 0,
        Clusters > 0,
        convert_list_to_float_array(DataList, DataRows, array(Xsize, Xrows, X)),
        pellegMooreKMeansI(MaxIterations, InitialPartition, EmptyCluster, X, Xsize, Xrows, Clusters, Y, Ysize, Z, ZCols, ZRows),
        convert_float_array_to_list(Y, Ysize, AssignmentsList),
        convert_float_array_to_2d_list(Z, ZCols, ZRows, CentroidsList).

foreign(pellegMooreKMeans, c, pellegMooreKMeansI(       +integer, +string, +string, 
                                                        +pointer(float_array), +integer, +integer, 
                                                        +integer,
                                                        -pointer(float_array), -integer, 
                                                        -pointer(float_array), -integer, -integer)).


%% Defines the functions that get connected from main.cpp
foreign_resource(kmeans, [      naiveKMeans,
                                dualTreeKMeans, 
                                elkanKMeans, 
                                hamerlyKMeans, 
                                pellegMooreKMeans]).

:- load_foreign_resource(kmeans).
