
:- module(kmeans_tests, [run_kmeans_tests/0]).

:- use_module(library(plunit)).

:- use_module(kmeans).
:- use_module('../../helper_files/helper.pl').


%%
%% TESTING predicate naiveKMeans/9
%%
:- begin_tests(naiveKMeans).      

%% Failure Tests

test(naiveKMeans_Negative_MaxIterations, fail) :-
        naiveKMeans(-1000, sampleInitialization, randomPartition, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, _, _, _).

test(naiveKMeans_Wrong_InitialPartition_Input, [error(domain_error('The given InitialPartitionPolicy is unknown!' , wrongInput), _)]) :-
        naiveKMeans(1000, wrongInput, randomPartition, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, _, _, _).

test(naiveKMeans_Wrong_EmptyCluster_Input, [error(domain_error('The given EmptyClusterPolicy is unknown!' , wrongInput), _)]) :-
        naiveKMeans(1000, sampleInitialization, wrongInput, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, _, _, _).

test(naiveKMeans_Negative_Clusters, fail) :-
        naiveKMeans(1000, sampleInitialization, randomPartition, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, -2, _, _, _).
        

%% Successful Tests

test(naiveKMeans_Sample_MaxVariance) :-
        naiveKMeans(1000, sampleInitialization, maxVarianceNewCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(naiveKMeans_Sample_KillEmpty) :-
        naiveKMeans(1000, sampleInitialization, killEmptyCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(naiveKMeans_Sample_AllowEmpty) :-
        naiveKMeans(1000, sampleInitialization, allowEmptyCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(naiveKMeans_Random_MaxVariance) :-
        naiveKMeans(1000, randomPartition, maxVarianceNewCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(naiveKMeans_Random_KillEmpty) :-
        naiveKMeans(1000, randomPartition, killEmptyCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(naiveKMeans_Random_AllowEmpty) :-
        naiveKMeans(1000, randomPartition, allowEmptyCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).



test(naiveKMeans_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow, 30, Data),
        naiveKMeans(100, randomPartition, allowEmptyCluster, Data, 4, 3, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

:- end_tests(naiveKMeans).



%%
%% TESTING predicate dualTreeKMeans/9
%%
:- begin_tests(dualTreeKMeans).      

%% Failure Tests

test(dualTreeKMeans_Negative_MaxIterations, fail) :-
        dualTreeKMeans(-1000, sampleInitialization, randomPartition, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, _, _, _).

test(dualTreeKMeans_Wrong_InitialPartition_Input, [error(domain_error('The given InitialPartitionPolicy is unknown!' , wrongInput), _)]) :-
        dualTreeKMeans(1000, wrongInput, randomPartition, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, _, _, _).

test(dualTreeKMeans_Wrong_EmptyCluster_Input, [error(domain_error('The given EmptyClusterPolicy is unknown!' , wrongInput), _)]) :-
        dualTreeKMeans(1000, sampleInitialization, wrongInput, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, _, _, _).

test(dualTreeKMeans_Negative_Clusters, fail) :-
        dualTreeKMeans(1000, sampleInitialization, randomPartition, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, -2, _, _, _).
        

%% Successful Tests

test(dualTreeKMeans_Sample_MaxVariance) :-
        dualTreeKMeans(1000, sampleInitialization, maxVarianceNewCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(dualTreeKMeans_Sample_KillEmpty) :-
        dualTreeKMeans(1000, sampleInitialization, killEmptyCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(dualTreeKMeans_Sample_AllowEmpty) :-
        dualTreeKMeans(1000, sampleInitialization, allowEmptyCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(dualTreeKMeans_Random_MaxVariance) :-
        dualTreeKMeans(1000, randomPartition, maxVarianceNewCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(dualTreeKMeans_Random_KillEmpty) :-
        dualTreeKMeans(1000, randomPartition, killEmptyCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(dualTreeKMeans_Random_AllowEmpty) :-
        dualTreeKMeans(1000, randomPartition, allowEmptyCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).



test(dualTreeKMeans_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow, 30, Data),
        dualTreeKMeans(100, randomPartition, allowEmptyCluster, Data, 4, 3, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

:- end_tests(dualTreeKMeans).



%%
%% TESTING predicate elkanKMeans/9
%%
:- begin_tests(elkanKMeans).      

%% Failure Tests

test(elkanKMeans_Negative_MaxIterations, fail) :-
        elkanKMeans(-1000, sampleInitialization, randomPartition, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, _, _, _).

test(elkanKMeans_Wrong_InitialPartition_Input, [error(domain_error('The given InitialPartitionPolicy is unknown!' , wrongInput), _)]) :-
        elkanKMeans(1000, wrongInput, randomPartition, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, _, _, _).

test(elkanKMeans_Wrong_EmptyCluster_Input, [error(domain_error('The given EmptyClusterPolicy is unknown!' , wrongInput), _)]) :-
        elkanKMeans(1000, sampleInitialization, wrongInput, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, _, _, _).

test(elkanKMeans_Negative_Clusters, fail) :-
        elkanKMeans(1000, sampleInitialization, randomPartition, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, -2, _, _, _).
        

%% Successful Tests

test(elkanKMeans_Sample_MaxVariance) :-
        elkanKMeans(1000, sampleInitialization, maxVarianceNewCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(elkanKMeans_Sample_KillEmpty) :-
        elkanKMeans(1000, sampleInitialization, killEmptyCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(elkanKMeans_Sample_AllowEmpty) :-
        elkanKMeans(1000, sampleInitialization, allowEmptyCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(elkanKMeans_Random_MaxVariance) :-
        elkanKMeans(1000, randomPartition, maxVarianceNewCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(elkanKMeans_Random_KillEmpty) :-
        elkanKMeans(1000, randomPartition, killEmptyCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(elkanKMeans_Random_AllowEmpty) :-
        elkanKMeans(1000, randomPartition, allowEmptyCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).



test(elkanKMeans_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow, 30, Data),
        elkanKMeans(100, randomPartition, allowEmptyCluster, Data, 4, 3, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

:- end_tests(elkanKMeans).



%%
%% TESTING predicate hamerlyKMeans/9
%%
:- begin_tests(hamerlyKMeans).      

%% Failure Tests

test(hamerlyKMeans_Negative_MaxIterations, fail) :-
        hamerlyKMeans(-1000, sampleInitialization, randomPartition, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, _, _, _).

test(hamerlyKMeans_Wrong_InitialPartition_Input, [error(domain_error('The given InitialPartitionPolicy is unknown!' , wrongInput), _)]) :-
        hamerlyKMeans(1000, wrongInput, randomPartition, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, _, _, _).

test(hamerlyKMeans_Wrong_EmptyCluster_Input, [error(domain_error('The given EmptyClusterPolicy is unknown!' , wrongInput), _)]) :-
        hamerlyKMeans(1000, sampleInitialization, wrongInput, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, _, _, _).

test(hamerlyKMeans_Negative_Clusters, fail) :-
        hamerlyKMeans(1000, sampleInitialization, randomPartition, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, -2, _, _, _).
        

%% Successful Tests

test(hamerlyKMeans_Sample_MaxVariance) :-
        hamerlyKMeans(1000, sampleInitialization, maxVarianceNewCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(hamerlyKMeans_Sample_KillEmpty) :-
        hamerlyKMeans(1000, sampleInitialization, killEmptyCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(hamerlyKMeans_Sample_AllowEmpty) :-
        hamerlyKMeans(1000, sampleInitialization, allowEmptyCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(hamerlyKMeans_Random_MaxVariance) :-
        hamerlyKMeans(1000, randomPartition, maxVarianceNewCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(hamerlyKMeans_Random_KillEmpty) :-
        hamerlyKMeans(1000, randomPartition, killEmptyCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(hamerlyKMeans_Random_AllowEmpty) :-
        hamerlyKMeans(1000, randomPartition, allowEmptyCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).



test(hamerlyKMeans_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow, 30, Data),
        hamerlyKMeans(100, randomPartition, allowEmptyCluster, Data, 4, 3, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

:- end_tests(hamerlyKMeans).



%%
%% TESTING predicate pellegMooreKMeans/9
%%
:- begin_tests(pellegMooreKMeans).      

%% Failure Tests

test(pellegMooreKMeans_Negative_MaxIterations, fail) :-
        pellegMooreKMeans(-1000, sampleInitialization, randomPartition, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, _, _, _).

test(pellegMooreKMeans_Wrong_InitialPartition_Input, [error(domain_error('The given InitialPartitionPolicy is unknown!' , wrongInput), _)]) :-
        pellegMooreKMeans(1000, wrongInput, randomPartition, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, _, _, _).

test(pellegMooreKMeans_Wrong_EmptyCluster_Input, [error(domain_error('The given EmptyClusterPolicy is unknown!' , wrongInput), _)]) :-
        pellegMooreKMeans(1000, sampleInitialization, wrongInput, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, _, _, _).

test(pellegMooreKMeans_Negative_Clusters, fail) :-
        pellegMooreKMeans(1000, sampleInitialization, randomPartition, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, -2, _, _, _).
        

%% Successful Tests

test(pellegMooreKMeans_Sample_MaxVariance) :-
        pellegMooreKMeans(1000, sampleInitialization, maxVarianceNewCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(pellegMooreKMeans_Sample_KillEmpty) :-
        pellegMooreKMeans(1000, sampleInitialization, killEmptyCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(pellegMooreKMeans_Sample_AllowEmpty) :-
        pellegMooreKMeans(1000, sampleInitialization, allowEmptyCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(pellegMooreKMeans_Random_MaxVariance) :-
        pellegMooreKMeans(1000, randomPartition, maxVarianceNewCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(pellegMooreKMeans_Random_KillEmpty) :-
        pellegMooreKMeans(1000, randomPartition, killEmptyCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

test(pellegMooreKMeans_Random_AllowEmpty) :-
        pellegMooreKMeans(1000, randomPartition, allowEmptyCluster, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).



test(pellegMooreKMeans_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow, 30, Data),
        pellegMooreKMeans(100, randomPartition, allowEmptyCluster, Data, 4, 3, AssignmentsList, CentroidsList, _),
        print('\nAssignments :'),
        print(AssignmentsList),
        print('\nCentroids :'),
        print(CentroidsList).

:- end_tests(pellegMooreKMeans).

run_kmeans_tests :-
        run_tests.

