
:- module(lars_tests, [run_lars_tests/0]).

:- use_module(library(plunit)).

:- use_module(library(structs)).

:- use_module(lars).
:- use_module('../../helper_files/helper.pl').


reset_Model_WithTrain :-
        lars_initAndTrainModel(0, 0.0, 0.0, 1.0e-16, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 0, _).



%%
%% TESTING predicate lars_initAndTrainModel/9
%%
:- begin_tests(lars_initAndTrainModel).      

%% Failure Tests
                                            
test(lars_initAndTrainModel_Negative_Tolerance, fail) :-
        lars_initAndTrainModel(0, 0.0, 0.0, -1.0e-16, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 0, _).

test(lars_initAndTrainModel_Too_Few_Labels, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        lars_initAndTrainModel(0, 0.0, 0.0, 1.0e-16, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], 0, _).

test(lars_initAndTrainModel_Too_Many_Labels, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        lars_initAndTrainModel(0, 0.0, 0.0, 1.0e-16, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], 0, _).


test(lars_initAndTrainModel_Wrongly_Transposed, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        lars_initAndTrainModel(0, 0.0, 0.0, 1.0e-16, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 1, _).
        

%% Successful Tests

test(lars_initAndTrainModel_Normal_Use) :-
        lars_initAndTrainModel(0, 0.0, 0.0, 1.0e-16, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 0, _).

test(lars_initAndTrainModel_Alternative_Use) :-
        lars_initAndTrainModel(1, -1.0, 1.0, 1.0e-16, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, [0,1,0], 0, _).

:- end_tests(lars_initAndTrainModel).



%%
%% TESTING predicate lars_activeSet/1
%%
:- begin_tests(lars_activeSet).      

%% Failure Tests
        

%% Successful Tests

test(lars_ActiveSet_Normal_Use) :-
        reset_Model_WithTrain,
        lars_activeSet(ActiveSetList),
        print('\nActive Set: '),
        print(ActiveSetList).

:- end_tests(lars_activeSet).



%%
%% TESTING predicate lars_beta/1
%%
:- begin_tests(lars_beta).      

%% Failure Tests
        

%% Successful Tests

test(lars_Beta_Normal_Use) :-
        reset_Model_WithTrain,
        lars_beta(BetaList),
        print('\nBeta : '),
        print(BetaList).

:- end_tests(lars_beta).



%%
%% TESTING predicate lars_betaPath/2
%%
:- begin_tests(lars_betaPath).      

%% Failure Tests
        

%% Successful Tests

test(lars_BetaPath_Normal_Use) :-
        reset_Model_WithTrain,
        lars_betaPath(BetaPathList, _),
        print('\nBeta List: '),
        print(BetaPathList).

:- end_tests(lars_betaPath).



%%
%% TESTING predicate lars_computeError/5
%%
:- begin_tests(lars_computeError).      

%% Failure Tests

test(lars_ComputeError_Too_Few_Labels, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model_WithTrain,
        lars_computeError([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], 1, _).

test(lars_ComputeError_Too_Many_Labels, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model_WithTrain,
        lars_computeError([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], 1, _).


%% Successful Tests

test(lars_ComputeError_Normal_Use) :-
        reset_Model_WithTrain,
        lars_computeError([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 1, Error),
        print('\nError : '),
        print(Error).

test(lars_ComputeError_Alternative_Use) :-
        reset_Model_WithTrain,
        lars_computeError([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 1, Error),
        print('\nError : '),
        print(Error).

test(lars_ComputeError_CSV_Input) :-
        lars_initAndTrainModel(0, 0.0, 0.0, 1.0e-16, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, [0,1,0], 0, _),
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        lars_computeError(Data, 4, [0,1,0,1,1,0,1,1,1,0], 1, Error),
        print('\nError : '),
        print(Error).

:- end_tests(lars_computeError).



%%
%% TESTING predicate lars_lambdaPath/1
%%
:- begin_tests(lars_lambdaPath).      

%% Failure Tests

%% Successful Tests

test(lars_LambdaPath_Normal_Use) :-
        reset_Model_WithTrain,
        lars_lambdaPath(LambdaPathList),
        print('\nLambda Path: '),
        print(LambdaPathList).

:- end_tests(lars_lambdaPath).



%%
%% TESTING predicate lars_matUtriCholFactor/2
%%
:- begin_tests(lars_matUtriCholFactor).      

%% Failure Tests

%% Successful Tests

test(lars_MatUtriCholFactor_Normal_Use) :-
        reset_Model_WithTrain,
        lars_matUtriCholFactor(FactorList, _),
        print('\nUtriCholFactor: '),
        print(FactorList).

:- end_tests(lars_matUtriCholFactor).



%%
%% TESTING predicate lars_predict/10
%%
:- begin_tests(lars_predict).      

%% Failure Tests

test(lars_Predict_Diffrent_Dims, [error(_,system_error('The given Datapoints have a diffrent Dimension than trained!'))]) :-
        reset_Model_WithTrain,
        lars_predict([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, _, 1).


%% Successful Tests

test(lars_Predict_Normal_Use) :-
        reset_Model_WithTrain,
        lars_predict([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, PredictList, 1),
        print('\nPrediction: '),
        print(PredictList).

test(lars_Predict_Alternative_Use) :-
        reset_Model_WithTrain,
        lars_predict([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, PredictList, 1),
        print('\nPrediction: '),
        print(PredictList).

test(lars_Predict_CSV_Input) :-
        lars_initAndTrainModel(0, 0.0, 0.0, 1.0e-16, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, [0,1,0], 0, _),
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        lars_predict(Data, 4, PredictList, 1),
        print('\nPrediction: '),
        print(PredictList).

:- end_tests(lars_predict).

run_lars_tests :-
        run_tests.
