
:- module(linear_SVM_tests, [run_linear_SVM_tests/0]).

:- use_module(library(plunit)).

:- use_module(linear_SVM).
:- use_module('../../helper_files/helper.pl').

reset_Model_NoTrain :-
        linear_SVM_initModelNoTrain(2, 0.0001, 1.0, 0).

reset_Model_WithTrain :-
        linear_SVM_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.0001, 1.0, 0, lbfgs).

:- begin_tests(lists).


%%
%% TESTING predicate linear_SVM_initModelWithTrain/8
%%
:- begin_tests(linear_SVM_initModelWithTrain).      

%% Failure Tests
                                            
test(linear_SVM_InitModelWithTrain_Negative_NumClasses, fail) :-
        linear_SVM_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], -2, 0.0001, 1.0, 0, lbfgs).

test(linear_SVM_InitModelWithTrain_Negative_Lambda, fail) :-
        linear_SVM_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, -0.0001, 1.0, 0, lbfgs).

test(linear_SVM_InitModelWithTrain_Negative_Delta, fail) :-
        linear_SVM_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.0001, -1.0, 0, lbfgs).

test(linear_SVM_InitModelWithTrain_Wrong_Optimizer_Input, [error(domain_error('The given Optimizer is unkown!' , wrongInput), _)]) :-
        linear_SVM_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.0001, 1.0, 0, wrongInput).

test(linear_SVM_InitModelWithTrain_Too_Few_Labels, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        linear_SVM_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], 2, 0.0001, 1.0, 0, lbfgs).

test(linear_SVM_InitModelWithTrain_Too_Many_Labels, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        linear_SVM_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], 2, 0.0001, 1.0, 0, lbfgs).
        

%% Successful Tests

test(linear_SVM_InitModelWithTrain_Normal_Use_LBFGS) :-
        linear_SVM_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.0001, 1.0, 0, lbfgs).

test(linear_SVM_InitModelWithTrain_Normal_Use_PSGD) :-
        linear_SVM_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.0001, 1.0, 0, psgd).

test(linear_SVM_InitModelWithTrain_CSV_Inpupt_LBFGS) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        linear_SVM_initModelWithTrain(Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 0.000021, 2.0, 1, lbfgs).

:- end_tests(linear_SVM_initModelWithTrain).



%%
%% TESTING predicate linear_SVM_initModelNoTrain/4
%%
:- begin_tests(linear_SVM_initModelNoTrain).      

%% Failure Tests
                                            
test(linear_SVM_InitModelNoTrain_Negative_NumClasses, fail) :-
        linear_SVM_initModelNoTrain(-2, 0.0001, 1.0, 0).

test(linear_SVM_InitModelNoTrain_Negative_Lambda, fail) :-
        linear_SVM_initModelNoTrain(2, -0.0001, 1.0, 0).

test(linear_SVM_InitModelNoTrain_Negative_Delta, fail) :-
        linear_SVM_initModelNoTrain(2, 0.0001, -1.0, 0).
        

%% Successful Tests

test(linear_SVM_InitModelNoTrain_Normal_Use) :-
        linear_SVM_initModelNoTrain(2, 0.0001, 1.0, 0).

test(linear_SVM_InitModelNoTrain_Alternative) :-
        linear_SVM_initModelNoTrain(2, 0.042, 0.5, 1).

:- end_tests(linear_SVM_initModelNoTrain).



%%
%% TESTING predicate linear_SVM_classify/5
%%
:- begin_tests(linear_SVM_classify).      

%% Failure Tests
                                            
test(linear_SVM_Classify_Before_Train, [error(_,system_error('The Model is not trained!'))]) :-
        reset_Model_NoTrain,
        linear_SVM_classify([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _).

test(linear_SVM_Classify_Diffrent_Dims_Than_Train, [error(_,system_error('The given Datapoints Dimension is != the trained Datapoints Dimension!'))]) :-
        reset_Model_WithTrain,
        linear_SVM_classify([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, _, _, _).
        

%% Successful Tests

test(linear_SVM_Classify_Normal_Use_LBFGS) :-
        linear_SVM_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.0001, 1.0, 0, lbfgs),
        linear_SVM_classify([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, LabelsList, ScoresList, _),
        print('\nLabels: '),
        print(LabelsList),
        print('\nScores: '),
        print(ScoresList).

test(linear_SVM_Classify_Normal_Use_PSGD) :-
        linear_SVM_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.0001, 1.0, 0, psgd),
        linear_SVM_classify([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, LabelsList, ScoresList, _),
        print('\nLabels: '),
        print(LabelsList),
        print('\nScores: '),
        print(ScoresList).

test(linear_SVM_Classify_CSV_Input) :-
        linear_SVM_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, [0,1,0], 2, 0.0001, 2.0, 1, lbfgs),
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        linear_SVM_classify(Data, 4, LabelsList, ScoresList, _),
        print('\nLabels: '),
        print(LabelsList),
        print('\nScores: '),
        print(ScoresList).

:- end_tests(linear_SVM_classify).



%%
%% TESTING predicate linear_SVM_classifyPoint/2
%%
:- begin_tests(linear_SVM_classifyPoint).      

%% Failure Tests
                                            
test(linear_SVM_ClassifyPoint_Before_Train, [error(_,system_error('The Model is not trained!'))]) :-
        reset_Model_NoTrain,
        linear_SVM_classifyPoint([5.1,3.5,1.4], _).

test(linear_SVM_ClassifyPoint_Diffrent_Dims_Than_Train, [error(_,system_error('The given Datapoint Dimension is != the trained Datapoints Dimension!'))]) :-
        reset_Model_WithTrain,
        linear_SVM_classifyPoint([5.1,3.5,1.4,4.9], _).
        

%% Successful Tests

test(linear_SVM_ClassifyPoint_Normal_Use_LBFGS) :-
        linear_SVM_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.0001, 1.0, 0, lbfgs),
        linear_SVM_classifyPoint([5.1,3.5,1.4], Label),
        print('\nLabel: '),
        print(Label).

test(linear_SVM_ClassifyPoint_Normal_Use_PSGD) :-
        linear_SVM_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.0001, 1.0, 0, psgd),
        linear_SVM_classifyPoint([5.1,3.5,1.4], Label),
        print('\nLabel: '),
        print(Label).

test(linear_SVM_ClassifyPoint_CSV_Input) :-
        linear_SVM_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, [0,1,0], 2, 0.0001, 2.0, 1, lbfgs),
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,1, Data),
        linear_SVM_classifyPoint(Data, Label),
        print('\nLabel: '),
        print(Label).

:- end_tests(linear_SVM_classifyPoint).



%%
%% TESTING predicate linear_SVM_computeAccuracy/4
%%
:- begin_tests(linear_SVM_computeAccuracy).      

%% Failure Tests

test(linear_SVM_ComputeAccuracy_Before_Train, [error(_,system_error('The Model is not trained!'))]) :-
        reset_Model_NoTrain,
        linear_SVM_computeAccuracy([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], _).

test(linear_SVM_ComputeAccuracy_Too_Few_Labels, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model_WithTrain,
        linear_SVM_computeAccuracy([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], _).

test(linear_SVM_ComputeAccuracy_Too_Many_Labels, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model_WithTrain,
        linear_SVM_computeAccuracy([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], _).

test(linear_SVM_ComputeAccuracy_Wrong_Data_Dims, [error(_,system_error('The given Datapoints Dimension is != the trained Datapoints Dimension!'))]) :-
        reset_Model_WithTrain,
        linear_SVM_computeAccuracy([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, [0,1,0], _).
        

%% Successful Tests

test(linear_SVM_ComputeAccuracy_Normal_Use_LBFGS) :-
        linear_SVM_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.0001, 1.0, 0, lbfgs),
        linear_SVM_computeAccuracy([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], Accuracy),
        print('\nAccuracy: '),
        print(Accuracy).

test(linear_SVM_ComputeAccuracy_Normal_Use_PSGD) :-
        linear_SVM_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.0001, 1.0, 0, psgd),
        linear_SVM_computeAccuracy([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], Accuracy),
        print('\nAccuracy: '),
        print(Accuracy).

test(linear_SVM_ComputeAccuracy_CSV_Input) :-
        linear_SVM_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, [0,1,0], 2, 0.0001, 2.0, 1, lbfgs),
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        linear_SVM_computeAccuracy(Data, 4, [0,1,0,1,1,0,1,1,1,0], Accuracy),
        print('\nAccuracy: '),
        print(Accuracy).

:- end_tests(linear_SVM_computeAccuracy).



%%
%% TESTING predicate linear_SVM_train/6
%%
:- begin_tests(linear_SVM_train).      

%% Failure Tests
                                            
test(linear_SVM_Train_Negative_NumClasses, fail) :-
        reset_Model_NoTrain,
        linear_SVM_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], -2, lbfgs, _).

test(linear_SVM_Train_Wrong_Optimizer_Input, [error(domain_error('The given Optimizer is unkown!' , wrongInput), _)]) :-
        reset_Model_NoTrain,
        linear_SVM_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, wrongInput, _).

test(linear_SVM_Train_Too_Few_Labels, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model_NoTrain,
        linear_SVM_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], 2, lbfgs, _).

test(linear_SVM_Train_Too_Many_Labels, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model_NoTrain,
        linear_SVM_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], 2, lbfgs, _).
        

%% Successful Tests

test(linear_SVM_Train_Normal_Use_LBFGS) :-
        reset_Model_NoTrain,
        linear_SVM_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, lbfgs, ObjectiveValue),
        print('\nObjectiveValue: '),
        print(ObjectiveValue).

test(linear_SVM_Train_Normal_Use_PSGD) :-
        reset_Model_NoTrain,
        linear_SVM_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, psgd, ObjectiveValue),
        print('\nObjectiveValue: '),
        print(ObjectiveValue).

test(linear_SVM_Train_CSV_Inpupt_LBFGS) :-
        reset_Model_NoTrain,
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        linear_SVM_train(Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, lbfgs, ObjectiveValue),
        print('\nObjectiveValue: '),
        print(ObjectiveValue).

:- end_tests(linear_SVM_train).

run_linear_SVM_tests :-
        run_tests.

