
:- module(linear_regression_tests, [run_linear_regression_tests/0]).

:- use_module(library(plunit)).

:- use_module(linear_regression).
:- use_module('../../helper_files/helper.pl').

reset_Model :-
        linear_regression_initModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 0.0, 1).

reset_Model_WithWeights :-
        linear_regression_initModelWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], [0.2,0.1,0.2,0.5], 0.0, 1).



%%
%% TESTING predicate linear_regression_initModel/5
%%
:- begin_tests(linear_regression_initModel).      

%% Failure Tests
                                            
test(linear_regression_InitModel_Too_Few_Labels, [error(_, system_error('The number of data points does not match the number of labels!'))]) :-
        linear_regression_initModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], 0.0, 1).

test(linear_regression_InitModel_Too_Many_Labels, [error(_, system_error('The number of data points does not match the number of labels!'))]) :-
        linear_regression_initModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], 0.0, 1).



%% Successful Tests

test(linear_regression_InitModel_Normal_Use) :-
        linear_regression_initModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 0.0, 1).

test(linear_regression_InitModel_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        linear_regression_initModel(Data, 4, [0,1,0,1,1,0,1,1,1,0], 0.5, 0).

:- end_tests(linear_regression_initModel).



%%
%% TESTING predicate linear_regression_initModelWithWeights/6
%%
:- begin_tests(linear_regression_initModelWithWeights).      

%% Failure Tests
                                            
test(linear_regression_InitModelWithWeights_Too_Few_Labels, [error(_, system_error('The number of data points does not match the number of labels!'))]) :-
        linear_regression_initModelWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], [0.2,0.1,0.2,0.5], 0.0, 1).

test(linear_regression_InitModelWithWeights_Too_Many_Labels, [error(_, system_error('The number of data points does not match the number of labels!'))]) :-
        linear_regression_initModelWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], [0.2,0.1,0.2,0.5], 0.0, 1).


test(linear_regression_InitModelWithWeights_Too_Few_Weights, [error(_, system_error('The number of data points does not match the number of weights!'))]) :-
        linear_regression_initModelWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], [0.2,0.5], 0.0, 1).

test(linear_regression_InitModelWithWeights_Too_Many_Weights, [error(_, system_error('The number of data points does not match the number of weights!'))]) :-
        linear_regression_initModelWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], [0.2,0.1,0.2,0.1,0.2,0.5], 0.0, 1).


%% Successful Tests

test(linear_regression_InitModelWithWeights_Normal_Use) :-
        linear_regression_initModelWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], [0.2,0.1,0.2,0.5], 0.0, 1).

test(linear_regression_InitModelWithWeights_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        linear_regression_initModelWithWeights(Data, 4, [0,1,0,1,1,0,1,1,1,0], [0.2,0.1,0.2,0.5,0.2,0.4,1.2,0.7,2.1,0.1], 0.5, 0).

:- end_tests(linear_regression_initModelWithWeights).



%%
%% TESTING predicate linear_regression_computeError/4
%%
:- begin_tests(linear_regression_computeError).      

%% Failure Tests
                                            
test(linear_regression_ComputeError_Too_Few_Labels, [error(_, system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model,
        linear_regression_computeError([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], _).

test(linear_regression_ComputeError_Too_Many_Labels, [error(_, system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model,
        linear_regression_computeError([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], _).


%% Successful Tests

test(linear_regression_ComputeError_Normal_Use) :-
        reset_Model,
        linear_regression_computeError([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], Error),
        print('\nError: '),
        print(Error).


:- end_tests(linear_regression_computeError).



%%
%% TESTING predicate linear_regression_parameters/1
%%
:- begin_tests(linear_regression_parameters).      

%% Failure Tests
        

%% Successful Tests

test(linear_regression_Parameters_Normal_Use) :-
        reset_Model,
        linear_regression_parameters(Parameters),
        print('\nParameters: '),
        print(Parameters).

:- end_tests(linear_regression_parameters).



%%
%% TESTING predicate linear_regression_modifyParameters/1
%%
:- begin_tests(linear_regression_modifyParameters).      

%% Failure Tests
                                            
test(linear_regression_ModifyParameters_Too_Few_Parameters) :-
        reset_Model,
        linear_regression_modifyParameters([3,4]),
        linear_regression_parameters(Parameters),
        print('\nParameters: '),
        print(Parameters).

test(linear_regression_ModifyParameters_Too_Many_Parameters) :-
        reset_Model,
        linear_regression_modifyParameters([1,2,1,2,3,4]),
        linear_regression_parameters(Parameters),
        print('\nParameters: '),
        print(Parameters).

test(linear_regression_ModifyParameters_Bad_Parameter_Value) :-
        reset_Model,
        linear_regression_modifyParameters([1,2,-3,4]),
        linear_regression_parameters(Parameters),
        print('\nParameters: '),
        print(Parameters).
        

%% Successful Tests

test(linear_regression_ModifyParameters_Normal_Use) :-
        reset_Model,
        linear_regression_modifyParameters([1,2,3,4]),
        linear_regression_parameters(Parameters),
        print('\nParameters: '),
        print(Parameters).

:- end_tests(linear_regression_modifyParameters).



%%
%% TESTING predicate linear_regression_predict/10
%%
:- begin_tests(linear_regression_predict).      

%% Failure Tests
                                            
test(linear_regression_Predict_Diffrent_Dims_Than_Trained, [error(_, system_error('The given Points have a diffrent Dimension than the trained Model!'))]) :-
        reset_Model,
        linear_regression_predict([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, _).
        

%% Successful Tests

test(linear_regression_Predict_Normal_Use) :-
        reset_Model,
        linear_regression_predict([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, Responses),
        print('\nResponses: '),
        print(Responses).

test(linear_regression_Predict_Normal_Use) :-
        reset_Model_WithWeights,
        linear_regression_predict([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, Responses),
        print('\nResponses: '),
        print(Responses).

:- end_tests(linear_regression_predict).



%%
%% TESTING predicate linear_regression_train/5
%%
:- begin_tests(linear_regression_train).      

%% Failure Tests
                                            
test(linear_regression_Train_Too_Few_Labels, [error(_, system_error('The number of data points does not match the number of labels!'))]) :-
        linear_regression_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], 1, _).

test(linear_regression_Train_Too_Many_Labels, [error(_, system_error('The number of data points does not match the number of labels!'))]) :-
        linear_regression_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], 1, _).


%% Successful Tests

test(linear_regression_Train_Normal_Use) :-
        linear_regression_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 1, Error),
        print('\nError: '),
        print(Error).

test(linear_regression_Train_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        linear_regression_train(Data, 4, [0,1,0,1,1,0,1,1,1,0], 0, Error),
        print('\nError: '),
        print(Error).

:- end_tests(linear_regression_train).



%%
%% TESTING predicate linear_regression_trainWithWeights/6
%%
:- begin_tests(linear_regression_trainWithWeights).      

%% Failure Tests
                                            
test(linear_regression_TrainWithWeights_Too_Few_Labels, [error(_, system_error('The number of data points does not match the number of labels!'))]) :-
        linear_regression_trainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], [0.2,0.1,0.2,0.5], 1 ,_).

test(linear_regression_TrainWithWeights_Too_Many_Labels, [error(_, system_error('The number of data points does not match the number of labels!'))]) :-
        linear_regression_trainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], [0.2,0.1,0.2,0.5], 1 ,_).


test(linear_regression_TrainWithWeights_Too_Few_Weights, [error(_, system_error('The number of data points does not match the number of weights!'))]) :-
        linear_regression_trainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], [0.2,0.5], 1 ,_).

test(linear_regression_TrainWithWeights_Too_Many_Weights, [error(_, system_error('The number of data points does not match the number of weights!'))]) :-
        linear_regression_trainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], [0.2,0.1,0.2,0.1,0.2,0.5], 1 ,_).


%% Successful Tests

test(linear_regression_TrainWithWeights_Normal_Use) :-
        linear_regression_trainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], [0.2,0.1,0.2,0.5], 1 ,Error),
        print('\nError: '),
        print(Error).

test(linear_regression_TrainWithWeights_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        linear_regression_trainWithWeights(Data, 4, [0,1,0,1,1,0,1,1,1,0], [0.2,0.1,0.2,0.5,0.2,0.4,1.2,0.7,2.1,0.1], 0, Error),
        print('\nError: '),
        print(Error).

:- end_tests(linear_regression_trainWithWeights).

run_linear_regression_tests :-
        run_tests.

