
:- module(lmnn, [lmnn/17]).

%% requirements of library(struct)
:- load_files(library(str_decl),
        [when(compile_time), if(changed)]).

%% needed for using the array type
:- use_module(library(structs)).
:- use_module('../../helper_files/helper.pl').

%% type definitions for the float array
:- foreign_type
        float32          = float_32,
        float_array      = array(float32).

%% definitions for the connected function


%% --Input--
%%              string  optimizer       => "amsgrad", "bbsgd", "sgd", "lbfgs",
%%              mat     data,
%%              vec     labels,
%%              int     k               => 1,
%%              float32 regularization  => 0.5,
%%              float32 stepSize        => 0.01,
%%              int     passes          => 50,
%%              int     maxIterations   => 100000,
%%              int     tolerance       => 1e-7,
%%              bool    center          => (1)true / (0)false => false,
%%              bool    shuffle         => (1)true / (0)false,
%%              int     batchSize       => 50,
%%              int     range           => 1,
%%              int     rank            => 0,
%%
%% --Output--
%%              mat     distance
%%
%% --Description--
%%              Is a single predicate that initiates the lmnn model with all the given params and then performs Large Margin Nearest Neighbors metric learning on the reference data.
%%
lmnn(Optimizer, DataList, DataRows, LabelsList, K, Regularization, StepSize, Passes, MaxIterations, Tolerance, Center, Shuffle, BatchSize, Range, Rank, DistanceList, ZCols) :-
        K > 0,
        Regularization >= 0.0,
        StepSize >= 0.0,
        Passes >= 0,
        MaxIterations >= 0,
        Tolerance >= 0.0,
        BatchSize > 0,
        Range > 0,
        Rank >= 0,
        convert_list_to_float_array(DataList, DataRows, array(Xsize, Xrownum, X)),
        convert_list_to_float_array(LabelsList, array(Ysize, Y)),
        lmnnI(Optimizer, X, Xsize, Xrownum, Y, Ysize, K, Regularization, StepSize, Passes, MaxIterations, Tolerance, Center, Shuffle, BatchSize, Range, Rank, Z, ZCols, ZRows),
        convert_float_array_to_2d_list(Z, ZCols, ZRows, DistanceList).
        
foreign(lmnn, c, lmnnI( +string,
                        +pointer(float_array), +integer, +integer,
                        +pointer(float_array), +integer,
                        +integer, 
                        +float32, +float32, +integer, +integer, +float32,
                        +integer, +integer,
                        +integer, +integer, +integer,
                        -pointer(float_array), -integer, -integer)).


%% Defines the functions that get connected from main.cpp
foreign_resource(lmnn, [lmnn]).

:- load_foreign_resource(lmnn).
