
:- module(lmnn_tests, [run_lmnn_tests/0]).

:- use_module(library(plunit)).

:- use_module(lmnn).
:- use_module('../../helper_files/helper.pl').


%%
%% TESTING predicate lmnn/10
%%
:- begin_tests(lmnn).      

%% Failure Tests
                                            
test(lmnn_Wrong_Optimizer_Input, [error(domain_error('The given Optimizer is unkown!' , wrongInput), _)]) :-
        lmnn(wrongInput, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.5, 0.01, 50, 100000, 0.000001, 0, 0, 50, 1, 0, _, _).

test(lmnn_Negative_K, fail) :-
        lmnn(lbfgs, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], -2, 0.5, 0.01, 50, 100000, 0.000001, 0, 0, 50, 1, 0, _, _).

test(lmnn_Too_Many_K, error(_, system_error('fatal error; see Log::Fatal output'))) :-
        lmnn(lbfgs, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 10, 0.5, 0.01, 50, 100000, 0.000001, 0, 0, 50, 1, 0, _, _).

test(lmnn_Negative_Regularization, fail) :-
        lmnn(lbfgs, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, -0.5, 0.01, 50, 100000, 0.000001, 0, 0, 50, 1, 0, _, _).

test(lmnn_Negative_StepSize, fail) :-
        lmnn(lbfgs, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.5, -0.01, 50, 100000, 0.000001, 0, 0, 50, 1, 0, _, _).

test(lmnn_Negative_Passes, fail) :-
        lmnn(lbfgs, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.5, 0.01, -50, 100000, 0.000001, 0, 0, 50, 1, 0, _, _).

test(lmnn_Negative_MaxIterations, fail) :-
        lmnn(lbfgs, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.5, 0.01, 50, -100000, 0.000001, 0, 0, 50, 1, 0, _, _).

test(lmnn_Negative_Tolerance, fail) :-
        lmnn(lbfgs, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.5, 0.01, 50, 100000, -0.000001, 0, 0, 50, 1, 0, _, _).

test(lmnn_Negative_BatchSize, fail) :-
        lmnn(lbfgs, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.5, 0.01, 50, 100000, 0.000001, 0, 0, -50, 1, 0, _, _).

test(lmnn_Negative_Range, fail) :-
        lmnn(lbfgs, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.5, 0.01, 50, 100000, 0.000001, 0, 0, 50, -1, 0, _, _).

test(lmnn_Negative_Rank, fail) :-
        lmnn(lbfgs, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.5, 0.01, 50, 100000, 0.000001, 0, 0, 50, 1, -1, _, _).


test(lmnn_Negative_Too_Few_Labels, [error(_, system_error('The number of data points does not match the number of labels!'))]) :-
        lmnn(lbfgs, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], 2, 0.5, 0.01, 50, 100000, 0.000001, 0, 0, 50, 1, 0, _, _).

test(lmnn_Negative_Too_Many_Labels, [error(_, system_error('The number of data points does not match the number of labels!'))]) :-
        lmnn(lbfgs, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], 2, 0.5, 0.01, 50, 100000, 0.000001, 0, 0, 50, 1, 0, _, _).

        

%% Successful Tests

%% k need to be < than the amount of elements per label class 
test(lmnn_Normal_Use_Amsgrad) :-
        lmnn(amsgrad, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 1, 0.5, 0.01, 50, 100000, 0.000001, 0, 0, 50, 1, 0, DistanceList, _),
        print('\nDistance: '),
        print(DistanceList).

test(lmnn_Normal_Use_BBSGD) :-
        lmnn(bbsgd, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 1, 0.5, 0.01, 50, 100000, 0.000001, 0, 0, 50, 1, 0, DistanceList, _),
        print('\nDistance: '),
        print(DistanceList).

test(lmnn_Normal_Use_SGD) :-
        lmnn(sgd, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 1, 0.5, 0.01, 50, 100000, 0.000001, 0, 0, 50, 1, 0, DistanceList, _),
        print('\nDistance: '),
        print(DistanceList).

test(lmnn_Normal_Use_LBFGS) :-
        lmnn(lbfgs, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5,3.2,1.3,0.5], 3, [0,1,0,1,0], 1, 0.5, 0.01, 50, 100000, 0.000001, 0, 0, 50, 1, 0, DistanceList, _),
        print('\nDistance: '),
        print(DistanceList).

test(lmnn_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow, 10, Data),
        lmnn(lbfgs, Data, 4, [0,1,0,1,1,0,1,1,1,0], 3, 0.7, 0.00021, 30, 1000, 0.00031, 1, 1, 30, 2, 2, DistanceList, _),
        print('\nDistance: '),
        print(DistanceList).

:- end_tests(lmnn).

run_lmnn_tests :-
        run_tests.

