#include <sicstus/sicstus.h>
/* ex_glue.h is generated by splfr from the foreign/[2,3] facts.
   Always include the glue header in your foreign resource code.
*/
#include "local_coordinate_coding_glue.h"
#include <mlpack/methods/local_coordinate_coding/lcc.hpp>
#include <mlpack/core.hpp>

// including helper functions for converting between arma structures and arrays
#include "../../helper_files/helper.hpp"

// some of the most used namespaces
using namespace arma;
using namespace mlpack;
using namespace std;
using namespace mlpack::lcc;


// input:   const arma::mat & 	            data,
//          const size_t 	                atoms,
//          const double 	                lambda,
//          const size_t 	                maxIterations = 0,
//          const double 	                tolerance = 0.01,
//          const DictionaryInitializer & 	initializer = DictionaryInitializer()
//          const arma::mat & 	            toEncodedata,
//          arma::mat & 	                codes <-
// output: 
// description: 
//          Initializes the model, trains it and encodes each point of the given data via distance-weighted LARS.
//
void lcc(float *dataMatArr, SP_integer dataMatSize, SP_integer dataMatRowNum,
            SP_integer normalize, SP_integer atoms, double lambda,  SP_integer maxIterations, double tolerance,
            float *toEncodeDataMatArr, SP_integer toEncodeDataMatSize, SP_integer toEncodeDataMatRowNum,
            float **codesMatArr, SP_integer *codesMatColNum, SP_integer *codesMatRowNum)
{
    // convert the Prolog array to arma::mat
    mat data = convertArrayToMat(dataMatArr, dataMatSize, dataMatRowNum);
    mat toEncodeData = convertArrayToMat(toEncodeDataMatArr, toEncodeDataMatSize, toEncodeDataMatRowNum);

    // create the ReturnMat
    mat codesReturnMat;

    if (normalize == 1)
    {
        Log::Info << "Normalizing data before coding..." << endl;
        for (size_t i = 0; i < data.n_cols; ++i)
            data.col(i) /= arma::norm(data.col(i), 2);

        Log::Info << "Normalizing test data before coding..." << endl;
        for (size_t i = 0; i < toEncodeData.n_cols; ++i)
            toEncodeData.col(i) /= arma::norm(toEncodeData.col(i), 2);
    }

    
    try
    {
        LocalCoordinateCoding(data, atoms, lambda, maxIterations, tolerance)
            .Encode(toEncodeData, codesReturnMat);
    }
    catch(const std::exception& e)
    {
        raisePrologSystemExeption(e.what());
    }


    // return the Matrix
    returnMatrixInformation(codesReturnMat, codesMatArr, codesMatColNum, codesMatRowNum);
}