
:- module(local_coordinate_coding, [lcc/11]).

%% requirements of library(struct)
:- load_files(library(str_decl),
        [when(compile_time), if(changed)]).

%% needed for using the array type
:- use_module(library(structs)).
:- use_module('../../helper_files/helper.pl').

%% type definitions for the float array
:- foreign_type
        float32          = float_32,
        float_array      = array(float32).

%% definitions for the connected function


%% --Input--
%%              mat     data,
%%              bool    normalize       => (1)true / (0)false => false,
%%              int     atoms           => 0,
%%              float32 lambda          => 0.0,
%%              int     maxIterations   => 0,
%%              float32 tolerance       => 0.01
%%              mat     data
%%
%% --Output--
%%              mat     codes
%%
%% --Description--
%%              Initializes the model and trains it so encode/6 can be called after
%%
lcc(DataList, DataRows, Normalize, Atoms, Lambda, MaxIterations, Tolerance, TestDataList, TestDataRows, CodesList, ZRows) :-
        Atoms > 0,
        Lambda >= 0,
        MaxIterations >= 0,
        Tolerance > 0,
        convert_list_to_float_array(DataList, DataRows, array(Xsize, Xrownum, X)),
        Atoms < (Xsize / Xrownum),
        convert_list_to_float_array(TestDataList, TestDataRows, array(Ysize, Yrownum, Y)),
        lccI(X, Xsize, Xrownum, Normalize, Atoms, Lambda, MaxIterations, Tolerance, Y, Ysize, Yrownum, Z, ZCols, ZRows),
        convert_float_array_to_2d_list(Z, ZCols, ZRows, CodesList).

foreign(lcc, c, lccI(   +pointer(float_array), +integer, +integer, 
                        +integer, 
                        +integer, +float32, +integer, +float32,
                        +pointer(float_array), +integer, +integer,
                        -pointer(float_array), -integer, -integer)).


%% Defines the functions that get connected from main.cpp
foreign_resource(local_coordinate_coding, [lcc]).

:- load_foreign_resource(local_coordinate_coding).
