
:- module(local_coordinate_coding_tests, [run_local_coordinate_coding_tests/0]).

:- use_module(library(plunit)).

:- use_module(local_coordinate_coding).
:- use_module('../../helper_files/helper.pl').


%%
%% TESTING predicate lcc/11
%%
:- begin_tests(lcc).      

%% Failure Tests
                                            
test(lcc_Bad_Atom_Input, fail) :-
        lcc([5.1,3.5,1.4, 4.9,3.0,1.4, 4.7,3.2,1.3, 4.6,3.1,1.5], 3, 0, -2, 0.0, 0, 0.01, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _),
        lcc([5.1,3.5,1.4, 4.9,3.0,1.4, 4.7,3.2,1.3, 4.6,3.1,1.5], 3, 0, 4, 0.0, 0, 0.01, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _).

test(lcc_Negative_Lambda, fail) :-
        lcc([5.1,3.5,1.4, 4.9,3.0,1.4, 4.7,3.2,1.3, 4.6,3.1,1.5], 3, 0, 2, -1.0, 0, 0.01, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _).

test(lcc_Negative_MaxIterations, fail) :-
        lcc([5.1,3.5,1.4, 4.9,3.0,1.4, 4.7,3.2,1.3, 4.6,3.1,1.5], 3, 0, 2, 0.0, -1, 0.01, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _).

test(lcc_Negative_Tolerance, fail) :-
        lcc([5.1,3.5,1.4, 4.9,3.0,1.4, 4.7,3.2,1.3, 4.6,3.1,1.5], 3, 0, 2, 0.0, 0, -0.01, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _).

test(lcc_Different_Dims, [error(_,system_error('matrix multiplication: incompatible matrix dimensions: 2x4 and 3x4'))]) :-
        lcc([5.1,3.5,1.4, 4.9,3.0,1.4, 4.7,3.2,1.3, 4.6,3.1,1.5], 4, 0, 2, 0.0, 0, 0.01, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _).


%% Successful Tests

test(lcc_Normal_Use) :-
        lcc([5.1,3.5,1.4, 4.9,3.0,1.4, 4.7,3.2,1.3, 4.6,3.1,1.5], 3, 0, 2, 0.0, 0, 0.01, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, CodesList, _),
        print('\nCodes: '),
        print(CodesList).

test(lcc_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        lcc(Data, 4, 1, 5, 1.5, 50, 0.042, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, CodesList, _),
        print('\nCodes: '),
        print(CodesList).

:- end_tests(lcc).

run_local_coordinate_coding_tests :-
        run_tests.

