
:- module(logistic_regression_tests, [run_logistic_regression_tests/0]).

:- use_module(library(plunit)).

:- use_module(logistic_regression).
:- use_module('../../helper_files/helper.pl').

reset_Model_NoOptimizer :-
        logistic_regression_initModelNoOptimizer([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 0.0).

reset_Model_WithOptimizer :-
        logistic_regression_initModelWithOptimizer([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], lbfgs, 0.0, 1000, 1.0e-10, 0.01, 64).

%%
%% TESTING predicate logistic_regression_initModelNoOptimizer/4
%%
:- begin_tests(logistic_regression_initModelNoOptimizer).      

%% Failure Tests
                                            
test(logistic_regression_InitModelNoOptimizer_Negative_Lambda, fail) :-
        logistic_regression_initModelNoOptimizer([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], -1.0).


test(logistic_regression_InitModelNoOptimizer_Too_Few_Labels, [error(_, system_error('The number of data points does not match the number of labels!'))]) :-
        logistic_regression_initModelNoOptimizer([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], 0.0).

test(logistic_regression_InitModelNoOptimizer_Too_Many_Labels, [error(_, system_error('The number of data points does not match the number of labels!'))]) :-
        logistic_regression_initModelNoOptimizer([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], 0.0).

%% doesnt cause error
test(logistic_regression_InitModelNoOptimizer_Too_Many_Labelclasses) :-
        logistic_regression_initModelNoOptimizer([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,2,3], 0.0).
        

%% Successful Tests

test(logistic_regression_InitModelNoOptimizer_Normal_Use) :-
        logistic_regression_initModelNoOptimizer([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 0.0).

test(logistic_regression_InitModelNoOptimizer_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        logistic_regression_initModelNoOptimizer(Data, 4, [0,1,0,1,1,0,1,1,1,0], 0.0).

:- end_tests(logistic_regression_initModelNoOptimizer).



%%
%% TESTING predicate logistic_regression_initModelWithOptimizer/10
%%
:- begin_tests(logistic_regression_initModelWithOptimizer).      

%% Failure Tests

test(logistic_regression_InitModelWithOptimizer_Wrong_Optimizer, [error(domain_error('The given Optimizer is unkown!' , wrongInput), _)]) :-
        logistic_regression_initModelWithOptimizer([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], wrongInput, 0.0, 1000, 1.0e-10, 0.01, 64).
                                            
test(logistic_regression_InitModelWithOptimizer_Negative_Lambda, fail) :-
        logistic_regression_initModelWithOptimizer([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], lbfgs, -1.0, 1000, 1.0e-10, 0.01, 64).

test(logistic_regression_InitModelWithOptimizer_Negative_MaxIteration, fail) :-
        logistic_regression_initModelWithOptimizer([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], lbfgs, 0.0, -1000, 1.0e-10, 0.01, 64).

test(logistic_regression_InitModelWithOptimizer_Negative_Tolerance, fail) :-
        logistic_regression_initModelWithOptimizer([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], lbfgs, 0.0, 1000, -1.0e-10, 0.01, 64).

test(logistic_regression_InitModelWithOptimizer_Negative_StepSize, fail) :-
        logistic_regression_initModelWithOptimizer([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], lbfgs, 0.0, 1000, 1.0e-10, -0.01, 64).

test(logistic_regression_InitModelWithOptimizer_Negative_BatchSize, fail) :-
        logistic_regression_initModelWithOptimizer([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], lbfgs, 0.0, 1000, 1.0e-10, 0.01, -64).


test(logistic_regression_InitModelWithOptimizer_Too_Few_Labels, [error(_, system_error('The number of data points does not match the number of labels!'))]) :-
        logistic_regression_initModelWithOptimizer([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], lbfgs, 0.0, 1000, 1.0e-10, 0.01, 64).

test(logistic_regression_InitModelWithOptimizer_Too_Many_Labels, [error(_, system_error('The number of data points does not match the number of labels!'))]) :-
        logistic_regression_initModelWithOptimizer([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], lbfgs, 0.0, 1000, 1.0e-10, 0.01, 64).

%% doesnt cause error
test(logistic_regression_InitModelWithOptimizer_Too_Many_Labelclasses) :-
        logistic_regression_initModelWithOptimizer([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,2,3], lbfgs, 0.0, 1000, 1.0e-10, 0.01, 64).
        

%% Successful Tests

test(logistic_regression_InitModelWithOptimizer_Normal_Use_LBFGS) :-
        logistic_regression_initModelWithOptimizer([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], lbfgs, 0.0, 1000, 1.0e-10, 0.01, 64).

test(logistic_regression_InitModelWithOptimizer_CSV_Input_LBFGS) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        logistic_regression_initModelWithOptimizer(Data, 4, [0,1,0,1,1,0,1,1,1,0], lbfgs, 0.0, 1000, 1.0e-10, 0.01, 64).

test(logistic_regression_InitModelWithOptimizer_Normal_Use_PSGD) :-
        logistic_regression_initModelWithOptimizer([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], psgd, 0.0, 1000, 1.0e-10, 0.01, 64).

test(logistic_regression_InitModelWithOptimizer_CSV_Input_PSGD) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        logistic_regression_initModelWithOptimizer(Data, 4, [0,1,0,1,1,0,1,1,1,0], psgd, 0.0, 1000, 1.0e-10, 0.01, 64).

:- end_tests(logistic_regression_initModelWithOptimizer).



%%
%% TESTING predicate logistic_regression_classifyPoint/3
%%
:- begin_tests(logistic_regression_classifyPoint).      

%% Failure Tests
                                            
test(logistic_regression_ClassifyPoint_Different_Dims, [error(_, system_error('The given Datapoint has a different Dimension than the trained Model!'))]) :-
        reset_Model_WithOptimizer,
        logistic_regression_classifyPoint([5.1,3.5,1.4,4.9], 0.5, _).

test(logistic_regression_ClassifyPoint_Bad_Boundary_Input, fail) :-
        reset_Model_WithOptimizer,
        logistic_regression_classifyPoint([5.1,3.5,1.4], -0.5, _),
        logistic_regression_classifyPoint([5.1,3.5,1.4], 1.5, _).
        

%% Successful Tests

test(logistic_regression_ClassifyPoint_Normal_Use) :-
        reset_Model_WithOptimizer,
        logistic_regression_classifyPoint([5.1,3.5,1.4], 0.5, PredictLabel),
        print('\nPredicted Label: '),
        print(PredictLabel).

test(logistic_regression_ClassifyPoint_NoOptimizer) :-
        reset_Model_NoOptimizer,
        logistic_regression_classifyPoint([5.1,3.5,1.4], 0.5, PredictLabel),
        print('\nPredicted Label: '),
        print(PredictLabel).

:- end_tests(logistic_regression_classifyPoint).



%%
%% TESTING predicate logistic_regression_classifyMatrix/10
%%
:- begin_tests(logistic_regression_classifyMatrix).      

%% Failure Tests
                                            
test(logistic_regression_ClassifyMatrix_Different_Dims, [error(_, system_error('The given Datapoints have a different Dimension than the trained Model!'))]) :-
        reset_Model_WithOptimizer,
        logistic_regression_classifyMatrix([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, _, _, _, 0.5).

test(logistic_regression_ClassifyMatrix_Bad_Boundary_Input, fail) :-
        reset_Model_WithOptimizer,
        logistic_regression_classifyMatrix([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _, -0.5),
        logistic_regression_classifyMatrix([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _, 1.5).
        

%% Successful Tests

test(logistic_regression_ClassifyMatrix_Normal_Use) :-
        reset_Model_WithOptimizer,
        logistic_regression_classifyMatrix([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, PredictLabels, ProbsList, _, 0.5),
        print('\nPredicted Labels: '),
        print(PredictLabels),
        print('\nProbabilities: '),
        print(ProbsList).

test(logistic_regression_ClassifyMatrix_NoOptimizer) :-
        reset_Model_NoOptimizer,
        logistic_regression_classifyMatrix([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, PredictLabels, ProbsList, _, 0.5),
        print('\nPredicted Labels: '),
        print(PredictLabels),
        print('\nProbabilities: '),
        print(ProbsList).

:- end_tests(logistic_regression_classifyMatrix).



%%
%% TESTING predicate logistic_regression_computeAccuracy/5
%%
:- begin_tests(logistic_regression_computeAccuracy).      

%% Failure Tests

test(logistic_regression_ComputeAccuracy_Different_Dims, [error(_, system_error('The given Datapoints have a different Dimension than the trained Model!'))]) :-
        reset_Model_WithOptimizer,
        logistic_regression_computeAccuracy([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, [0,1,0], 0.5, _).

test(logistic_regression_ComputeAccuracy_Bad_Boundary_Input, fail) :-
        reset_Model_WithOptimizer,
        logistic_regression_computeAccuracy([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], -0.5, _),
        logistic_regression_computeAccuracy([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 1.5, _).
        

test(logistic_regression_ComputeAccuracy_Too_Few_Labels, [error(_, system_error('The number of data points does not match the number of labels!'))]) :-
        logistic_regression_computeAccuracy([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], 0.5, _).

test(logistic_regression_ComputeAccuracy_Too_Many_Labels, [error(_, system_error('The number of data points does not match the number of labels!'))]) :-
        logistic_regression_computeAccuracy([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], 0.5, _).

%% Successful Tests

test(logistic_regression_ComputeAccuracy_Normal_Use) :-
        reset_Model_WithOptimizer,
        logistic_regression_computeAccuracy([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 0.5, AccuracyList),
        print('\nAccuracy: '),
        print(AccuracyList).

test(logistic_regression_ComputeAccuracy_NoOptimizer) :-
        reset_Model_NoOptimizer,
        logistic_regression_computeAccuracy([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 0.5, AccuracyList),
        print('\nAccuracy: '),
        print(AccuracyList).

:- end_tests(logistic_regression_computeAccuracy).



%%
%% TESTING predicate logistic_regression_computeError/4
%%
:- begin_tests(logistic_regression_computeError).      

%% Failure Tests

test(logistic_regression_computeError_Different_Dims, [error(_, system_error('The given Datapoints have a different Dimension than the trained Model!'))]) :-
        reset_Model_WithOptimizer,
        logistic_regression_computeError([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, [0,1,0], _).
        

test(logistic_regression_computeError_Too_Few_Labels, [error(_, system_error('The number of data points does not match the number of labels!'))]) :-
        logistic_regression_computeError([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], _).

test(logistic_regression_computeError_Too_Many_Labels, [error(_, system_error('The number of data points does not match the number of labels!'))]) :-
        logistic_regression_computeError([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], _).

test(logistic_regression_computeError_Too_Many_Labelclasses, [error(_, system_error('Error contains nan Value!'))]) :-
        logistic_regression_computeError([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,2,3], _).

%% Successful Tests

test(logistic_regression_computeError_Normal_Use) :-
        reset_Model_WithOptimizer,
        logistic_regression_computeError([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], ErrorList),
        print('\nError: '),
        print(ErrorList).

test(logistic_regression_computeError_NoOptimizer) :-
        reset_Model_NoOptimizer,
        logistic_regression_computeError([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], ErrorList),
        print('\nError: '),
        print(ErrorList).

:- end_tests(logistic_regression_computeError).

run_logistic_regression_tests :-
        run_tests.

