
:- module(lsh_tests, [run_lsh_tests/0]).

:- use_module(library(plunit)).

:- use_module(lsh).
:- use_module('../../helper_files/helper.pl').

reset_Model :-
        lsh_initAndTrainModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 20, 10, 0.0, 99901, 500).

%%
%% TESTING predicate lsh_initAndTrainModel/7
%%
:- begin_tests(lsh_initAndTrainModel).      

%% Failure Tests
                                            

test(lsh_InitAndTrainModel_Negative_NumProj, fail) :-
        lsh_initAndTrainModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, -20, 10, 0.0, 99901, 500).

test(lsh_InitAndTrainModel_Negative_NumTables, fail) :-
        lsh_initAndTrainModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 20, -10, 0.0, 99901, 500).

test(lsh_InitAndTrainModel_Negative_HashWidth, fail) :-
        lsh_initAndTrainModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 20, 10, -1.0, 99901, 500).

test(lsh_InitAndTrainModel_Negative_SecondHashSize, fail) :-
        lsh_initAndTrainModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 20, 10, 0.0, -99901, 500).

test(lsh_InitAndTrainModel_Negative_BucketSize, fail) :-
        lsh_initAndTrainModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 20, 10, 0.0, 99901, -500).
        

%% Successful Tests

test(lsh_InitAndTrainModel_Normal_Use) :-
        lsh_initAndTrainModel([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 20, 10, 0.0, 99901, 500).

test(lsh_InitAndTrainModel_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        lsh_initAndTrainModel(Data, 4, 25, 15, 1.5, 99901, 200).

:- end_tests(lsh_initAndTrainModel).



%%
%% TESTING predicate lsh_computeRecall/5
%%
:- begin_tests(lsh_computeRecall).      

%% Failure Tests
                                            
test(lsh_ComputeRecall_Wrong_Dimensions, [error(_, system_error('LSHSearch::ComputeRecall(): matrices provided must have equal size'))]) :-
        reset_Model,
        lsh_computeRecall([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, _).
        

%% Successful Tests

test(lsh_ComputeRecall_Normal_Use) :-
        reset_Model,
        lsh_computeRecall([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _).


:- end_tests(lsh_computeRecall).



%%
%% TESTING predicate lsh_searchWithQuery/10
%%
:- begin_tests(lsh_searchWithQuery).      

%% Failure Tests
                                            
test(lsh_SearchWithQuery_Wrong_Dimensions, [error(_, system_error('LSHSearch::Search(): dimensionality of query set (4) is not equal to the dimensionality the model was trained on (3)!\n'))]) :-
        reset_Model,
        lsh_searchWithQuery([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, 3, _, _, _, _, 0, 0).

test(lsh_SearchWithQuery_Too_High_K, [error(_, system_error('K should not be greater than the amount of datapoints!'))]) :-
        reset_Model,
        lsh_searchWithQuery([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 10, _, _, _, _, 0, 0).

test(lsh_SearchWithQuery_Negative_K, fail) :-
        reset_Model,
        lsh_searchWithQuery([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, -3, _, _, _, _, 0, 0).

test(lsh_SearchWithQuery_Negative_NumTablesToSearch, fail) :-
        reset_Model,
        lsh_searchWithQuery([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 3, _, _, _, _, -10, 0).

test(lsh_SearchWithQuery_Negative_T, fail) :-
        reset_Model,
        lsh_searchWithQuery([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 3, _, _, _, _, 0, -10).
        

%% Successful Tests

test(lsh_SearchWithQuery_Normal_Use) :-
        reset_Model,
        lsh_searchWithQuery([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 3, NeighborResultsList, _, DistancesList, _, 0, 0),
        print('\nNeighborResults: '),
        print(NeighborResultsList),
        print('\nDistances: '),
        print(DistancesList).

:- end_tests(lsh_searchWithQuery).



%%
%% TESTING predicate lsh_searchNoQuery/10
%%
:- begin_tests(lsh_searchNoQuery).      

%% Failure Tests

test(lsh_SearchNoQuery_Too_High_K, [error(_, system_error('K should not be greater than the amount of datapoints!'))]) :-
        reset_Model,
        lsh_searchNoQuery(10, _, _, _, _, 0, 0).

test(lsh_SearchNoQuery_Negative_K, fail) :-
        reset_Model,
        lsh_searchNoQuery(-3, _, _, _, _, 0, 0).

test(lsh_SearchNoQuery_Negative_NumTablesToSearch, fail) :-
        reset_Model,
        lsh_searchNoQuery(3, _, _, _, _, -10, 0).

test(lsh_SearchNoQuery_Negative_T, fail) :-
        reset_Model,
        lsh_searchNoQuery(3, _, _, _, _, 0, -10).
        

%% Successful Tests

test(lsh_SearchNoQuery_Normal_Use) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        lsh_initAndTrainModel(Data, 4, 25, 15, 1.5, 99901, 200),
        lsh_searchNoQuery(6, NeighborResultsList, _, DistancesList, _, 0, 0),
        print('\nNeighborResults: '),
        print(NeighborResultsList),
        print('\nDistances: '),
        print(DistancesList).

:- end_tests(lsh_searchNoQuery).

run_lsh_tests :-
        run_tests.

