#include <sicstus/sicstus.h>
/* ex_glue.h is generated by splfr from the foreign/[2,3] facts.
   Always include the glue header in your foreign resource code.
*/
#include "mean_shift_glue.h"
#include <mlpack/methods/mean_shift/mean_shift.hpp>
#include <mlpack/core.hpp>

// including helper functions for converting between arma structures and arrays
#include "../../helper_files/helper.hpp"

// some of the most used namespaces
using namespace arma;
using namespace mlpack;
using namespace std;
using namespace mlpack::meanshift;


// TODO:  	adding support for different Kernels
//
// input: 	const double 			radius = 0,
//			const size_t 			maxIterations = 1000,
//			const KernelType 		kernel = KernelType()
//			
//			const MatType & 		data,
//			arma::Row< size_t > & 	assignments,
//			arma::mat & 			centroids,
//			bool 					forceConvergence = true,
//			bool 					useSeeds = true
// output: 
// description: 
//			Perform mean shift clustering on the data, returning a list of cluster assignments and centroids.
//
void meanShift(double radius, SP_integer maxIterations, 
				float *dataMatArr, SP_integer dataMatSize, SP_integer dataMatRowNum, 
				float **assignmentsArr, SP_integer *assignmentsArrSize, 
				float **centroidsMatArr, SP_integer *centroidsMatColNum, SP_integer *centroidsMatRowNum, 
				SP_integer forceConvergence, 
				SP_integer useSeeds)
{
	// convert the Prolog array to arma::mat
	mat data = convertArrayToMat(dataMatArr, dataMatSize, dataMatRowNum);
	// create the ReturnVector
	arma::Row<size_t> assignmentsReturnVector;
	// create the ReturnMat
	mat centroidsReturnMat;


	try
	{
		MeanShift<>(radius, maxIterations).Cluster(data, assignmentsReturnVector, centroidsReturnMat, (forceConvergence == 1), (useSeeds == 1));
	}
	catch(const std::exception& e)
	{
		raisePrologSystemExeption(e.what());
		return;
	}


	// return the Matrix
	returnMatrixInformation(centroidsReturnMat, centroidsMatArr, centroidsMatColNum, centroidsMatRowNum);
	// return the Vector
	returnVectorInformation(assignmentsReturnVector, assignmentsArr, assignmentsArrSize);
}
