
:- module(mean_shift, [meanShift/9]).

%% requirements of library(struct)
:- load_files(library(str_decl),
                [when(compile_time), if(changed)]).

%% needed for using the array type
:- use_module(library(structs)).
:- use_module('../../helper_files/helper.pl').

%% type definitions for the float array
:- foreign_type
        float32          = float_32,
        float_array      = array(float32).

%% definitions for the connected function


%% --Input--
%%              flaot32 radius                  => 0,
%%              int     maxIterations           => 1000,
%%              mat     data,
%%              bool    forceConvergence        => (1)true / (0)false => true,
%%              bool    useSeeds                => (1)true / (0)false => true
%%
%% --Output--
%%              vec     assignments,
%%              mat     centroids
%%
%% --Description--
%%              Perform mean shift clustering on the data, returning a list of cluster assignments and centroids.
%%
meanShift(Radius, MaxIterations, DataList, DataRows, AssignmentsList, CentroidsList, ZCols, ForceConvergence, UseSeeds) :-
        MaxIterations >= 0,
        convert_list_to_float_array(DataList, DataRows, array(Xsize, Xrownum, X)),
        meanShiftI(Radius, MaxIterations, X, Xsize, Xrownum, Y, Ysize, Z, ZCols, ZRows, ForceConvergence, UseSeeds),
        convert_float_array_to_list(Y, Ysize, AssignmentsList),
        convert_float_array_to_2d_list(Z, ZCols, ZRows, CentroidsList).

foreign(meanShift, c, meanShiftI(+float32, +integer, 
                                +pointer(float_array), +integer, +integer, 
                                -pointer(float_array), -integer,
                                -pointer(float_array), -integer, -integer, 
                                +integer, +integer)).


%% Defines the functions that get connected from main.cpp
foreign_resource(mean_shift, [meanShift]).

:- load_foreign_resource(mean_shift).
