
:- module(naive_bayes_classifier_tests, [run_naive_bayes_classifier_tests/0]).

:- use_module(library(plunit)).

:- use_module(naive_bayes_classifier).
:- use_module('../../helper_files/helper.pl').

reset_Model_NoTrain :-
        nbc_initModelNoTrain(3, 0, 0.000001).

reset_Model_WithTrain :-
        nbc_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0, 0.000001).

%%
%% TESTING predicate nbc_initModelNoTrain/3
%%
:- begin_tests(nbc_initModelNoTrain).      

%% Failure Tests

test(nbc_InitModelNoTrain_Negative_Dimensionality, fail) :-
        nbc_initModelNoTrain(-3, 0, 0.000001).
                                            
test(nbc_InitModelNoTrain_Negative_NumClasses, fail) :-
        nbc_initModelNoTrain(3, -1, 0.000001).

test(nbc_InitModelNoTrain_Negative_Epsilon, fail) :-
        nbc_initModelNoTrain(3, 0, -0.0001).
        

%% Successful Tests

test(nbc_InitModelNoTrain_Normal_Use) :-
        nbc_initModelNoTrain(3, 0, 0.000001).

:- end_tests(nbc_initModelNoTrain).



%%
%% TESTING predicate nbc_initModelWithTrain/6
%%
:- begin_tests(nbc_initModelWithTrain).      

%% Failure Tests
                                            
test(nbc_InitModelWithTrain_Negative_NumClasses, fail) :-
        nbc_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], -2, 0, 0.000001).

test(nbc_InitModelWithTrain_Negative_Epsilon, fail) :-
        nbc_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0, -0.000001).


test(nbc_InitModelWithTrain_Too_Short_Label, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        nbc_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], 2, 0, 0.000001).

test(nbc_InitModelWithTrain_Too_Long_Label, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        nbc_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], 2, 0, 0.000001).
        

%% Successful Tests

test(nbc_InitModelWithTrain_Normal_Use) :-
        nbc_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0, 0.000001).

test(nbc_InitModelWithTrain_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        nbc_initModelWithTrain(Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 1, 0.0042).

:- end_tests(nbc_initModelWithTrain).



%%
%% TESTING predicate nbc_classifyMatrix/5
%%
:- begin_tests(nbc_classifyMatrix).      

%% Failure Tests

test(nbc_ClassifyMatrix_Before_Train, [error(_,system_error('The Model is not trained!'))]) :-
        reset_Model_NoTrain,
        nbc_classifyMatrix([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _).
                         
test(nbc_ClassifyMatrix_Diffrent_Dims_To_Train, [error(_,system_error('The Datapoints Dimensionality doesnt fit the trained Dimensionality!'))]) :-
        reset_Model_WithTrain,
        nbc_classifyMatrix([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, _, _, _).
        

%% Successful Tests

test(nbc_ClassifyMatrix_Normal_Use) :-
        reset_Model_WithTrain,
        nbc_classifyMatrix([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, PredictionList, _, ProbabilitiesList),
        print('\nPredictions: '),
        print(PredictionList),
        print('\nProbabilities: '),
        print(ProbabilitiesList).

test(nbc_ClassifyMatrix_CSV_Input) :-
        nbc_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, [0,1,0], 2, 0, 0.000001),
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        nbc_classifyMatrix(Data, 4, PredictionList, _, ProbabilitiesList),
        print('\nPredictions: '),
        print(PredictionList),
        print('\nProbabilities: '),
        print(ProbabilitiesList).

:- end_tests(nbc_classifyMatrix).



%%
%% TESTING predicate nbc_means/2
%%
:- begin_tests(nbc_means).      

%% Failure Tests
test(nbc_Means_Before_Train, [error(_,system_error('The Model is not trained!'))]) :-
        reset_Model_NoTrain,
        nbc_means(MeansList, _),
        print('\nMeans: '),
        print(MeansList).


%% Successful Tests

test(nbc_Means_Normal_Use) :-
        reset_Model_WithTrain,
        nbc_means(MeansList, _),
        print('\nMeans: '),
        print(MeansList).

:- end_tests(nbc_means).



%%
%% TESTING predicate nbc_probabilities/2
%%
:- begin_tests(nbc_probabilities).      

%% Failure Tests
test(nbc_Probabilities_Before_Train, [error(_,system_error('The Model is not trained!'))]) :-
        reset_Model_NoTrain,
        nbc_probabilities(ProbabilitiesList, _),
        print('\nProbabilities: '),
        print(ProbabilitiesList).


%% Successful Tests

test(nbc_Probabilities_Normal_Use) :-
        reset_Model_WithTrain,
        nbc_probabilities(ProbabilitiesList, _),
        print('\nProbabilities: '),
        print(ProbabilitiesList).

:- end_tests(nbc_probabilities).



%%
%% TESTING predicate nbc_trainMatrix/5
%%
:- begin_tests(nbc_trainMatrix).      

%% Failure Tests
                                            
test(nbc_TrainMatrix_Negative_NumClasses, fail) :-
        reset_Model_NoTrain,
        nbc_trainMatrix([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], -2, 0).

test(nbc_TrainMatrix_Too_Short_Label, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model_NoTrain,
        nbc_trainMatrix([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], 2, 0).

test(nbc_TrainMatrix_Too_Long_Label, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model_NoTrain,
        nbc_trainMatrix([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], 2, 0).
        
test(nbc_TrainMatrix_After_InitTrain, [error(_,system_error('The Datapoints Dimensionality doesnt fit the trained Dimensionality!'))]) :-
        reset_Model_WithTrain,
        nbc_trainMatrix([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, [0,1,0], 2, 0).

%% Successful Tests

test(nbc_TrainMatrix_Normal_Use) :-
        reset_Model_NoTrain,
        nbc_trainMatrix([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0).



test(nbc_TrainMatrix_CSV_Input) :-
        nbc_initModelNoTrain(4, 0, 0.000001),
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        nbc_trainMatrix(Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 0).

:- end_tests(nbc_trainMatrix).



%%
%% TESTING predicate nbc_trainPoint/5
%%
:- begin_tests(nbc_trainPoint).      

%% Failure Tests

test(nbc_TrainPoint_Bad_Label, [error(_,system_error('The given Label should be positive!'))]) :-
        reset_Model_NoTrain,
        nbc_trainPoint([5.1,3.5,1.4], -5).

test(nbc_TrainPoint_Different_Dim, [error(_,system_error('The Datapoints Dimensionality doesnt fit the trained Dimensionality!'))]) :-
        reset_Model_NoTrain,
        nbc_trainPoint([5.1,3.5,1.4,3.5], 0).

test(nbc_TrainPoint_Too_Long_Point, [error(_,system_error('The Datapoints Dimensionality doesnt fit the trained Dimensionality!'))]) :-
        reset_Model_WithTrain,
        nbc_trainPoint([5.1,3.5,1.4,3.5], 0).
        

%% Successful Tests

%%test(nbc_TrainPoint_Normal_Use) :-
%%        reset_Model_NoTrain,
%%        nbc_trainPoint([5.1,3.5,1.4], 1).

test(nbc_TrainPoint_After_InitTrain) :-
        reset_Model_WithTrain,
        nbc_trainPoint([5.1,3.5,1.4], 1).

:- end_tests(nbc_trainPoint).



%%
%% TESTING predicate nbc_variances/2
%%
:- begin_tests(nbc_variances).      

%% Failure Tests
test(nbc_Variances_Before_Train, [error(_,system_error('The Model is not trained!'))]) :-
        reset_Model_NoTrain,
        nbc_variances(VariancesList, _),
        print('\nVariances: '),
        print(VariancesList).


%% Successful Tests

test(nbc_Variances_Normal_Use) :-
        reset_Model_WithTrain,
        nbc_variances(VariancesList, _),
        print('\nVariances: '),
        print(VariancesList).

:- end_tests(nbc_variances).

run_naive_bayes_classifier_tests :-
        run_tests.

