#include <sicstus/sicstus.h>
/* ex_glue.h is generated by splfr from the foreign/[2,3] facts.
   Always include the glue header in your foreign resource code.
*/
#include "nca_glue.h"
#include <mlpack/methods/nca/nca.hpp>
#include <mlpack/core.hpp>

// including helper functions for converting between arma structures and arrays
#include "../../helper_files/helper.hpp"

// some of the most used namespaces
using namespace arma;
using namespace mlpack;
using namespace std;
using namespace mlpack::nca;
using namespace mlpack::metric;


// input: 	const arma::mat & 				dataset,
//			const arma::Row< size_t > & 	labels,
//			MetricType 						metric = MetricType()
//			arma::mat & 					outputMatrix,
// output: 
// description: 
//			Initialize the nca model and the selected optimizer.
//
//			Then perform nca on the given data and return the learned distance.
//
void nca(	char const *optimizerType,
			double stepSize, SP_integer maxIterations, double tolerance, 
			SP_integer shuffle, 
			SP_integer numBasis, double armijoConstant, double wolfe, SP_integer maxLineSearchTrials, double minStep, double maxStep, SP_integer batchSize, 
			float *dataMatArr, SP_integer dataMatSize, SP_integer dataMatRowNum,
			float *rawLabelsArr, SP_integer rawLabelsArrSize,
			float **distanceMatArr, SP_integer *distanceMatColNum, SP_integer *distanceMatRowNum)
{	
	// convert the Prolog array to arma::mat
	mat data = convertArrayToMat(dataMatArr, dataMatSize, dataMatRowNum);
	// check if labels fit the data
    if (data.n_cols != rawLabelsArrSize)
    {
        raisePrologSystemExeption("The number of data points does not match the number of labels!");
        return;
    }

	// convert the Prolog array to arma::rowvec
	Row< size_t > rawLabels = convertArrayToVec(rawLabelsArr, rawLabelsArrSize);

	arma::mat distance;
	distance.eye();

	try
	{
		// copied parts of the nca.main file
		// Now, normalize the labels.
		arma::Col<size_t> mappings;
		arma::Row<size_t> labels;
		data::NormalizeLabels(rawLabels, labels, mappings);

		// Now create the NCA object and run the optimization.
  		if (strcmp(optimizerType,"sgd") == 0)
  		{
  		  	NCA<LMetric<2> > nca(data, labels);
  		  	nca.Optimizer().StepSize() = stepSize;
  		  	nca.Optimizer().MaxIterations() = maxIterations;
  		  	nca.Optimizer().Tolerance() = tolerance;
  		  	nca.Optimizer().Shuffle() = (shuffle == 1);
  		  	nca.Optimizer().BatchSize() = batchSize;

  		  	nca.LearnDistance(distance);
  		}
  		else if (strcmp(optimizerType,"lbfgs") == 0)
  		{
  		  	NCA<LMetric<2>, ens::L_BFGS> nca(data, labels);
  		  	nca.Optimizer().NumBasis() = numBasis;
  		  	nca.Optimizer().MaxIterations() = maxIterations;
  		  	nca.Optimizer().ArmijoConstant() = armijoConstant;
  		  	nca.Optimizer().Wolfe() = wolfe;
  		  	nca.Optimizer().MinGradientNorm() = tolerance;
  		  	nca.Optimizer().MaxLineSearchTrials() = maxLineSearchTrials;
  		  	nca.Optimizer().MinStep() = minStep;
  		  	nca.Optimizer().MaxStep() = maxStep;

  		  	nca.LearnDistance(distance);
  		}
		else
		{
			raisePrologDomainExeption(optimizerType, 1, "The given OptimizerType is unkown!", "nca");
		}
	}
	catch(const std::exception& e)
	{
		raisePrologSystemExeption(e.what());
		return;
	}

	
	// return the Matrix
	returnMatrixInformation(distance, distanceMatArr, distanceMatColNum, distanceMatRowNum);
}