:- module(nca, [nca/17]).

%% requirements of library(struct)
:- load_files(library(str_decl),
        [when(compile_time), if(changed)]).

%% needed for using the array type
:- use_module(library(structs)).
:- use_module('../../helper_files/helper.pl').

%% type definitions for the float array
:- foreign_type
        float32          = float_32,
        float_array      = array(float32).

%% definitions for the connected function


%% --Input--
%%              string  optimizerType           => "sgd", "lbfgs",
%%              float32 stepSize                => 0.01 , used in "sgd",
%%              int     maxIterations           => 500000 , used in "sgd, lbfgs",
%%              float32 tolerance               => 1e-7 , used in "sgd",
%%              bool    shuffle                 => (1)true / (0)false, => 0.01 , used in "sgd",
%%              int     numBasis                => 5 , used in "lbfgs",
%%              float32 armijoConstant          => 0.0001 , used in "lbfgs",
%%              float32 wolfe                   => 0.9 , used in "lbfgs",
%%              int     maxLineSearchTrials     => 50 , used in "lbfgs",
%%              float32 minStep                 => 1e-20 , used in "lbfgs",
%%              float32 maxStep                 => 1e+20 , used in "lbfgs",
%%              int     batchSize               => 50 , used in "sgd",
%%              mat     data,
%%              vec     labels
%%
%% --Output--
%%              mat     distance
%%
%% --Description--
%%              Initialize the nca model and the selected optimizer.
%%
%%              Then perform nca on the given data and return the learned distance.
%%
nca(OptimizerType, StepSize, MaxIterations, Tolerance, Shuffle, NumBasis, ArmijoConstant, Wolfe, MaxLine, MinStep, MaxStep, BatchSize, 
        DataList, DataRows, PredictionList, DistanceList, ZCols) :-
        StepSize > 0,
        MaxIterations >= 0,
        Tolerance >= 0,
        NumBasis > 0,
        ArmijoConstant > 0,
        Wolfe > 0,
        MaxLine > 0,
        MinStep > 0,
        MaxStep > 0,
        MaxStep >= MinStep,
        BatchSize > 0,
        convert_list_to_float_array(DataList, DataRows, array(Xsize, Xrows, X)),
        convert_list_to_float_array(PredictionList, array(Ysize, Y)),
        ncaI(OptimizerType, StepSize, MaxIterations, Tolerance, Shuffle, NumBasis, ArmijoConstant, Wolfe, MaxLine, MinStep, MaxStep, BatchSize, 
                X, Xsize, Xrows, Y, Ysize, Z, ZCols, ZRows),
        convert_float_array_to_2d_list(Z, ZCols, ZRows, DistanceList).

foreign(nca, c, ncaI(   +string,
                        +float32, +integer, +float32,
                        +integer,
                        +integer, +float32, +float32, +integer, +float32, +float32, +integer,
                        +pointer(float_array), +integer, +integer,
                        +pointer(float_array), +integer,
                        -pointer(float_array), -integer, -integer)).


%% Defines the functions that get connected from main.cpp
foreign_resource(nca, [nca]).

:- load_foreign_resource(nca).
