#include <sicstus/sicstus.h>
/* ex_glue.h is generated by splfr from the foreign/[2,3] facts.
   Always include the glue header in your foreign resource code.
*/
#include "nmf_glue.h"
#include <mlpack/methods/amf/amf.hpp>
#include <mlpack/core.hpp>

#include <mlpack/methods/amf/update_rules/nmf_mult_div.hpp>

// including helper functions for converting between arma structures and arrays
#include "../../helper_files/helper.hpp"

// some of the most used namespaces
using namespace arma;
using namespace mlpack;
using namespace std;
using namespace mlpack::amf;


// input:   const TerminationPolicyType & 	terminationPolicy = TerminationPolicyType(),
//          const InitializationRuleType & 	initializeRule = InitializationRuleType(),
//          const UpdateRuleType & 	        update = UpdateRuleType()
//          const MatType & 	            V,
//          const size_t 	                r,
//          arma::mat & 	                W,
//          arma::mat & 	                H
//
// output:  double
//
// description: 
//          Initilizes the nmf model and applies it to the given data.
//
void nmf(char const *updateRule, 
            SP_integer maxIterations, double minResidue,
            float *dataMatArr, SP_integer dataMatSize, SP_integer dataMatRowNum,
            SP_integer rank,
            float **WMatArr, SP_integer *WMatColNum, SP_integer *WMatRowNum,
            float **HMatArr, SP_integer *HMatColNum, SP_integer *HMatRowNum)
{
    // convert the Prolog array to arma::mat
    mat data = convertArrayToMat(dataMatArr, dataMatSize, dataMatRowNum);
    
    // create the ReturnMats
    mat WReturnMat;
    mat HReturnMat;


    try
    {
        // decide which update rule to use
        if (strcmp(updateRule, "multdist") == 0)        
        {
            AMF<SimpleResidueTermination, RandomAcolInitialization<>,NMFMultiplicativeDistanceUpdate>(SimpleResidueTermination(minResidue, maxIterations))
            .Apply(data, rank, WReturnMat, HReturnMat);
        }
        else if (strcmp(updateRule, "multdiv") == 0)
        {
            AMF<SimpleResidueTermination, RandomAcolInitialization<>,NMFMultiplicativeDivergenceUpdate>(SimpleResidueTermination(minResidue, maxIterations))
            .Apply(data, rank, WReturnMat, HReturnMat);
        }
        else if (strcmp(updateRule, "als") == 0)
        {
            AMF<SimpleResidueTermination, RandomAcolInitialization<>,NMFALSUpdate>(SimpleResidueTermination(minResidue, maxIterations))
            .Apply(data, rank, WReturnMat, HReturnMat);
        }
        else
        {
            raisePrologDomainExeption(updateRule, 1, "The given UpdateRule is unkown!", "nmf");
            return;
        }
    }
    catch(const std::exception& e)
    {
        raisePrologSystemExeption(e.what());
        return;
    }
    
    
    // return the Matrices
    returnMatrixInformation(WReturnMat, WMatArr, WMatColNum, WMatRowNum);
    returnMatrixInformation(HReturnMat, HMatArr, HMatColNum, HMatRowNum);
}
