
:- module(nmf, [nmf/10]).

%% requirements of library(struct)
:- load_files(library(str_decl),
        [when(compile_time), if(changed)]).

%% needed for using the array type
:- use_module(library(structs)).
:- use_module('../../helper_files/helper.pl').

%% type definitions for the float array
:- foreign_type
        float32          = float_32,
        float_array      = array(float32).

%% definitions for the connected function


%% --Input--
%%              string  updateRule      => ,
%%              int     maxIterations   => 10000,
%%              float32 minResidue      => 1e-05,
%%              mat     data,
%%              int     rank
%%
%% --Output--
%%              mat     W,
%%              mat     H
%%
%% --Description--
%%              Initilizes the nmf model and applies it to the given data.
%%
nmf(UpdateRule, MaxIterations, MinResidue, DataList, DataRows, Rank, WList, YCols, HList, ZCols) :-
        MaxIterations >= 0,
        MinResidue >= 0,
        Rank > 0,
        convert_list_to_float_array(DataList, DataRows, array(Xsize, Xrows, X)),
        nmfI(UpdateRule, MaxIterations, MinResidue, X, Xsize, Xrows, Rank, Y, YCols, YRows, Z, ZCols, ZRows),
        convert_float_array_to_2d_list(Y, YCols, YRows, WList),
        convert_float_array_to_2d_list(Z, ZCols, ZRows, HList).

foreign(nmf, c, nmfI(   +string, +integer, +float32,
                        +pointer(float_array), +integer, +integer,
                        +integer,
                        -pointer(float_array), -integer, -integer,
                        -pointer(float_array), -integer, -integer)).


%% Defines the functions that get connected from main.cpp
foreign_resource(nmf, [nmf]).

:- load_foreign_resource(nmf).
