
:- module(pca_tests, [run_pca_tests/0]).

:- use_module(library(plunit)).

:- use_module(pca).
:- use_module('../../helper_files/helper.pl').



%%
%% TESTING predicate pca/9
%%
:- begin_tests(pca).      

%% Failure Tests
                                            
test(pca_Wrong_DecompositionPolicy_Input, [error(domain_error('The given DecompositionPolicy is unkown!' , wrongInput), _)]) :-
        pca(0, wrongInput, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _, _, _).
        

%% Successful Tests

test(pca_Normal_Use_Exact) :-
        pca(0, exact, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, TDataList, _, EigValList, EigVecList, _),
        print('\nTransformed Data: '),
        print(TDataList),
        print('\nEigenValues: '),
        print(EigValList),
        print('\nEigenVectors: '),
        print(EigVecList).

test(pca_Normal_Use_Randomized) :-
        pca(0, randomized, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, TDataList, _, EigValList, EigVecList, _),
        print('\nTransformed Data: '),
        print(TDataList),
        print('\nEigenValues: '),
        print(EigValList),
        print('\nEigenVectors: '),
        print(EigVecList).

test(pca_Normal_Use_Randomized_Block_Krylov) :-
        pca(0, randomized_block_krylov, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, TDataList, _, EigValList, EigVecList, _),
        print('\nTransformed Data: '),
        print(TDataList),
        print('\nEigenValues: '),
        print(EigValList),
        print('\nEigenVectors: '),
        print(EigVecList).

test(pca_Normal_Use_Quic) :-
        pca(0, quic, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, TDataList, _, EigValList, EigVecList, _),
        print('\nTransformed Data: '),
        print(TDataList),
        print('\nEigenValues: '),
        print(EigValList),
        print('\nEigenVectors: '),
        print(EigVecList).

test(pca_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        pca(1, exact, Data, 4, TDataList, _, EigValList, EigVecList, _),
        print('\nTransformed Data: '),
        print(TDataList),
        print('\nEigenValues: '),
        print(EigValList),
        print('\nEigenVectors: '),
        print(EigVecList).

:- end_tests(pca).



%%
%% TESTING predicate pcaDimReduction/8
%%
:- begin_tests(pcaDimReduction).      

%% Failure Tests
                                            
test(pcaDimReduction_Wrong_DecompositionPolicy_Input, [error(domain_error('The given DecompositionPolicy is unkown!' , wrongInput), _)]) :-
        pcaDimReduction(0, wrongInput, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, _, _, _).

test(pcaDimReduction_Negative_NewDimension, fail) :-
        pcaDimReduction(0, randomized, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, -2, _, _, _).

test(pcaDimReduction_Too_Big_NewDimension, [error(_, system_error('NewDimension cant be greater than the Dimensionality of the Data!'))]) :-
        pcaDimReduction(0, randomized, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 5, _, _, _).

%% Successful Tests

test(pcaDimReduction_Normal_Use_Exact) :-
        pcaDimReduction(0, exact, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, TDataList, _, RetainedVar),
        print('\nTransformed Data: '),
        print(TDataList),
        print('\nRetained Variance: '),
        print(RetainedVar).

test(pcaDimReduction_Normal_Use_Randomized) :-
        pcaDimReduction(0, randomized, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, TDataList, _, RetainedVar),
        print('\nTransformed Data: '),
        print(TDataList),
        print('\nRetained Variance: '),
        print(RetainedVar).

test(pcaDimReduction_Normal_Use_Randomized_Block_Krylov) :-
        pcaDimReduction(0, randomized_block_krylov, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 2, TDataList, _, RetainedVar),
        print('\nTransformed Data: '),
        print(TDataList),
        print('\nRetained Variance: '),
        print(RetainedVar).

test(pcaDimReduction_Normal_Use_Quic) :-
        pcaDimReduction(0, quic, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5,5.1,3.5,1.4,4.9], 4, 1, TDataList, _, RetainedVar),
        print('\nTransformed Data: '),
        print(TDataList),
        print('\nRetained Variance: '),
        print(RetainedVar).

test(pcaDimReduction_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        pcaDimReduction(1, exact, Data, 4, 2, TDataList, _, RetainedVar),
        print('\nTransformed Data: '),
        print(TDataList),
        print('\nRetained Variance: '),
        print(RetainedVar).

:- end_tests(pcaDimReduction).



%%
%% TESTING predicate pcaVarianceDimReduction/8
%%
:- begin_tests(pcaVarianceDimReduction).      

%% Failure Tests

test(pcaVarianceDimReduction_Wrong_DecompositionPolicy_Input, [error(domain_error('The given DecompositionPolicy is unkown!' , wrongInput), _)]) :-
        pcaVarianceDimReduction(0, wrongInput, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 0.5, _, _, _).

test(pcaVarianceDimReduction_Bad_ToRetainVar_Input, fail) :-
        pcaVarianceDimReduction(0, randomized, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, -1.0, _, _, _),
        pcaVarianceDimReduction(0, randomized, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 1.1, _, _, _).

%% Successful Tests

test(pcaVarianceDimReduction_Normal_Use_Exact) :-
        pcaVarianceDimReduction(0, exact, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 0.5, TDataList, _, RetainedVar),
        print('\nTransformed Data: '),
        print(TDataList),
        print('\nRetained Variance: '),
        print(RetainedVar).

test(pcaVarianceDimReduction_Normal_Use_Randomized) :-
        pcaVarianceDimReduction(0, randomized, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 0.5, TDataList, _, RetainedVar),
        print('\nTransformed Data: '),
        print(TDataList),
        print('\nRetained Variance: '),
        print(RetainedVar).

test(pcaVarianceDimReduction_Normal_Use_Randomized_Block_Krylov) :-
        pcaVarianceDimReduction(0, randomized_block_krylov, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 0.5, TDataList, _, RetainedVar),
        print('\nTransformed Data: '),
        print(TDataList),
        print('\nRetained Variance: '),
        print(RetainedVar).

test(pcaVarianceDimReduction_Normal_Use_Quic) :-
        pcaVarianceDimReduction(0, quic, [5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, 0.2, TDataList, _, RetainedVar),
        print('\nTransformed Data: '),
        print(TDataList),
        print('\nRetained Variance: '),
        print(RetainedVar).

test(pcaVarianceDimReduction_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        pcaVarianceDimReduction(1, exact, Data, 4, 0.2, TDataList, _, RetainedVar),
        print('\nTransformed Data: '),
        print(TDataList),
        print('\nRetained Variance: '),
        print(RetainedVar).

:- end_tests(pcaVarianceDimReduction).

run_pca_tests :-
        run_tests.
