
:- module(perceptron_tests, [run_perceptron_tests/0]).

:- use_module(library(plunit)).

:- use_module(perceptron).
:- use_module('../../helper_files/helper.pl').

reset_Model_NoTrain :-
        perceptron_initModelNoTrain(2, 3, 1000).

reset_Model_WithTrain :-
        perceptron_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 1000).



%%
%% TESTING predicate perceptron_initModelNoTrain/3
%%
:- begin_tests(perceptron_initModelNoTrain).      

%% Failure Tests
                                            
test(perceptron_InitModelNoTrain_Negative_NumClasses, fail) :-
        perceptron_initModelNoTrain(-2, 3, 1000).

test(perceptron_InitModelNoTrain_Negative_Dimensionality, fail) :-
        perceptron_initModelNoTrain(2, -3, 1000).

test(perceptron_InitModelNoTrain_Negative_MaxIterations, fail) :-
        perceptron_initModelNoTrain(2, 3, -1000).
        

%% Successful Tests

test(perceptron_InitModelNoTrain_Normal_Use) :-
        perceptron_initModelNoTrain(2, 3, 1000).

test(perceptron_InitModelNoTrain_Alternative_Input) :-
        perceptron_initModelNoTrain(0, 1, 1000).

:- end_tests(perceptron_initModelNoTrain).



%%
%% TESTING predicate perceptron_initModelWithTrain/5
%%
:- begin_tests(perceptron_initModelWithTrain).      

%% Failure Tests

test(perceptron_InitModelWithTrain_Negative_NumClasses, fail) :-
        perceptron_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], -2, 1000).


test(perceptron_InitModelWithTrain_Negative_MaxIterations, fail) :-
        perceptron_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, -1000).
                                            

test(random_forest_InitModelWithTrainNoWeights_Too_Short_Label, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        perceptron_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], 2, 1000).

test(random_forest_InitModelWithTrainNoWeights_Too_Long_Label, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        perceptron_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], 2, 1000).
        

%% Successful Tests

test(perceptron_InitModelWithTrain_Normal_Use) :-
        perceptron_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 1000).

test(perceptron_InitModelWithTrain_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        perceptron_initModelWithTrain(Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 1000).

:- end_tests(perceptron_initModelWithTrain).



%%
%% TESTING predicate perceptron_biases/1
%%
:- begin_tests(perceptron_biases).      

%% Failure Tests

%% Doesnt cause an Error                            
test(perceptron_Biases_Before_Train) :-
        reset_Model_NoTrain,
        perceptron_biases(Biases),
        print('\nBiases: '),
        print(Biases).
        

%% Successful Tests

test(perceptron_Biases_AfterTrain) :-
        reset_Model_WithTrain,
        perceptron_biases(Biases),
        print('\nBiases: '),
        print(Biases).

:- end_tests(perceptron_biases).



%%
%% TESTING predicate perceptron_classify/3
%%
:- begin_tests(perceptron_classify).      

%% Failure Tests

test(perceptron_Classify_Before_Train_Wrong_Dims, [error(_,system_error('matrix multiplication: incompatible matrix dimensions: 2x3 and 4x1'))]) :-
        reset_Model_NoTrain,
        perceptron_classify([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, _).

test(perceptron_Classify_Different_Dims_Than_Train, [error(_,system_error('matrix multiplication: incompatible matrix dimensions: 2x3 and 4x1'))]) :-
        reset_Model_WithTrain,
        perceptron_classify([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, _).
        

%% Successful Tests

test(perceptron_Classify_Before_Train) :-
        reset_Model_NoTrain,
        perceptron_classify([1.0,2.0,3.0], 3, _).

test(perceptron_Classify_Normal_Use) :-
        reset_Model_WithTrain,
        perceptron_classify([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, PredictList),
        print('\nPredicted Labels: '),
        print(PredictList).

:- end_tests(perceptron_classify).



%%
%% TESTING predicate perceptron_train/5
%%
:- begin_tests(perceptron_train).      

%% Failure Tests
                                            
test(perceptron_Train_Negaitve_NumClasses, fail) :-
        reset_Model_NoTrain,
        perceptron_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], -2, [0.1,0.2,0.3,0.4]).

%% Seems to overide the dimensionality from reset_Model_NoTrain
test(perceptron_Train_Too_Small_Data_Dims) :-
        reset_Model_NoTrain,
        perceptron_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, [0,1,0], 2, [0.1,0.2,0.3]).


test(perceptron_Train_Too_Short_Label, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model_NoTrain,
        perceptron_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], 2, [0.1,0.2,0.3,0.4]).

test(perceptron_Train_Too_Long_Label, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model_NoTrain,
        perceptron_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], 2, [0.1,0.2,0.3,0.4]).


test(perceptron_Train_Too_Short_Weights, [error(_,system_error('The number of data points does not match the number of weights!'))]) :-
        reset_Model_NoTrain,
        perceptron_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, [0.1,0.2]).

test(perceptron_Train_Too_Long_Weights, [error(_,system_error('The number of data points does not match the number of weights!'))]) :-
        reset_Model_NoTrain,
        perceptron_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, [0.1,0.2,0.3,0.4,0.6,0.7]).


%% Successful Tests

test(perceptron_Train_Normal_Use) :-
        reset_Model_NoTrain,
        perceptron_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, [0.1,0.2,0.3,0.4]).


:- end_tests(perceptron_train).



%%
%% TESTING predicate perceptron_weights/2
%%
:- begin_tests(perceptron_weights).      

%% Failure Tests

%% Doesnt cause an error                                          
test(perceptron_Weights_Before_Train) :-
        reset_Model_NoTrain,
        perceptron_weights(Weights, _),
        print('\nWeights: '),
        print(Weights).
        

%% Successful Tests

test(perceptron_Weights_AfterTrain) :-
        reset_Model_WithTrain,
        perceptron_weights(Weights, _),
        print('\nWeights: '),
        print(Weights).

:- end_tests(perceptron_weights).

run_perceptron_tests :-
        run_tests.

