#include <sicstus/sicstus.h>
/* ex_glue.h is generated by splfr from the foreign/[2,3] facts.
   Always include the glue header in your foreign resource code.
*/
#include "radical_glue.h"
#include <mlpack/methods/radical/radical.hpp>
#include <mlpack/core.hpp>

// including helper functions for converting between arma structures and arrays
#include "../../helper_files/helper.hpp"

// some of the most used namespaces
using namespace arma;
using namespace mlpack;
using namespace std;
using namespace mlpack::radical;

// Global Variable of the Radical object so it can be accessed from all functions
Radical radicalObj;


// input:   const double 	noiseStdDev = 0.175,
//          const size_t 	replicates = 30,
//          const size_t 	angles = 150,
//          const size_t 	sweeps = 0,
//          const size_t 	m = 0 
// output: 
// description: 
//          Initilizes the radical model.
//
void initModel(double noiseStdDev, SP_integer replicates, SP_integer angles, SP_integer sweeps, SP_integer m)
{
    try
    {
        radicalObj = Radical(noiseStdDev, replicates, angles, sweeps, m);
    }
    catch(const std::exception& e)
    {
        raisePrologSystemExeption(e.what());
    }
    
}


// input:   const arma::mat & 	matX,
//          arma::mat & 	    matY,
//          arma::mat & 	    matW,
//          util::Timers & 	    timers = IO::GetTimers()
// output: 
// description: 
//          Run RADICAL.
//
void doRadical(float *xMatArr, SP_integer xMatSize, SP_integer xMatRowNum, 
                float **yMatArr, SP_integer *yMatColNum, SP_integer *yMatRowNum, 
                float **wMatArr, SP_integer *wMatColNum, SP_integer *wMatRowNum)
{
    // convert the Prolog array to arma::mat
    mat x = convertArrayToMat(xMatArr, xMatSize, xMatRowNum);

    // create the ReturnMats
    mat yReturnMat;
    mat wReturnMat;

    
    try
    {
        radicalObj.DoRadical(x, yReturnMat, wReturnMat);
    }
    catch(const std::exception& e)
    {
        raisePrologSystemExeption(e.what());
        return;
    }

    
    // return the Matrices
    returnMatrixInformation(yReturnMat, yMatArr, yMatColNum, yMatRowNum);
    returnMatrixInformation(wReturnMat, wMatArr, wMatColNum, wMatRowNum);
}


// input:   const arma::mat & 	matX,
//          util::Timers & 	    timers = IO::GetTimers()
//
// output:  double              output
// description: 
//          Two-dimensional version of RADICAL.
//
double doRadical2D(float *xMatArr, SP_integer xMatSize, SP_integer xMatRowNum)
{
    // convert the Prolog array to arma::mat
    mat x = convertArrayToMat(xMatArr, xMatSize, xMatRowNum);


    try
    {
        return radicalObj.DoRadical2D(x);
    }
    catch(const std::exception& e)
    {
        raisePrologSystemExeption(e.what());
        return 0.0;
    }
}
