
:- module(radical, [    radical_initModel/5,
                        doRadical/6,
                        doRadical2D/3]).

%% requirements of library(struct)
:- load_files(library(str_decl),
        [when(compile_time), if(changed)]).

%% needed for using the array type
:- use_module(library(structs)).
:- use_module('../../helper_files/helper.pl').

%% type definitions for the float array
:- foreign_type
        float32          = float_32,
        float_array      = array(float32).

%% definitions for the connected function


%% --Input--
%%              float32 noiseStdDev     => 0.175,
%%              int     replicates      => 30,
%%              int     angles          => 150,
%%              int     sweeps          => 0,
%%              int     m               => 0
%%
%% --Output--
%%
%% --Description--
%%              Initilizes the radical model.
%%
radical_initModel(NoiseStdDev, Replicates, Angles, Sweeps, M) :-
        NoiseStdDev >= 0,
        Replicates > 0,
        Angles > 0,
        Sweeps >= 0,
        initModelI(NoiseStdDev, Replicates, Angles, Sweeps, M).

foreign(initModel, c, initModelI(+float32, +integer, +integer, +integer, +integer)).


%% --Input--
%%              mat     X
%%
%% --Output--
%%              mat     Y,
%%              mat     W
%%
%% --Description--
%%              Run RADICAL.
%%
doRadical(DataList, DataRows, YList, YCols, WList, ZCols) :-
        convert_list_to_float_array(DataList, DataRows, array(Xsize, Xrows, X)),
        doRadicalI(X, Xsize, Xrows, Y, YCols, YRows, Z, ZCols, ZRows),
        convert_float_array_to_2d_list(Y, YCols, YRows, YList),
        convert_float_array_to_2d_list(Z, ZCols, ZRows, WList).

foreign(doRadical, c, doRadicalI(+pointer(float_array), +integer, +integer, 
                                -pointer(float_array), -integer, -integer, 
                                -pointer(float_array), -integer, -integer)).


%% --Input--
%%              mat     X
%%
%% --Output--
%%              float32 result of 2D radical
%%
%% --Description--
%%              Two-dimensional version of RADICAL.
%%
doRadical2D(DataList, DataRows, Result) :-
        convert_list_to_float_array(DataList, DataRows, array(Xsize, Xrows, X)),
        doRadical2DI(X, Xsize, Xrows, Result).

foreign(doRadical2D, c, doRadical2DI(    +pointer(float_array), +integer, +integer, 
                                        [-float32])).


%% Defines the functions that get connected from main.cpp
foreign_resource(radical, [     initModel,
                                doRadical,
                                doRadical2D]).

:- load_foreign_resource(radical).
