
:- module(radical_tests, [run_radical_tests/0]).

:- use_module(library(plunit)).

:- use_module(radical).
:- use_module('../../helper_files/helper.pl').

reset_Model :-
        radical_initModel(0.175,30,150,0,0).

%%
%% TESTING predicate radical_initModel/5
%%
:- begin_tests(radical_initModel).      

%% Failure Tests

test(radical_InitModel_DoRadical_No_Init) :-
        doRadical([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, Y, _, W, _),
        print('\nY: '),
        print(Y),
        print('\nW: '),
        print(W).

test(radical_InitModel_DoRadical2D_No_Init) :-
        doRadical2D([5.1,3.5,1.4,4.9,3.0,1.4], 3, Result),
        print('\nResult: '),
        print(Result).
                                            
test(radical_InitModel_Negative_NoiseStdDev, fail) :-
        radical_initModel(-0.175,30,150,0,0).

test(radical_InitModel_Negative_Replicates, fail) :-
        radical_initModel(0.175,-30,150,0,0).

test(radical_InitModel_Negative_Angles, fail) :-
        radical_initModel(0.175,30,-150,0,0).

test(radical_InitModel_Negative_Sweeps, fail) :-
        radical_initModel(0.175,30,150,-1,0).


%% Successful Tests

test(radical_InitModel_Normal_Use) :-
        radical_initModel(0.175,30,150,0,0).

:- end_tests(radical_initModel).



%%
%% TESTING predicate doRadical/10
%%
:- begin_tests(doRadical).      

%% Failure Tests
        

%% Successful Tests

test(radical_DoRadical_Normal_Use) :-
        reset_Model,
        doRadical([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, Y, _, W, _),
        print('\nY: '),
        print(Y),
        print('\nW: '),
        print(W).

test(radical_DoRadical_CSV_Input) :-
        reset_Model,
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        doRadical(Data, 4, Y, _, W, _),
        print('\nY: '),
        print(Y),
        print('\nW: '),
        print(W).

:- end_tests(doRadical).



%%
%% TESTING predicate predicate/10
%%
:- begin_tests(predicate).      

%% Failure Tests

test(radical_DoRadical2D_More_Than_Two_Data_Points, [error(_, system_error('matrix multiplication: incompatible matrix dimensions: 90x4 and 2x2'))]) :-
        reset_Model,
        doRadical2D([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, Result),
        print('\nResult: '),
        print(Result).
        

%% Successful Tests

test(radical_DoRadical2D_Normal_Use) :-
        reset_Model,
        doRadical2D([5.1,3.5,1.4,4.9,3.0,1.4], 3, Result),
        print('\nResult: '),
        print(Result).

test(radical_DoRadical2D_CSV_Input) :-
        reset_Model,
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,2, Data),
        doRadical2D(Data, 4, Result),
        print('\nResult: '),
        print(Result).

:- end_tests(predicate).

run_radical_tests :-
        run_tests.

