
:- module(random_forest_tests, [run_random_forest_tests/0]).

:- use_module(library(plunit)).

:- use_module(random_forest).
:- use_module('../../helper_files/helper.pl').

reset_Model_NoTrain :-
        random_forest_initModelNoTrain.

reset_Model_WithTrain :-
        random_forest_initModelWithTrainNoWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 20, 1, 0.0000001, 0).


%%
%% TESTING predicate initModelNoTrainI/0
%%
:- begin_tests(initModelNoTrainI).      

%% Failure Tests
%% Successful Tests

test(random_forest_InitModelNoTrain) :-
        random_forest_initModelNoTrain.

:- end_tests(initModelNoTrainI).



%%
%% TESTING predicate random_forest_initModelWithTrainNoWeights/8
%%
:- begin_tests(random_forest_initModelWithTrainNoWeights).      

%% Failure Tests
                                            
test(random_forest_InitModelWithTrainNoWeights_Negative_NumClasses, fail) :-
        random_forest_initModelWithTrainNoWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], -2, 20, 1, 0.0000001, 0).


test(random_forest_InitModelWithTrainNoWeights_Negative_NumTrees, fail) :-
        random_forest_initModelWithTrainNoWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, -20, 1, 0.0000001, 0).

test(random_forest_InitModelWithTrainNoWeights_Negative_MinLeafSize, fail) :-
        random_forest_initModelWithTrainNoWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 20, -1, 0.0000001, 0).

test(random_forest_InitModelWithTrainNoWeights_Negative_MinGainSplit, fail) :-
        random_forest_initModelWithTrainNoWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 20, 1, -0.0000001, 0).

test(random_forest_InitModelWithTrainNoWeights_Negative_MaxDepth, fail) :-
        random_forest_initModelWithTrainNoWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 20, 1, 0.0000001, -1).


test(random_forest_InitModelWithTrainNoWeights_Too_Short_Label, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        random_forest_initModelWithTrainNoWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], 2, 20, 1, 0.0000001, 0).

test(random_forest_InitModelWithTrainNoWeights_Too_Long_Label, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        random_forest_initModelWithTrainNoWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], 2, 20, 1, 0.0000001, 0).
        

%% Successful Tests

test(random_forest_InitModelWithTrainNoWeights_Normal_Use) :-
        random_forest_initModelWithTrainNoWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 20, 1, 0.0000001, 0).

test(random_forest_InitModelWithTrainNoWeights_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        random_forest_initModelWithTrainNoWeights(Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 20, 1, 0.0000001, 0).

:- end_tests(random_forest_initModelWithTrainNoWeights).



%%
%% TESTING predicate random_forest_initModelWithTrainWithWeights/9
%%
:- begin_tests(random_forest_initModelWithTrainWithWeights).      

%% Failure Tests
                                            
test(random_forest_InitModelWithTrainWithWeights_Negative_NumClasses, fail) :-
        random_forest_initModelWithTrainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], -2, [0.5,0.1,0.7,1.2], 20, 1, 0.0000001, 0).

test(random_forest_InitModelWithTrainWithWeights_Negative_NumTrees, fail) :-
        random_forest_initModelWithTrainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, [0.5,0.1,0.7,1.2], -20, 1, 0.0000001, 0).

test(random_forest_InitModelWithTrainWithWeights_Negative_MinLeafSize, fail) :-
        random_forest_initModelWithTrainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, [0.5,0.1,0.7,1.2], 20, -1, 0.0000001, 0).

test(random_forest_initModelWithTrainWithWeights_Negative_MinGainSplit, fail) :-
        random_forest_initModelWithTrainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, [0.5,0.1,0.7,1.2], 20, 1, -0.0000001, 0).

test(random_forest_InitModelWithTrainWithWeights_Negative_MaxDepth, fail) :-
        random_forest_initModelWithTrainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, [0.5,0.1,0.7,1.2], 20, 1, 0.0000001, -1).


test(random_forest_InitModelWithTrainWithWeights_Too_Short_Label, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        random_forest_initModelWithTrainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], 2, [0.5,0.1,0.7,1.2], 20, 1, 0.0000001, 0).

test(random_forest_InitModelWithTrainWithWeights_Too_Long_Label, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        random_forest_initModelWithTrainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], 2, [0.5,0.1,0.7,1.2], 20, 1, 0.0000001, 0).


test(random_forest_InitModelWithTrainWithWeights_Too_Short_Weights, [error(_,system_error('The number of data points does not match the number of weights!'))]) :-
        random_forest_initModelWithTrainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, [0.5,0.1], 20, 1, 0.0000001, 0).

test(random_forest_InitModelWithTrainWithWeights_Too_Long_Weights, [error(_,system_error('The number of data points does not match the number of weights!'))]) :-
        random_forest_initModelWithTrainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, [0.5,0.1,0.7,1.2,0.43,2.0], 20, 1, 0.0000001, 0).
        

%% Successful Tests

test(random_forest_InitModelWithTrainWithWeights_Normal_Use) :-
        random_forest_initModelWithTrainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, [0.5,0.1,0.7,1.2], 20, 1, 0.0000001, 0).

test(random_forest_InitModelWithTrainWithWeights_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        random_forest_initModelWithTrainWithWeights(Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, [0.5,0.1,0.7,1.2,0.5,3.4,0.9,0.5,0.5,1.2], 20, 1, 0.0000001, 0).

:- end_tests(random_forest_initModelWithTrainWithWeights).



%%
%% TESTING predicate random_forest_classifyPoint/3
%%
:- begin_tests(random_forest_classifyPoint).      

%% Failure Tests
                                            
test(random_forest_ClassifyPoint_Before_Train, [error(_,system_error('RandomForest::Classify(): no random forest trained!'))]) :-
        reset_Model_NoTrain,
        random_forest_classifyPoint([5.1,3.5,1.4], _, _).

%% Point dim seems to not matter
test(random_forest_ClassifyPoint_Smaller_Dim_To_Train) :-
        reset_Model_WithTrain,
        random_forest_classifyPoint([5.1,3.5], Prediction, ProbabilitiesList),
        print('\nPrediction: '),
        print(Prediction),
        print('\nProbabilities: '),
        print(ProbabilitiesList).

%% Point dim seems to not matter
test(random_forest_ClassifyPoint_Larger_Dim_To_Train) :-
        reset_Model_WithTrain,
        random_forest_classifyPoint([5.1,3.5,1.4,4.3,0.4], Prediction, ProbabilitiesList),
        print('\nPrediction: '),
        print(Prediction),
        print('\nProbabilities: '),
        print(ProbabilitiesList).
        

%% Successful Tests

test(random_forest_ClassifyPoint_Normal_Use) :-
        reset_Model_WithTrain,
        random_forest_classifyPoint([5.1,3.5,1.4], Prediction, ProbabilitiesList),
        print('\nPrediction: '),
        print(Prediction),
        print('\nProbabilities: '),
        print(ProbabilitiesList).

:- end_tests(random_forest_classifyPoint).



%%
%% TESTING predicate random_forest_classifyMatrix/5
%%
:- begin_tests(random_forest_classifyMatrix).      

%% Failure Tests
                                            
test(random_forest_ClassifyMatrix_Before_Train, [error(_,system_error('RandomForest::Classify(): no random forest trained!'))]) :-
        reset_Model_NoTrain,
        random_forest_classifyMatrix([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, _, _, _).

%% Classify dim seems to not matter
test(random_forest_ClassifyMatrix_Smaller_Dim_To_Train) :-
        reset_Model_WithTrain,
        random_forest_classifyMatrix([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, Prediction, ProbabilitiesList, _),
        print('\nPrediction: '),
        print(Prediction),
        print('\nProbabilities: '),
        print(ProbabilitiesList).

%% Classify dim seems to not matter
test(random_forest_ClassifyMatrix_Larger_Dim_To_Train) :-
        reset_Model_WithTrain,
        random_forest_classifyMatrix([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 2, Prediction, ProbabilitiesList, _),
        print('\nPrediction: '),
        print(Prediction),
        print('\nProbabilities: '),
        print(ProbabilitiesList).
        

%% Successful Tests

test(random_forest_ClassifyMatrix_Normal_Use) :-
        reset_Model_WithTrain,
        random_forest_classifyMatrix([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, Prediction, ProbabilitiesList, _),
        print('\nPrediction: '),
        print(Prediction),
        print('\nProbabilities: '),
        print(ProbabilitiesList).

:- end_tests(random_forest_classifyMatrix).



%%
%% TESTING predicate random_forest_numTrees/1
%%
:- begin_tests(random_forest_numTrees).      

%% Failure Tests
                       
%% doesnt cause an error                     
test(random_forest_NumTrees_Before_Train) :-
        reset_Model_NoTrain,
        random_forest_numTrees(NumTrees),
        print('\nNumber of Trees: '),
        print(NumTrees).
        

%% Successful Tests

test(random_forest_NumTrees_Normal_Use) :-
        reset_Model_WithTrain,
        random_forest_numTrees(NumTrees),
        print('\nNumber of Trees: '),
        print(NumTrees).

:- end_tests(random_forest_numTrees).



%%
%% TESTING predicate random_forest_trainNoWeights/9
%%
:- begin_tests(random_forest_trainNoWeights).      
                         
%% Failure Tests
                                            
test(random_forest_TrainNoWeights_Negative_NumClasses, fail) :-
        reset_Model_NoTrain,
        random_forest_trainNoWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], -2, 20, 1, 0.0000001, 0, _).

test(random_forest_TrainNoWeights_Negative_NumTrees, fail) :-
        reset_Model_NoTrain,
        random_forest_trainNoWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, -20, 1, 0.0000001, 0, _).

test(random_forest_TrainNoWeights_Negative_MinLeafSize, fail) :-
        reset_Model_NoTrain,
        random_forest_trainNoWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 20, -1, 0.0000001, 0, _).

test(random_forest_TrainNoWeights_Negative_MinGainSplit, fail) :-
        reset_Model_NoTrain,
        random_forest_trainNoWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 20, 1, -0.0000001, 0, _).

test(random_forest_TrainNoWeights_Negative_MaxDepth, fail) :-
        reset_Model_NoTrain,
        random_forest_trainNoWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 20, 1, 0.0000001, -1, _).


test(random_forest_TrainNoWeights_Too_Short_Label, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model_NoTrain,
        random_forest_trainNoWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], 2, 20, 1, 0.0000001, 0, _).

test(random_forest_TrainNoWeights_Too_Long_Label, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model_NoTrain,
        random_forest_trainNoWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], 2, 20, 1, 0.0000001, 0, _).
        

%% Successful Tests

test(random_forest_TrainNoWeights_Normal_Use) :-
        reset_Model_NoTrain,
        random_forest_trainNoWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 20, 1, 0.0000001, 0, Entropy),
        print('\nEntropy: '),
        print(Entropy).

test(random_forest_TrainNoWeights_CSV_Input) :-
        reset_Model_NoTrain,
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        random_forest_trainNoWeights(Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 20, 1, 0.0000001, 0, Entropy),
        print('\nEntropy: '),
        print(Entropy).

:- end_tests(random_forest_trainNoWeights).



%%
%% TESTING predicate random_forest_trainWithWeights/10
%%
:- begin_tests(random_forest_trainWithWeights).      

%% Failure Tests
                                            
test(random_forest_TrainWithWeights_Negative_NumClasses, fail) :-
        reset_Model_NoTrain,
        random_forest_trainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], -2, [0.5,0.1,0.7,1.2], 20, 1, 0.0000001, 0, _).

test(random_forest_TrainWithWeights_Negative_NumTrees, fail) :-
        reset_Model_NoTrain,
        random_forest_trainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, [0.5,0.1,0.7,1.2], -20, 1, 0.0000001, 0, _).

test(random_forest_TrainWithWeights_Negative_MinLeafSize, fail) :-
        reset_Model_NoTrain,
        random_forest_trainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, [0.5,0.1,0.7,1.2], 20, -1, 0.0000001, 0, _).

test(random_forest_TrainWithWeights_Negative_MinGainSplit, fail) :-
        reset_Model_NoTrain,
        random_forest_trainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, [0.5,0.1,0.7,1.2], 20, 1, -0.0000001, 0, _).

test(random_forest_TrainWithWeights_Negative_MaxDepth, fail) :-
        reset_Model_NoTrain,
        random_forest_trainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, [0.5,0.1,0.7,1.2], 20, 1, 0.0000001, -1, _).


test(random_forest_TrainWithWeights_Too_Short_Label, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model_NoTrain,
        random_forest_trainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], 2, [0.5,0.1,0.7,1.2], 20, 1, 0.0000001, 0, _).

test(random_forest_TrainWithWeights_Too_Long_Label, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model_NoTrain,
        random_forest_trainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,1], 2, [0.5,0.1,0.7,1.2], 20, 1, 0.0000001, 0, _).


test(random_forest_TrainWithWeights_Too_Short_Weights, [error(_,system_error('The number of data points does not match the number of weights!'))]) :-
        reset_Model_NoTrain,
        random_forest_trainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, [0.5,0.1], 20, 1, 0.0000001, 0, _).

test(random_forest_TrainWithWeights_Too_Long_Weights, [error(_,system_error('The number of data points does not match the number of weights!'))]) :-
        reset_Model_NoTrain,
        random_forest_trainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, [0.5,0.1,0.7,1.2,0.43,2.0], 20, 1, 0.0000001, 0, _).
        

%% Successful Tests

test(random_forest_TrainWithWeights_Normal_Use) :-
        reset_Model_NoTrain,
        random_forest_trainWithWeights([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, [0.5,0.1,0.7,1.2], 20, 1, 0.0000001, 0, Entropy),
        print('\nEntropy: '),
        print(Entropy).

test(random_forest_TrainWithWeights_CSV_Input) :-
        reset_Model_NoTrain,
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        random_forest_trainWithWeights(Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, [0.5,0.1,0.7,1.2,0.5,3.4,0.9,0.5,0.5,1.2], 20, 1, 0.0000001, 0, Entropy),
        print('\nEntropy: '),
        print(Entropy).

:- end_tests(random_forest_trainWithWeights).

run_random_forest_tests :-
        run_tests.

