
:- module(softmax_regression_tests, [run_softmax_regression_tests/0]).

:- use_module(library(plunit)).

:- use_module(softmax_regression).
:- use_module('../../helper_files/helper.pl').

reset_Model_NoTrain :-
        softmax_regression_initModelNoTrain(3, 2, 0).

reset_Model_WithTrain :-
        softmax_regression_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.0001, 0).

%%
%% TESTING predicate softmax_regression_initModelNoTrain/3
%%
:- begin_tests(softmax_regression_initModelNoTrain).      

%% Failure Tests
                                            
test(softmax_InitNoTrain_Negative_InputSize, fail) :-
        softmax_regression_initModelNoTrain(-1, 0, 0).

test(softmax_InitNoTrain_Negative_InputSize, fail) :-
        softmax_regression_initModelNoTrain(3, -1, 0).
        

%% Successful Tests

test(softmax_InitNoTrain_FitIntercept_False) :-
        softmax_regression_initModelNoTrain(3, 2, 0).

test(softmax_InitNoTrain_FitIntercept_True) :-
        softmax_regression_initModelNoTrain(2, 3, 1).

:- end_tests(softmax_regression_initModelNoTrain).


%%
%% TESTING predicate softmax_regression_initModelWithTrain/6
%%
:- begin_tests(softmax_regression_initModelWithTrain).      

%% Failure Tests
                                            

test(softmax_InitWithTrain_Negative_NumClass, fail) :-
        softmax_regression_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], -1, 0.0001, 0).
        
test(softmax_InitWithTrain_Negative_Lambda, fail) :-
        softmax_regression_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, -0.0001, 0).

test(softmax_InitWithTrain_Wrong_Label_Dims1, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        softmax_regression_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], 2, 0.0001, 0).


test(softmax_InitWithTrain_Wrong_Label_Dims2, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        softmax_regression_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,0,1], 2, 0.0001, 0).
        

%% Successful Tests

test(softmax_InitWithTrain_Direct_Input) :-
        softmax_regression_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, 0.0001, 0).

test(softmax_InitWithTrain_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        softmax_regression_initModelWithTrain(Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 0.003, 1).

:- end_tests(softmax_regression_initModelWithTrain).


%%
%% TESTING predicate softmax_regression_classifyPoint/2
%%
:- begin_tests(softmax_regression_classifyPoint).      

%% Failure Tests
           
%% Doesnt cause an exception                                 
test(softmax_ClassifyPoint_On_Untrained_Model) :-
        reset_Model_NoTrain,
        softmax_regression_classifyPoint([5.1,3.5,1.4], _).

test(softmax_ClassifyPoint_With_Too_Big_Dims, [error(_,system_error('SoftmaxRegression::Classify(): dataset has 5 dimensions, but model has 3 dimensions!'))]) :-
        reset_Model_WithTrain,
        softmax_regression_classifyPoint([5.1,3.5,1.4,5.2,3.2], _).

test(softmax_ClassifyPoint_With_Too_Small_Dims, [error(_,system_error('SoftmaxRegression::Classify(): dataset has 2 dimensions, but model has 3 dimensions!'))]) :-
        reset_Model_WithTrain,
        softmax_regression_classifyPoint([5.1,3.5], _).
        

%% Successful Tests

test(softmax_ClassifyPoint) :-
        reset_Model_WithTrain,
        softmax_regression_classifyPoint([4.1,2.5,1.4], Prediction),
        print('\nPrediction: '),
        print(Prediction).

:- end_tests(softmax_regression_classifyPoint).


%%
%% TESTING predicate softmax_regression_classifyMatrix/5
%%
:- begin_tests(softmax_regression_classifyMatrix).      

%% Failure Tests
                        
%% Doesnt cause an exception                    
test(softmax_ClassifyMatrix_On_Untrained_Model) :-
        reset_Model_NoTrain,
        softmax_regression_classifyMatrix([3, 2, 0, 5, 1, 4, 1, 0, 4, 3, 3, 5, 0, 5, 5], 3, _, _, _).

test(softmax_ClassifyMatrix_With_Too_Big_Dims, [error(_,system_error('SoftmaxRegression::Classify(): dataset has 5 dimensions, but model has 3 dimensions!'))]) :-
        reset_Model_WithTrain,
        softmax_regression_classifyMatrix([3, 2, 0, 5, 1, 4, 1, 0, 4, 3, 3, 5, 0, 5, 5], 5, _, _, _).

test(softmax_ClassifyMatrix_With_Too_Small_Dims, [error(_,system_error('SoftmaxRegression::Classify(): dataset has 2 dimensions, but model has 3 dimensions!'))]) :-
        reset_Model_WithTrain,
        softmax_regression_classifyMatrix([3, 2, 0, 5, 1, 4, 1, 0, 4, 3, 3, 5, 0, 5], 2, _, _, _).
        

%% Successful Tests

test(softmax_ClassifyMatrix_Direct_Input) :-
        reset_Model_WithTrain,
        softmax_regression_classifyMatrix([3, 2, 0, 5, 1, 4, 1, 0, 4, 3, 3, 5, 0, 5, 5], 3, PredicList, ProbsList, _),
        print('\nPredicted Labels: '),
        print(PredicList),
        print('\nProbabilities: '),
        print(ProbsList).

test(softmax_ClassifyMatrix_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        softmax_regression_initModelWithTrain(Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 0.003, 1),
        softmax_regression_classifyMatrix(Data, 4, PredicList, ProbsList, _),
        print('\nPredicted Labels: '),
        print(PredicList),
        print('\nProbabilities: '),
        print(ProbsList).

:- end_tests(softmax_regression_classifyMatrix).


%%
%% TESTING predicate softmax_regression_computeAccuracy/4
%%
:- begin_tests(softmax_regression_computeAccuracy).      

%% Failure Tests

%% Doesnt cause an exception
test(softmax_ComputeAccuracy_On_Untrained_Model) :-
        reset_Model_NoTrain,
        softmax_regression_computeAccuracy([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], _).

test(softmax_ComputeAccuracy_Wrong_Label_Dims1, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model_WithTrain,
        softmax_regression_computeAccuracy([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], _).

%% Doesnt cause exception
test(softmax_ComputeAccuracy_Wrong_Label_Dims2, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model_WithTrain,
        softmax_regression_computeAccuracy([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,0,1], _).

test(softmax_ComputeAccuracy_Wrong_Data_Dims, [error(_,system_error('SoftmaxRegression::Classify(): dataset has 4 dimensions, but model has 3 dimensions!'))]) :-
        reset_Model_WithTrain,
        softmax_regression_computeAccuracy([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, [0,1,0], Accuracy),
        print('\nAccuracy: '),
        print(Accuracy).


%% Successful Tests

test(softmax_ComputeAccuracy_Direct_Input) :-
        reset_Model_WithTrain,
        softmax_regression_computeAccuracy([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], Accuracy),
        print('\nAccuracy: '),
        print(Accuracy).

test(softmax_ComputeAccuracy_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        softmax_regression_initModelWithTrain(Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 0.003, 1),
        softmax_regression_computeAccuracy([3, 2, 0, 5, 1, 4, 1, 0, 4, 3, 3, 5, 0, 5, 5, 2], 4, [0,1,0,1], Accuracy),
        print('\nAccuracy: '),
        print(Accuracy).

:- end_tests(softmax_regression_computeAccuracy).


%%
%% TESTING predicate softmax_regression_featureSize/1
%%
:- begin_tests(softmax_regression_featureSize).      

%% Failure Tests
                                            

        

%% Successful Tests

test(softmax_FeatureSize_No_Train) :-
        reset_Model_NoTrain,
        softmax_regression_featureSize(FeatureSize),
        print('\nFeatureSize: '),
        print(FeatureSize).

test(softmax_FeatureSize_Direct_Input) :-
        reset_Model_WithTrain,
        softmax_regression_featureSize(FeatureSize),
        print('\nFeatureSize: '),
        print(FeatureSize).

test(softmax_FeatureSize_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        softmax_regression_initModelWithTrain(Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 0.003, 1),
        softmax_regression_featureSize(FeatureSize),
        print('\nFeatureSize: '),
        print(FeatureSize).

:- end_tests(softmax_regression_featureSize).


%%
%% TESTING predicate softmax_regression_parameters/2
%%
:- begin_tests(softmax_regression_parameters).      

%% Failure Tests
        

%% Successful Tests

test(softmax_Parameters_No_Train) :-
        reset_Model_NoTrain,
        softmax_regression_parameters(PrametersList, _),
        print('\nParameters: '),
        print(PrametersList).

test(softmax_Parameters_Direct_Input) :-
        reset_Model_WithTrain,
        softmax_regression_parameters(PrametersList, _),
        print('\nParameters: '),
        print(PrametersList).

test(softmax_Parameters_CSV_Input) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        softmax_regression_initModelWithTrain(Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, 0.003, 1),
        softmax_regression_parameters(PrametersList, _),
        print('\nParameters: '),
        print(PrametersList).

:- end_tests(softmax_regression_parameters).


%%
%% TESTING predicate softmax_regression_train/5
%%
:- begin_tests(softmax_regression_train).      

%% Failure Tests
                                            
test(softmax_Train_Negative_NumClass, fail) :-
        reset_Model_NoTrain,
        softmax_regression_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], -1, _).

test(softmax_Train_Wrong_Label_Dims1, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model_NoTrain,
        softmax_regression_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1], 2, _).

%% If the label vector is to long it seems to cause no problems
test(softmax_Train_Wrong_Label_Dims2, [error(_,system_error('The number of data points does not match the number of labels!'))]) :-
        reset_Model_NoTrain,
        softmax_regression_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1,0,0,1], 2, _).

%% doesnt cause a exeption
test(softmax_Train_Wrong_Data_Dims) :-
        reset_Model_NoTrain,
        softmax_regression_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, [0,1,0], 2, _).
   

%% Successful Tests

test(softmax_Train_Direct_Input) :-
        reset_Model_NoTrain,
        softmax_regression_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, [0,1,0,1], 2, FinalValue),
        print('\nFinalValue: '),
        print(FinalValue).

test(softmax_Train_CSV_Input) :-
        softmax_regression_initModelNoTrain(4, 2, 0),
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        softmax_regression_train(Data, 4, [0,1,0,1,1,0,1,1,1,0], 2, FinalValue),
        print('\nFinalValue: '),
        print(FinalValue).

:- end_tests(softmax_regression_train).


run_softmax_regression_tests :-
        run_tests.

