#include <sicstus/sicstus.h>
/* ex_glue.h is generated by splfr from the foreign/[2,3] facts.
   Always include the glue header in your foreign resource code.
*/
#include "sparse_coding_glue.h"
#include <mlpack/methods/sparse_coding/sparse_coding.hpp>
#include <mlpack/core.hpp>

// including helper functions for converting between arma structures and arrays
#include "../../helper_files/helper.hpp"

// some of the most used namespaces
using namespace arma;
using namespace mlpack;
using namespace std;
using namespace mlpack::sparse_coding;

// Global Variable of the SparseCoding object so it can be accessed from all functions
SparseCoding sparseCoding;

// TODO: 
// input:   const arma::mat & 	            data,
//          const size_t 	                atoms,
//          const double 	                lambda1,
//          const double 	                lambda2 = 0,
//          const size_t 	                maxIterations = 0,
//          const double 	                objTolerance = 0.01,
//          const double 	                newtonTolerance = 1e-6,
//          const DictionaryInitializer & 	initializer = DictionaryInitializer()
// output: 
// description: 
void initModelWithTrain(float *dataMatArr, SP_integer dataMatSize, SP_integer dataMatRowNum,
                        SP_integer atoms, double lambda1, double lambda2, SP_integer maxIterations, double objTolerance, double newtonTolerance)
{
    if (dataMatSize/ dataMatRowNum < atoms)
    {
        raisePrologSystemExeption("There have to be more DataPoints than Atoms!");
        return;
    }
    

    // convert the Prolog array to arma::mat
    mat data = convertArrayToMat(dataMatArr, dataMatSize, dataMatRowNum);


    sparseCoding = SparseCoding(data, atoms, lambda1, lambda2, maxIterations, objTolerance, newtonTolerance, RandomInitializer());
}

// TODO: 
// input:   const size_t 	atoms,
//          const double 	lambda1,
//          const double 	lambda2 = 0,
//          const size_t 	maxIterations = 0,
//          const double 	objTolerance = 0.01,
//          const double 	newtonTolerance = 1e-6,
// output: 
// description: 
void initModelNoTrain(SP_integer atoms, double lambda1, double lambda2, SP_integer maxIterations, double objTolerance, double newtonTolerance)
{
    sparseCoding = SparseCoding(atoms, lambda1, lambda2, maxIterations, objTolerance, newtonTolerance);
}

// TODO: 
// input:   const arma::mat & 	data,
//          arma::mat & 	    codes
// output: 
// description: 
void encode(float *dataMatArr, SP_integer dataMatSize, SP_integer dataMatRowNum,
            float **codesMatArr, SP_integer *codesMatColNum, SP_integer *codesMatRowNum)
{
    // convert the Prolog array to arma::mat
    mat data = convertArrayToMat(dataMatArr, dataMatSize, dataMatRowNum);

    // create the ReturnMat
    mat codesReturnMat;


    try
    {
        sparseCoding.Encode(data, codesReturnMat);
    }
    catch(const std::exception& e)
    {
        std::cerr << e.what() << '\n';
        raisePrologSystemExeption("Matrix has not the correct dimensions!");
        return;
    }


    // return the Matrix
    returnMatrixInformation(codesReturnMat, codesMatArr, codesMatColNum, codesMatRowNum);
}

// TODO: 
// input:   const arma::mat & 	            data,
//          const DictionaryInitializer & 	initializer = DictionaryInitializer() 
//
// output:  double
// description: 
double train(float *dataMatArr, SP_integer dataMatSize, SP_integer dataMatRowNum)
{
    if (dataMatSize/ dataMatRowNum < sparseCoding.Atoms())
    {
        raisePrologSystemExeption("There have to be more DataPoints than Atoms!");
        return 0.0;
    }
    // convert the Prolog array to arma::mat
    mat data = convertArrayToMat(dataMatArr, dataMatSize, dataMatRowNum);


    return sparseCoding.Train(data, RandomInitializer());
}
