
:- module(sparse_coding_tests, [run_sparse_coding_tests/0]).

:- use_module(library(plunit)).

:- use_module(sparse_coding).
:- use_module('../../helper_files/helper.pl').


reset_Model_With_Train_A :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,20, Data),
        sparse_coding_initModelWithTrain(Data,4,15,0.1,0.0,100,0.01,0.000001).
reset_Model_No_Train_A :-
        sparse_coding_initModelNoTrain(2,0.1,0.0,100,0.01,0.000001).


%%
%% TESTING predicate sparse_coding_initModelWithTrain/8
%%
:- begin_tests(sparse_coding_initModelWithTrain).      

%% Failure Tests
                                            
test(initModelWithTrain_Negative_RowNum, fail) :-
        sparse_coding_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], -3,15,0.1,0.0,100,0.01,0.000001).

test(initModelWithTrain_Negative_Atoms, fail) :-
        sparse_coding_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3,-15,0.1,0.0,100,0.01,0.000001).

test(initModelWithTrain_Negative_MaxIterations, fail) :-
        sparse_coding_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3,15,0.1,0.0,-100,0.01,0.000001).

test(initModelWithTrain_Negative_ObjTolerance, fail) :-
        sparse_coding_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3,15,0.1,0.0,100,-0.01,0.000001).

test(initModelWithTrain_Negative_NewtonTolerance, fail) :-
        sparse_coding_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3,15,0.1,0.0,100,0.01,-0.000001).

test(initModelWithTrain_Empty_List, fail) :-
        sparse_coding_initModelWithTrain([], 3,15,0.1,0.0,100,0.01,-0.000001).

test(initModelWithTrain_Data_Amount_Smaller_Than_Atoms, [error(_,system_error('There have to be more DataPoints than Atoms!'))]) :-
        sparse_coding_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4,10,0.1,0.0,2,0.01,0.000001).
        

%% Successful Tests

test(initModelWithTrain_Direct_Input_Use) :-
        sparse_coding_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, 3,0.1,0.0,2,0.01,0.000001).

test(initModelWithTrain_Changed_Input) :-
        sparse_coding_initModelWithTrain([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 6,2,2.3,0.1,30,0.002,0.0001).

test(initModelWithTrain_CSV_Use) :-
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,20, Data),
        sparse_coding_initModelWithTrain(Data,4,15,0.1,0.0,100,0.01,0.000001).

:- end_tests(sparse_coding_initModelWithTrain).


%%
%% TESTING predicate sparse_coding_initModelNoTrain/6
%%
:- begin_tests(sparse_coding_initModelNoTrain).      

%% Failure Tests

test(initModelNoTrain_Negative_Atoms, fail) :-
        sparse_coding_initModelNoTrain(-15,0.1,0.0,100,0.01,0.000001).

test(initModelNoTrain_Negative_MaxIterations, fail) :-
        sparse_coding_initModelNoTrain(15,0.1,0.0,-100,0.01,0.000001).

test(initModelNoTrain_Negative_ObjTolerance, fail) :-
        sparse_coding_initModelNoTrain(15,0.1,0.0,100,-0.01,0.000001).

test(initModelNoTrain_Negative_NewtonTolerance, fail) :-
        sparse_coding_initModelNoTrain(15,0.1,0.0,100,0.01,-0.000001).

test(initModelNoTrain_Empty_List, fail) :-
        sparse_coding_initModelNoTrain(15,0.1,0.0,100,0.01,-0.000001).
        

%% Successful Tests

test(initModelNoTrain_Normal_Input_Use) :-
        sparse_coding_initModelNoTrain(15,0.1,0.0,100,0.01,0.000001).

test(initModelNoTrain_Changed_Input) :-
        sparse_coding_initModelNoTrain(5,2.3,0.2,200,0.002,0.0001).

:- end_tests(sparse_coding_initModelNoTrain).


%%
%% TESTING predicate sparse_coding_encode/4
%%
:- begin_tests(sparse_coding_encode).      

%% Failure Tests

test(encode_With_No_Trained_Model, [error(_,system_error('Matrix has not the correct dimensions!'))]) :-
        reset_Model_No_Train_A,
        sparse_coding_encode([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, CodesList, CodesRows),
        print(CodesList),
        print('\n'),
        print(CodesRows).

test(encode_With_Not_Fitting_Data, [error(_,system_error('Matrix has not the correct dimensions!'))]) :-
        reset_Model_No_Train_A,
        sparse_coding_encode([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 2, CodesList, CodesRows),
        print(CodesList),
        print('\n'),
        print(CodesRows).
        

%% Successful Tests

test(encode_Normal_Use1) :-
        reset_Model_With_Train_A,
        sparse_coding_encode([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, CodesList, CodesRows),
        print(CodesList),
        print('\n'),
        print(CodesRows).


test(encode_Normal_Use2) :-
        reset_Model_No_Train_A,
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,10, Data),
        sparse_coding_train(Data, 4, _),
        sparse_coding_encode([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 4, CodesList, CodesRows),
        print(CodesList),
        print('\n'),
        print(CodesRows).

:- end_tests(sparse_coding_encode).


%%
%% TESTING predicate sparse_coding_train/3
%%
:- begin_tests(sparse_coding_train).      

%% Failure Tests
                                            
test(train_With_Negative_RowNum, fail) :-
        reset_Model_No_Train_A,
        sparse_coding_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], -3, _).

test(train_With_Empty_Data_List1, fail) :-
        reset_Model_No_Train_A,
        sparse_coding_train([], 1, _).

test(train_With_Empty_Data_List2, fail) :-
        reset_Model_No_Train_A,
        sparse_coding_train([], 0, _).
        

%% Successful Tests

test(train_With_Direkt_Input) :-
        reset_Model_No_Train_A,
        sparse_coding_train([5.1,3.5,1.4,4.9,3.0,1.4,4.7,3.2,1.3,4.6,3.1,1.5], 3, ObjectiveValue),
        print(ObjectiveValue).

test(train_With_CSV_Input) :-
        reset_Model_No_Train_A,
        open('src/data_csv/iris2.csv', read, File),
        take_csv_row(File, skipFirstRow,20, Data),
        sparse_coding_train(Data, 4, ObjectiveValue),
        print(ObjectiveValue).

:- end_tests(sparse_coding_train).


run_sparse_coding_tests :-
        run_tests.

